@@ -41,6 +41,7 @@
 #include "wee-config.h"
 #include "wee-hook.h"
 #include "wee-log.h"
+#include "wee-network.h"
 #include "wee-util.h"
 #include "wee-list.h"
 #include "wee-proxy.h"
@@ -78,6 +79,7 @@ struct t_config_option *config_startup_display_version;
 struct t_config_option *config_look_align_end_of_lines;
 struct t_config_option *config_look_buffer_notify_default;
 struct t_config_option *config_look_buffer_time_format;
+struct t_config_option *config_look_color_pairs_auto_reset;
 struct t_config_option *config_look_color_real_white;
 struct t_config_option *config_look_command_chars;
 struct t_config_option *config_look_confirm_quit;
@@ -85,7 +87,12 @@ struct t_config_option *config_look_day_change;
 struct t_config_option *config_look_day_change_time_format;
 struct t_config_option *config_look_highlight;
 struct t_config_option *config_look_highlight_regex;
+struct t_config_option *config_look_highlight_tags;
 struct t_config_option *config_look_hline_char;
+struct t_config_option *config_look_hotlist_add_buffer_if_away;
+struct t_config_option *config_look_hotlist_buffer_separator;
+struct t_config_option *config_look_hotlist_count_max;
+struct t_config_option *config_look_hotlist_count_min_msg;
 struct t_config_option *config_look_hotlist_names_count;
 struct t_config_option *config_look_hotlist_names_length;
 struct t_config_option *config_look_hotlist_names_level;
@@ -93,6 +100,7 @@ struct t_config_option *config_look_hotlist_names_merged_buffers;
 struct t_config_option *config_look_hotlist_short_names;
 struct t_config_option *config_look_hotlist_sort;
 struct t_config_option *config_look_hotlist_unique_numbers;
+struct t_config_option *config_look_input_cursor_scroll;
 struct t_config_option *config_look_input_share;
 struct t_config_option *config_look_input_share_overwrite;
 struct t_config_option *config_look_input_undo_max;
@@ -113,6 +121,7 @@ struct t_config_option *config_look_prefix_buffer_align_more;
 struct t_config_option *config_look_prefix_suffix;
 struct t_config_option *config_look_read_marker;
 struct t_config_option *config_look_read_marker_always_show;
+struct t_config_option *config_look_read_marker_string;
 struct t_config_option *config_look_save_config_on_exit;
 struct t_config_option *config_look_save_layout_on_exit;
 struct t_config_option *config_look_scroll_amount;
@@ -157,6 +166,10 @@ struct t_config_option *config_color_status_data_msg;
 struct t_config_option *config_color_status_data_private;
 struct t_config_option *config_color_status_data_highlight;
 struct t_config_option *config_color_status_data_other;
+struct t_config_option *config_color_status_count_msg;
+struct t_config_option *config_color_status_count_private;
+struct t_config_option *config_color_status_count_highlight;
+struct t_config_option *config_color_status_count_other;
 struct t_config_option *config_color_status_more;
 struct t_config_option *config_color_status_time;
 struct t_config_option *config_color_input_text_not_found;
@@ -203,6 +216,8 @@ struct t_config_option *config_plugin_save_config_on_unload;
 struct t_hook *config_day_change_timer = NULL;
 int config_day_change_old_day = -1;
 regex_t *config_highlight_regex = NULL;
+char **config_highlight_tags = NULL;
+int config_num_highlight_tags = 0;
 
 
 /*
@@ -251,7 +266,7 @@ config_change_buffers (void *data, struct t_config_option *option)
     (void) data;
     (void) option;
     
-    gui_window_refresh_windows ();
+    gui_window_ask_refresh (1);
 }
 
 /*
@@ -269,6 +284,20 @@ config_change_buffer_content (void *data, struct t_config_option *option)
         gui_current_window->refresh_needed = 1;
 }
 
+/*
+ * config_change_buffer_notify_default: called when buffer default notify changes
+ */
+
+void
+config_change_buffer_notify_default (void *data, struct t_config_option *option)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) option;
+    
+    gui_buffer_notify_set_all ();
+}
+
 /*
  * config_change_buffer_time_format: called when buffer time format changes
  */
@@ -321,6 +350,32 @@ config_change_highlight_regex (void *data, struct t_config_option *option)
     }
 }
 
+/*
+ * config_change_highlight_tags: called when highlight_tags changes
+ */
+
+void
+config_change_highlight_tags (void *data, struct t_config_option *option)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) option;
+    
+    if (config_highlight_tags)
+    {
+        string_free_split (config_highlight_tags);
+        config_highlight_tags = NULL;
+    }
+    config_num_highlight_tags = 0;
+    
+    if (CONFIG_STRING(config_look_highlight_tags)
+        && CONFIG_STRING(config_look_highlight_tags)[0])
+    {
+        config_highlight_tags = string_split (CONFIG_STRING(config_look_highlight_tags),
+                                              ",", 0, 0, &config_num_highlight_tags);
+    }
+}
+
 /*
  * config_change_hotlist: called when hotlist changes
  */
@@ -384,7 +439,7 @@ config_change_prefix_align_min (void *data, struct t_config_option *option)
         if (ptr_buffer->mixed_lines)
             gui_line_compute_prefix_max_length (ptr_buffer->mixed_lines);
     }
-    gui_window_refresh_windows ();
+    gui_window_ask_refresh (1);
 }
 
 /*
@@ -400,9 +455,8 @@ config_change_color (void *data, struct t_config_option *option)
     
     if (gui_ok)
     {
-        gui_color_init_pairs ();
         gui_color_init_weechat ();
-        gui_window_refresh_windows ();
+        gui_window_ask_refresh (1);
     }
 }
 
@@ -420,6 +474,23 @@ config_change_nick_colors (void *data, struct t_config_option *option)
     gui_color_buffer_display ();
 }
 
+/*
+ * config_change_network_gnutls_ca_file: called when gnutls_ca_file is changed
+ */
+
+void
+config_change_network_gnutls_ca_file (void *data,
+                                      struct t_config_option *option)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) option;
+    
+    if (network_init_ok)
+        network_set_gnutls_ca_file ();
+}
+
+
 /*
  * config_day_change_timer_cb: timer callback for displaying
  *                             "Day changed to xxx" message
@@ -478,6 +549,38 @@ config_day_change_timer_cb (void *data, int remaining_calls)
     return WEECHAT_RC_OK;
 }
 
+/*
+ * config_weechat_init_after_read: initialize some things after reading (or
+ *                                 reloading) configuration file
+ */
+
+void
+config_weechat_init_after_read ()
+{
+    gui_buffer_notify_set_all ();
+    
+    proxy_use_temp_proxies ();
+    
+    gui_bar_use_temp_bars ();
+    if (gui_bars)
+    {
+        /*
+         * at least one bar defined => just ensure that at least one bar is
+         * using item "input_text"
+         */
+        gui_bar_create_default_input ();
+    }
+    else
+    {
+        /* no bar defined => create default bars */
+        gui_bar_create_default ();
+    }
+    
+    /* if no key was found config file, then we use default bindings */
+    if (!gui_keys)
+        gui_keyboard_default_bindings ();
+}
+
 /*
  * config_weechat_reload_cb: reload WeeChat configuration file
  *                           return one of these values:
@@ -516,15 +619,7 @@ config_weechat_reload_cb (void *data, struct t_config_file *config_file)
     rc = config_file_reload (config_file);
     
     if (rc == WEECHAT_CONFIG_READ_OK)
-    {
-        gui_buffer_notify_set_all ();
-        proxy_use_temp_proxies ();
-        gui_bar_use_temp_bars ();
-        gui_bar_create_default ();
-        /* if no key was found config file, then we use default bindings */
-        if (!gui_keys)
-            gui_keyboard_default_bindings ();
-    }
+        config_weechat_init_after_read ();
     
     return rc;
 }
@@ -742,10 +837,7 @@ config_weechat_palette_create_option_cb (void *data,
                     ptr_option = config_file_new_option (
                         config_file, section,
                         option_name, "string",
-                        _("custom color in palette, format is: \"alias;fg,bg\" "
-                          "where alias is color name, fg,bg is \"foreground,background\" "
-                          "(example: \"200,-1\") (everything is optional "
-                          "in this format and order is not important)"),
+                        _("alias for color"),
                         NULL, 0, 0, "", value, 0, NULL, NULL,
                         &config_weechat_palette_change_cb, NULL,
                         NULL, NULL);
@@ -1289,10 +1381,8 @@ config_weechat_filter_write_cb (void *data, struct t_config_file *config_file,
     {
         if (!config_file_write_line (config_file,
                                      ptr_filter->name,
-                                     "%s;%s%s%s;%s;%s",
+                                     "%s;%s;%s;%s",
                                      (ptr_filter->enabled) ? "on" : "off",
-                                     (ptr_filter->plugin_name) ? ptr_filter->plugin_name : "",
-                                     (ptr_filter->plugin_name) ? "." : "",
                                      ptr_filter->buffer_name,
                                      ptr_filter->tags,
                                      ptr_filter->regex))
@@ -1459,7 +1549,7 @@ config_weechat_init_options ()
            "message=messages+highlights, highlight=highlights only, "
            "none=never display in hotlist"),
         "none|highlight|message|all", 0, 0, "all", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, &config_change_buffer_notify_default, NULL, NULL, NULL);
     config_look_buffer_time_format = config_file_new_option (
         weechat_config_file, ptr_section,
         "buffer_time_format", "string",
@@ -1468,6 +1558,14 @@ config_weechat_init_options ()
            "\"${color}\", for example french time: "
            "\"${lightblue}%H${white}%M${lightred}%S\""),
         NULL, 0, 0, "%H:%M:%S", NULL, 0, NULL, NULL, &config_change_buffer_time_format, NULL, NULL, NULL);
+    config_look_color_pairs_auto_reset = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "color_pairs_auto_reset", "integer",
+        N_("automatically reset table of color pairs when number of available "
+           "pairs is lower or equal to this number (-1 = disable automatic "
+           "reset, and then a manual \"/color reset\" is needed when table "
+            "is full)"),
+        NULL, -1, 256, "5", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     config_look_color_real_white = config_file_new_option (
         weechat_config_file, ptr_section,
         "color_real_white", "boolean",
@@ -1514,13 +1612,43 @@ config_weechat_init_options ()
             "(alphanumeric, \"-\", \"_\" or \"|\"), regular expression is case "
            "sensitive, example: \"FlashCode|flashy\""),
         NULL, 0, 0, "", NULL, 0, NULL, NULL, &config_change_highlight_regex, NULL, NULL, NULL);
+    config_look_highlight_tags = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "highlight_tags", "string",
+        N_("comma separated list of tags to highlight (case insensitive "
+           "comparison, examples: \"irc_notice\" for IRC notices, "
+           "\"nick_flashcode\" for messages from nick \"FlashCode\")"),
+        NULL, 0, 0, "", NULL, 0, NULL, NULL, &config_change_highlight_tags, NULL, NULL, NULL);
     config_look_hline_char = config_file_new_option (
         weechat_config_file, ptr_section,
         "hline_char", "string",
-        N_("char used to draw horizontal lines, note that empty value will "
-           "draw a real line with ncurses, but may cause bugs with URL "
-           "selection under some terminals"),
+        N_("char used to draw horizontal separators around bars (empty value "
+           "will draw a real line with ncurses, but may cause bugs with URL "
+           "selection under some terminals), wide chars are NOT allowed here"),
         NULL, 0, 0, "-", NULL, 0, NULL, NULL, &config_change_buffers, NULL, NULL, NULL);
+    config_look_hotlist_add_buffer_if_away = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "hotlist_add_buffer_if_away", "boolean",
+        N_("add any buffer to hotlist (including current or visible buffers) "
+           "if local variable \"away\" is set on buffer"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+    config_look_hotlist_buffer_separator = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "hotlist_buffer_separator", "string",
+        N_("string displayed between buffers in hotlist"),
+        NULL, 0, 0, ", ", NULL, 0, NULL, NULL, &config_change_buffer_content, NULL, NULL, NULL);
+    config_look_hotlist_count_max = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "hotlist_count_max", "integer",
+        N_("max number of messages count to display in hotlist for a buffer "
+           "(0 = never display messages count)"),
+        NULL, 0, GUI_HOTLIST_NUM_PRIORITIES, "2", NULL, 0, NULL, NULL, &config_change_buffer_content, NULL, NULL, NULL);
+    config_look_hotlist_count_min_msg = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "hotlist_count_min_msg", "integer",
+        N_("display messages count if number of messages is greater or equal "
+           "to this value"),
+        NULL, 1, 100, "2", NULL, 0, NULL, NULL, &config_change_buffer_content, NULL, NULL, NULL);
     config_look_hotlist_names_count = config_file_new_option (
         weechat_config_file, ptr_section,
         "hotlist_names_count", "integer",
@@ -1565,6 +1693,12 @@ config_weechat_init_options ()
         N_("keep only unique numbers in hotlist (this applies only on hotlist "
            "items where name is NOT displayed after number)"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL, &config_change_buffer_content, NULL, NULL, NULL);
+    config_look_input_cursor_scroll = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "input_cursor_scroll", "integer",
+        N_("number of chars displayed after end of input line when scrolling "
+           "to display end of line"),
+        NULL, 0, 100, "20", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     config_look_input_share = config_file_new_option (
         weechat_config_file, ptr_section,
         "input_share", "integer",
@@ -1691,13 +1825,19 @@ config_weechat_init_options ()
         weechat_config_file, ptr_section,
         "read_marker", "integer",
         N_("use a marker (line or char) on buffers to show first unread line"),
-        "none|line|dotted-line|char",
-        0, 0, "dotted-line", NULL, 0, NULL, NULL, &config_change_read_marker, NULL, NULL, NULL);
+        "none|line|char",
+        0, 0, "line", NULL, 0, NULL, NULL, &config_change_read_marker, NULL, NULL, NULL);
     config_look_read_marker_always_show = config_file_new_option (
         weechat_config_file, ptr_section,
         "read_marker_always_show", "boolean",
         N_("always show read marker, even if it is after last buffer line"),
         NULL, 0, 0, "off", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+    config_look_read_marker_string = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "read_marker_string", "string",
+        N_("string used to draw read marker line (string is repeated until "
+           "end of line)"),
+        NULL, 0, 0, "- ", NULL, 0, NULL, NULL, &config_change_read_marker, NULL, NULL, NULL);
     config_look_save_config_on_exit = config_file_new_option (
         weechat_config_file, ptr_section,
         "save_config_on_exit", "boolean",
@@ -1879,7 +2019,10 @@ config_weechat_init_options ()
     config_color_chat_nick_colors = config_file_new_option (
         weechat_config_file, ptr_section,
         "chat_nick_colors", "string",
-        N_("text color for nicks (comma separated list of colors)"),
+        /* TRANSLATORS: please do not translate "lightred:blue" */
+        N_("text color for nicks (comma separated list of colors, background "
+           "is allowed with format: \"fg:bg\", for example: "
+           "\"lightred:blue\")"),
         NULL, 0, 0, "cyan,magenta,green,brown,lightblue,default,lightcyan,"
         "lightmagenta,lightgreen,blue", NULL, 0,
         NULL, NULL, &config_change_nick_colors, NULL, NULL, NULL);
@@ -2000,6 +2143,30 @@ config_weechat_init_options ()
            "(status bar)"),
         NULL, -1, 0, "default", NULL, 0,
         NULL, NULL, &config_change_color, NULL, NULL, NULL);
+    config_color_status_count_msg = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "status_count_msg", "color",
+        N_("text color for count of messages in hotlist (status bar)"),
+        NULL, -1, 0, "brown", NULL, 0,
+        NULL, NULL, &config_change_color, NULL, NULL, NULL);
+    config_color_status_count_private = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "status_count_private", "color",
+        N_("text color for count of private messages in hotlist (status bar)"),
+        NULL, -1, 0, "green", NULL, 0,
+        NULL, NULL, &config_change_color, NULL, NULL, NULL);
+    config_color_status_count_highlight = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "status_count_highlight", "color",
+        N_("text color for count of highlight messages in hotlist (status bar)"),
+        NULL, -1, 0, "magenta", NULL, 0,
+        NULL, NULL, &config_change_color, NULL, NULL, NULL);
+    config_color_status_count_other = config_file_new_option (
+        weechat_config_file, ptr_section,
+        "status_count_other", "color",
+        N_("text color for count of other messages in hotlist (status bar)"),
+        NULL, -1, 0, "default", NULL, 0,
+        NULL, NULL, &config_change_color, NULL, NULL, NULL);
     config_color_status_more = config_file_new_option (
         weechat_config_file, ptr_section,
         "status_more", "color",
@@ -2016,7 +2183,7 @@ config_weechat_init_options ()
     config_color_input_text_not_found = config_file_new_option (
         weechat_config_file, ptr_section,
         "input_text_not_found", "color",
-        N_("text color for unsucessful text search in input line"),
+        N_("text color for unsuccessful text search in input line"),
         NULL, -1, 0, "red", NULL, 0,
         NULL, NULL, &config_change_color, NULL, NULL, NULL);
     config_color_input_actions = config_file_new_option (
@@ -2083,7 +2250,7 @@ config_weechat_init_options ()
         weechat_config_file, ptr_section,
         "nick_ignore_chars", "string",
         N_("chars ignored for nick completion"),
-        NULL, 0, 0, "[]-^", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, 0, 0, "[]`_-^", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     config_completion_partial_completion_alert = config_file_new_option (
         weechat_config_file, ptr_section,
         "partial_completion_alert", "boolean",
@@ -2178,7 +2345,8 @@ config_weechat_init_options ()
         "gnutls_ca_file", "string",
         N_("file containing the certificate authorities (\"%h\" will be "
            "replaced by WeeChat home, \"~/.weechat\" by default)"),
-        NULL, 0, 0, "%h/ssl/CAs.pem", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, 0, 0, "%h/ssl/CAs.pem", NULL, 0, NULL, NULL,
+        &config_change_network_gnutls_ca_file, NULL, NULL, NULL);
     config_network_gnutls_handshake_timeout = config_file_new_option (
         weechat_config_file, ptr_section,
         "gnutls_handshake_timeout", "integer",
@@ -2341,6 +2509,8 @@ config_weechat_init ()
     }
     if (!config_highlight_regex)
         config_change_highlight_regex (NULL, NULL);
+    if (!config_highlight_tags)
+        config_change_highlight_tags (NULL, NULL);
     
     return rc;
 }
@@ -2360,14 +2530,7 @@ config_weechat_read ()
     
     rc = config_file_read (weechat_config_file);
     if (rc == WEECHAT_CONFIG_READ_OK)
-    {
-        proxy_use_temp_proxies ();
-        gui_bar_use_temp_bars ();
-        gui_bar_create_default ();
-        /* if no key was found config file, then we use default bindings */
-        if (!gui_keys)
-            gui_keyboard_default_bindings ();
-    }
+        config_weechat_init_after_read ();
     
     if (rc != WEECHAT_CONFIG_READ_OK)
     {
