@@ -81,7 +81,6 @@ enum t_config_look_read_marker
 {
     CONFIG_LOOK_READ_MARKER_NONE = 0,
     CONFIG_LOOK_READ_MARKER_LINE,
-    CONFIG_LOOK_READ_MARKER_DOTTED_LINE,
     CONFIG_LOOK_READ_MARKER_CHAR,
 };
 
@@ -109,12 +108,17 @@ extern struct t_config_option *config_look_buffer_notify_default;
 extern struct t_config_option *config_look_buffer_time_format;
 extern struct t_config_option *config_look_command_chars;
 extern struct t_config_option *config_look_confirm_quit;
+extern struct t_config_option *config_look_color_pairs_auto_reset;
 extern struct t_config_option *config_look_color_real_white;
 extern struct t_config_option *config_look_day_change;
 extern struct t_config_option *config_look_day_change_time_format;
 extern struct t_config_option *config_look_highlight;
 extern struct t_config_option *config_look_highlight_regex;
 extern struct t_config_option *config_look_hline_char;
+extern struct t_config_option *config_look_hotlist_add_buffer_if_away;
+extern struct t_config_option *config_look_hotlist_buffer_separator;
+extern struct t_config_option *config_look_hotlist_count_max;
+extern struct t_config_option *config_look_hotlist_count_min_msg;
 extern struct t_config_option *config_look_hotlist_names_count;
 extern struct t_config_option *config_look_hotlist_names_length;
 extern struct t_config_option *config_look_hotlist_names_level;
@@ -122,6 +126,7 @@ extern struct t_config_option *config_look_hotlist_names_merged_buffers;
 extern struct t_config_option *config_look_hotlist_short_names;
 extern struct t_config_option *config_look_hotlist_sort;
 extern struct t_config_option *config_look_hotlist_unique_numbers;
+extern struct t_config_option *config_look_input_cursor_scroll;
 extern struct t_config_option *config_look_input_share;
 extern struct t_config_option *config_look_input_share_overwrite;
 extern struct t_config_option *config_look_input_undo_max;
@@ -142,6 +147,7 @@ extern struct t_config_option *config_look_prefix_buffer_align_more;
 extern struct t_config_option *config_look_prefix_suffix;
 extern struct t_config_option *config_look_read_marker;
 extern struct t_config_option *config_look_read_marker_always_show;
+extern struct t_config_option *config_look_read_marker_string;
 extern struct t_config_option *config_look_save_config_on_exit;
 extern struct t_config_option *config_look_save_layout_on_exit;
 extern struct t_config_option *config_look_scroll_amount;
@@ -184,6 +190,10 @@ extern struct t_config_option *config_color_status_data_msg;
 extern struct t_config_option *config_color_status_data_private;
 extern struct t_config_option *config_color_status_data_highlight;
 extern struct t_config_option *config_color_status_data_other;
+extern struct t_config_option *config_color_status_count_msg;
+extern struct t_config_option *config_color_status_count_private;
+extern struct t_config_option *config_color_status_count_highlight;
+extern struct t_config_option *config_color_status_count_other;
 extern struct t_config_option *config_color_status_more;
 extern struct t_config_option *config_color_status_time;
 extern struct t_config_option *config_color_input_text_not_found;
@@ -218,6 +228,8 @@ extern struct t_config_option *config_plugin_path;
 extern struct t_config_option *config_plugin_save_config_on_unload;
 
 extern regex_t *config_highlight_regex;
+extern char **config_highlight_tags;
+extern int config_num_highlight_tags;
 
 
 extern struct t_config_option *config_weechat_debug_get (const char *plugin_name);
