@@ -26,6 +26,9 @@
 #endif
 
 #include <stdlib.h>
+#ifdef HAVE_MALLINFO
+#include <malloc.h>
+#endif
 
 #include "weechat.h"
 #include "wee-backtrace.h"
@@ -191,14 +194,15 @@ debug_windows_tree_display (struct t_gui_window_tree *tree, int indent)
             /* leaf */
             snprintf (format,
                       sizeof (format),
-                      "%%-%dsleaf: 0x%%lx (parent:0x%%lx), win=0x%%lx, "
-                      "child1=0x%%lx, child2=0x%%lx, %%d,%%d %%dx%%d, "
-                      "%%d%%%%x%%d%%%%",
+                      "%%-%dsleaf: 0x%%lx, parent:0x%%lx, child1=0x%%lx, "
+                      "child2=0x%%lx, win=0x%%lx (%%d,%%d %%dx%%d"
+                      " %%d%%%%x%%d%%%%)",
                       indent * 2);
             gui_chat_printf (NULL,
                              format,
-                             " ", tree, tree->parent_node, tree->window,
+                             " ", tree, tree->parent_node,
                              tree->child1, tree->child2,
+                             tree->window,
                              tree->window->win_x, tree->window->win_y,
                              tree->window->win_width, tree->window->win_height,
                              tree->window->win_width_pct,
@@ -209,12 +213,13 @@ debug_windows_tree_display (struct t_gui_window_tree *tree, int indent)
             /* node */
             snprintf (format,
                       sizeof (format),
-                      "%%-%dsnode: 0x%%lx (parent:0x%%lx), win=0x%%lx, "
-                      "child1=0x%%lx, child2=0x%%lx)",
+                      "%%-%dsnode: 0x%%lx, parent:0x%%lx, pct:%%d, "
+                      "horizontal:%%d, child1=0x%%lx, child2=0x%%lx",
                       indent * 2);
             gui_chat_printf (NULL,
                              format,
-                             " ", tree, tree->parent_node, tree->window,
+                             " ", tree, tree->parent_node, tree->split_pct,
+                             tree->split_horizontal,
                              tree->child1, tree->child2);
         }
         
@@ -256,6 +261,37 @@ debug_windows_cb (void *data, const char *signal, const char *type_data,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * debug_memory: display information about dynamic memory allocation
+ */
+
+void
+debug_memory ()
+{
+#ifdef HAVE_MALLINFO
+    struct mallinfo info;
+    
+    info = mallinfo ();
+    
+    gui_chat_printf (NULL, "");
+    gui_chat_printf (NULL, _("Memory usage (see \"man mallinfo\" for help):"));
+    gui_chat_printf (NULL, "  arena   :%10d", info.arena);
+    gui_chat_printf (NULL, "  ordblks :%10d", info.ordblks);
+    gui_chat_printf (NULL, "  smblks  :%10d", info.smblks);
+    gui_chat_printf (NULL, "  hblks   :%10d", info.hblks);
+    gui_chat_printf (NULL, "  hblkhd  :%10d", info.hblkhd);
+    gui_chat_printf (NULL, "  usmblks :%10d", info.usmblks);
+    gui_chat_printf (NULL, "  fsmblks :%10d", info.fsmblks);
+    gui_chat_printf (NULL, "  uordblks:%10d", info.uordblks);
+    gui_chat_printf (NULL, "  fordblks:%10d", info.fordblks);
+    gui_chat_printf (NULL, "  keepcost:%10d", info.keepcost);
+#else
+    gui_chat_printf (NULL,
+                     _("Memory usage not available (function \"mallinfo\" not "
+                       "found)"));
+#endif
+}
+
 /*
  * debug_init: hook signals for debug
  */
