@@ -205,7 +205,8 @@ typedef int (t_hook_callback_connect)(void *data, int status,
 typedef int (gnutls_callback_t)(void *data, gnutls_session_t tls_session,
                                 const gnutls_datum_t *req_ca, int nreq,
                                 const gnutls_pk_algorithm_t *pk_algos,
-                                int pk_algos_len, gnutls_retr_st *answer);
+                                int pk_algos_len, gnutls_retr_st *answer,
+                                int action);
 #endif
 
 struct t_hook_connect
@@ -220,6 +221,7 @@ struct t_hook_connect
     gnutls_session_t *gnutls_sess;     /* GnuTLS session (SSL connection)   */
     gnutls_callback_t *gnutls_cb;      /* GnuTLS callback during handshake  */
     int gnutls_dhkey_size;             /* Diffie Hellman Key Exchange size  */
+    char *gnutls_priorities;           /* GnuTLS priorities                 */
 #endif
     char *local_hostname;              /* force local hostname (optional)   */
     int child_read;                    /* to read data in pipe from child   */
@@ -407,10 +409,12 @@ extern struct t_hook *hook_connect (struct t_weechat_plugin *plugin,
                                     int port, int sock, int ipv6,
                                     void *gnutls_session, void *gnutls_cb,
                                     int gnutls_dhkey_size,
+                                    const char *gnutls_priorities,
                                     const char *local_hostname,
                                     t_hook_callback_connect *callback,
                                     void *callback_data);
 #ifdef HAVE_GNUTLS
+extern int hook_connect_gnutls_verify_certificates (gnutls_session_t tls_session);
 extern int hook_connect_gnutls_set_certificates (gnutls_session_t tls_session,
                                                  const gnutls_datum_t *req_ca, int nreq,
                                                  const gnutls_pk_algorithm_t *pk_algos,
@@ -498,7 +502,7 @@ extern void unhook (struct t_hook *hook);
 extern void unhook_all_plugin (struct t_weechat_plugin *plugin);
 extern void unhook_all ();
 extern int hook_add_to_infolist (struct t_infolist *infolist,
-                                 const char *type);
+                                 const char *arguments);
 extern void hook_print_log ();
 
 #endif /* __WEECHAT_HOOK_H */
