@@ -55,32 +55,22 @@
 #include "../plugins/plugin.h"
 
 
-#ifdef HAVE_GNUTLS
 int network_init_ok = 0;
+
+#ifdef HAVE_GNUTLS
 gnutls_certificate_credentials gnutls_xcred; /* GnuTLS client credentials   */
-const int gnutls_cert_type_prio[] = { GNUTLS_CRT_X509, GNUTLS_CRT_OPENPGP, 0 };
-#if LIBGNUTLS_VERSION_NUMBER >= 0x010700
-    const int gnutls_prot_prio[] = { GNUTLS_TLS1_2, GNUTLS_TLS1_1,
-                                     GNUTLS_TLS1_0, GNUTLS_SSL3, 0 };
-#else
-    const int gnutls_prot_prio[] = { GNUTLS_TLS1_1, GNUTLS_TLS1_0,
-                                     GNUTLS_SSL3, 0 };
-#endif
 #endif
 
 
 /*
- * network_init: init network
+ * network_set_gnutls_ca_file: set trust file with option gnutls_ca_file
  */
 
 void
-network_init ()
+network_set_gnutls_ca_file ()
 {
 #ifdef HAVE_GNUTLS
     char *ca_path, *ca_path2;
-
-    gnutls_global_init ();
-    gnutls_certificate_allocate_credentials (&gnutls_xcred);
     
     ca_path = string_expand_home (CONFIG_STRING(config_network_gnutls_ca_file));
     if (ca_path)
@@ -94,15 +84,34 @@ network_init ()
         }
         free (ca_path);
     }
+#endif
+}
+
+/*
+ * network_init: init network
+ */
+
+void
+network_init ()
+{
+#ifdef HAVE_GNUTLS
+    gnutls_global_init ();
+    gnutls_certificate_allocate_credentials (&gnutls_xcred);
+    
+    network_set_gnutls_ca_file ();
+#if LIBGNUTLS_VERSION_NUMBER >= 0x02090a
+    gnutls_certificate_set_verify_function (gnutls_xcred,
+                                            &hook_connect_gnutls_verify_certificates);
+#endif
     gnutls_certificate_client_set_retrieve_function (gnutls_xcred,
                                                      &hook_connect_gnutls_set_certificates);
-    network_init_ok = 1;
 #endif
 #ifdef HAVE_GCRYPT
     gcry_check_version (GCRYPT_VERSION);
     gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
     gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
 #endif
+    network_init_ok = 1;
 }
 
 /*
@@ -774,6 +783,24 @@ network_connect_gnutls_handshake_fd_cb (void *arg_hook_connect, int fd)
     {
         fcntl (HOOK_CONNECT(hook_connect, sock), F_SETFL,
                HOOK_CONNECT(hook_connect, handshake_fd_flags));
+#if LIBGNUTLS_VERSION_NUMBER < 0x02090a
+        /*
+         * gnutls only has the gnutls_certificate_set_verify_function()
+         * function since version 2.9.10. We need to call our verify
+         * function manually after the handshake for old gnutls versions
+         */
+        if (hook_connect_gnutls_verify_certificates (*HOOK_CONNECT(hook_connect, gnutls_sess)) != 0)
+        {
+            (void) (HOOK_CONNECT(hook_connect, callback))
+                (hook_connect->callback_data,
+                 WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR,
+                 rc,
+                 "Error in the certificate.",
+                 HOOK_CONNECT(hook_connect, handshake_ip_address));
+            unhook (hook_connect);
+            return WEECHAT_RC_OK;
+        }
+#endif
         unhook (HOOK_CONNECT(hook_connect, handshake_hook_fd));
         (void) (HOOK_CONNECT(hook_connect, callback))
                 (hook_connect->callback_data, WEECHAT_HOOK_CONNECT_OK, 0, NULL,
@@ -927,6 +954,26 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
                 }
                 fcntl (HOOK_CONNECT(hook_connect, sock), F_SETFL,
                        HOOK_CONNECT(hook_connect, handshake_fd_flags));
+#if LIBGNUTLS_VERSION_NUMBER < 0x02090a
+                /*
+                 * gnutls only has the gnutls_certificate_set_verify_function()
+                 * function since version 2.9.10. We need to call our verify
+                 * function manually after the handshake for old gnutls versions
+                 */
+                if (hook_connect_gnutls_verify_certificates (*HOOK_CONNECT(hook_connect, gnutls_sess)) != 0)
+                {
+                    (void) (HOOK_CONNECT(hook_connect, callback))
+                        (hook_connect->callback_data,
+                         WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR,
+                         rc,
+                         "Error in the certificate.",
+                         ip_address);
+                    unhook (hook_connect);
+                    if (ip_address)
+                        free (ip_address);
+                    return WEECHAT_RC_OK;
+                }
+#endif
             }
 #endif
         }
@@ -949,6 +996,10 @@ void
 network_connect_with_fork (struct t_hook *hook_connect)
 {
     int child_pipe[2];
+#ifdef HAVE_GNUTLS
+    int rc;
+    const char *pos_error;
+#endif
 #ifndef __CYGWIN__
     pid_t pid;
 #endif
@@ -957,20 +1008,27 @@ network_connect_with_fork (struct t_hook *hook_connect)
     /* initialize GnuTLS if SSL asked */
     if (HOOK_CONNECT(hook_connect, gnutls_sess))
     {
-        if (gnutls_init (HOOK_CONNECT(hook_connect, gnutls_sess), GNUTLS_CLIENT) != 0)
+        if (gnutls_init (HOOK_CONNECT(hook_connect, gnutls_sess), GNUTLS_CLIENT) != GNUTLS_E_SUCCESS)
         {
             (void) (HOOK_CONNECT(hook_connect, callback))
                 (hook_connect->callback_data,
-                 '0' + WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR,
+                 WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR,
                  0, NULL, NULL);
             unhook (hook_connect);
             return;
         }
-        gnutls_set_default_priority (*HOOK_CONNECT(hook_connect, gnutls_sess));
-        gnutls_certificate_type_set_priority (*HOOK_CONNECT(hook_connect, gnutls_sess),
-                                              gnutls_cert_type_prio);
-        gnutls_protocol_set_priority (*HOOK_CONNECT(hook_connect, gnutls_sess),
-                                      gnutls_prot_prio);
+        rc = gnutls_priority_set_direct (*HOOK_CONNECT(hook_connect, gnutls_sess),
+                                         HOOK_CONNECT(hook_connect, gnutls_priorities),
+                                         &pos_error);
+        if (rc != GNUTLS_E_SUCCESS)
+        {
+            (void) (HOOK_CONNECT(hook_connect, callback))
+                (hook_connect->callback_data,
+                 WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR,
+                 0, _("invalid priorities"), NULL);
+            unhook (hook_connect);
+            return;
+        }
         gnutls_credentials_set (*HOOK_CONNECT(hook_connect, gnutls_sess),
                                 GNUTLS_CRD_CERTIFICATE,
                                 gnutls_xcred);
@@ -984,7 +1042,7 @@ network_connect_with_fork (struct t_hook *hook_connect)
     {
         (void) (HOOK_CONNECT(hook_connect, callback))
             (hook_connect->callback_data,
-             '0' + WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
+             WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
              0, NULL, NULL);
         unhook (hook_connect);
         return;
@@ -1007,7 +1065,7 @@ network_connect_with_fork (struct t_hook *hook_connect)
         case -1:
             (void) (HOOK_CONNECT(hook_connect, callback))
                 (hook_connect->callback_data,
-                 '0' + WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
+                 WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
                  0, NULL, NULL);
             unhook (hook_connect);
             return;
