@@ -27,7 +27,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 #include <limits.h>
 
 #include "../../core/weechat.h"
@@ -155,10 +154,8 @@ gui_bar_window_print_string (struct t_gui_bar_window *bar_window,
                              int reset_color_before_display,
                              int hide_chars_if_scrolling)
 {
-    int weechat_color, x_with_hidden, size_on_screen, fg, bg, low_char, hidden;
-    int pair;
-    char str_fg[3], str_bg[3], str_pair[6], utf_char[16], *next_char, *output;
-    char *error;
+    int x_with_hidden, size_on_screen, low_char, hidden;
+    char utf_char[16], *next_char, *output;
     
     if (!string || !string[0])
         return 1;
@@ -185,83 +182,45 @@ gui_bar_window_print_string (struct t_gui_bar_window *bar_window,
                 switch (string[0])
                 {
                     case GUI_COLOR_FG_CHAR: /* fg color */
-                        if (string[1] && string[2])
-                        {
-                            str_fg[0] = string[1];
-                            str_fg[1] = string[2];
-                            str_fg[2] = '\0';
-                            sscanf (str_fg, "%d", &fg);
-                            gui_window_set_custom_color_fg (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                            fg);
-                            string += 3;
-                        }
+                        string++;
+                        gui_window_string_apply_color_fg ((unsigned char **)&string,
+                                                          GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar);
                         break;
                     case GUI_COLOR_BG_CHAR: /* bg color */
-                        if (string[1] && string[2])
-                        {
-                            str_bg[0] = string[1];
-                            str_bg[1] = string[2];
-                            str_bg[2] = '\0';
-                            sscanf (str_bg, "%d", &bg);
-                            gui_window_set_custom_color_bg (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                            bg);
-                            string += 3;
-                        }
+                        string++;
+                        gui_window_string_apply_color_bg ((unsigned char **)&string,
+                                                          GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar);
                         break;
                     case GUI_COLOR_FG_BG_CHAR: /* fg + bg color */
-                        if (string[1] && string[2] && (string[3] == ',')
-                            && string[4] && string[5])
-                        {
-                            str_fg[0] = string[1];
-                            str_fg[1] = string[2];
-                            str_fg[2] = '\0';
-                            str_bg[0] = string[4];
-                            str_bg[1] = string[5];
-                            str_bg[2] = '\0';
-                            sscanf (str_fg, "%d", &fg);
-                            sscanf (str_bg, "%d", &bg);
-                            gui_window_set_custom_color_fg_bg (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                               fg, bg);
-                            string += 6;
-                        }
+                        string++;
+                        gui_window_string_apply_color_fg_bg ((unsigned char **)&string,
+                                                             GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar);
                         break;
-                    case GUI_COLOR_PAIR_CHAR: /* pair number */
-                        if ((isdigit (string[1])) && (isdigit (string[2]))
-                            && (isdigit (string[3])) && (isdigit (string[4]))
-                            && (isdigit (string[5])))
-                        {
-                            memcpy (str_pair, string + 1, 5);
-                            str_pair[5] = '\0';
-                            error = NULL;
-                            pair = (int)strtol (str_pair, &error, 10);
-                            if (error && !error[0])
-                            {
-                                gui_window_set_custom_color_pair (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                                  pair);
-                            }
-                            string += 6;
-                        }
+                    case GUI_COLOR_EXTENDED_CHAR: /* pair number */
+                        string++;
+                        gui_window_string_apply_color_pair ((unsigned char **)&string,
+                                                            GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar);
                         break;
                     case GUI_COLOR_BAR_CHAR: /* bar color */
                         switch (string[1])
                         {
                             case GUI_COLOR_BAR_FG_CHAR:
                                 /* bar foreground */
+                                string += 2;
                                 gui_window_set_custom_color_fg (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
                                                                 CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_FG]));
-                                string += 2;
                                 break;
                             case GUI_COLOR_BAR_DELIM_CHAR:
                                 /* bar delimiter */
+                                string += 2;
                                 gui_window_set_custom_color_fg (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
                                                                 CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_DELIM]));
-                                string += 2;
                                 break;
                             case GUI_COLOR_BAR_BG_CHAR:
                                 /* bar background */
+                                string += 2;
                                 gui_window_set_custom_color_bg (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
                                                                 CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_BG]));
-                                string += 2;
                                 break;
                             case GUI_COLOR_BAR_START_INPUT_CHAR:
                                 string += 2;
@@ -273,12 +232,12 @@ gui_bar_window_print_string (struct t_gui_bar_window *bar_window,
                                 break;
                             case GUI_COLOR_BAR_MOVE_CURSOR_CHAR:
                                 /* move cursor to current position on screen */
+                                string += 2;
                                 getyx (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
                                        bar_window->cursor_y,
                                        bar_window->cursor_x);
                                 bar_window->cursor_x += bar_window->x;
                                 bar_window->cursor_y += bar_window->y;
-                                string += 2;
                                 break;
                             default:
                                 string++;
@@ -286,77 +245,28 @@ gui_bar_window_print_string (struct t_gui_bar_window *bar_window,
                         }
                         break;
                     default:
-                        if (isdigit (string[0]) && isdigit (string[1]))
-                        {
-                            str_fg[0] = string[0];
-                            str_fg[1] = string[1];
-                            str_fg[2] = '\0';
-                            sscanf (str_fg, "%d", &weechat_color);
-                            gui_window_set_weechat_color (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                          weechat_color);
-                            string += 2;
-                        }
+                        gui_window_string_apply_color_weechat ((unsigned char **)&string,
+                                                               GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar);
                         break;
                 }
                 break;
+            case GUI_COLOR_SET_ATTR_CHAR:
+                string++;
+                gui_window_string_apply_color_set_attr ((unsigned char **)&string,
+                                                        GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar);
+                break;
+            case GUI_COLOR_REMOVE_ATTR_CHAR:
+                string++;
+                gui_window_string_apply_color_remove_attr ((unsigned char **)&string,
+                                                           GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar);
+                break;
             case GUI_COLOR_RESET_CHAR:
-                gui_window_set_custom_color_fg (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_FG]));
-                gui_window_set_custom_color_bg (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_BG]));
+                string++;
                 gui_window_remove_color_style (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
                                                A_BOLD | A_UNDERLINE | A_REVERSE);
-                string++;
-                break;
-            case GUI_COLOR_SET_WEECHAT_CHAR:
-                string++;
-                switch (string[0])
-                {
-                    case GUI_COLOR_ATTR_BOLD_CHAR:
-                        string++;
-                        gui_window_set_color_style (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                    A_BOLD);
-                        break;
-                    case GUI_COLOR_ATTR_REVERSE_CHAR:
-                        string++;
-                        gui_window_set_color_style (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                    A_REVERSE);
-                        break;
-                    case GUI_COLOR_ATTR_ITALIC_CHAR:
-                        /* not available in Curses GUI */
-                        string++;
-                        break;
-                    case GUI_COLOR_ATTR_UNDERLINE_CHAR:
-                        string++;
-                        gui_window_set_color_style (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                    A_UNDERLINE);
-                        break;
-                }
-                break;
-            case GUI_COLOR_REMOVE_WEECHAT_CHAR:
-                string++;
-                switch (string[0])
-                {
-                    case GUI_COLOR_ATTR_BOLD_CHAR:
-                        string++;
-                        gui_window_remove_color_style (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                       A_BOLD);
-                        break;
-                    case GUI_COLOR_ATTR_REVERSE_CHAR:
-                        string++;
-                        gui_window_remove_color_style (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                       A_REVERSE);
-                        break;
-                    case GUI_COLOR_ATTR_ITALIC_CHAR:
-                        /* not available in Curses GUI */
-                        string++;
-                        break;
-                    case GUI_COLOR_ATTR_UNDERLINE_CHAR:
-                        string++;
-                        gui_window_remove_color_style (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
-                                                       A_UNDERLINE);
-                        break;
-                }
+                gui_window_set_custom_color_fg_bg (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
+                                                   CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_FG]),
+                                                   CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_BG]));
                 break;
             default:
                 next_char = utf8_next_char (string);
@@ -442,7 +352,7 @@ gui_bar_window_draw (struct t_gui_bar_window *bar_window,
     int chars_available, index, size;
     int length_screen_before_cursor, length_screen_after_cursor;
     int diff, max_length, optimal_number_of_lines;
-    int some_data_not_displayed;
+    int some_data_not_displayed, hline_char;
     
     if (!gui_init_ok)
         return;
@@ -485,6 +395,7 @@ gui_bar_window_draw (struct t_gui_bar_window *bar_window,
             if (CONFIG_INTEGER(bar_window->bar->options[GUI_BAR_OPTION_SIZE]) == 0)
                 gui_bar_window_set_current_size (bar_window, window, 1);
             gui_window_clear (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
+                              CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_FG]),
                               CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_BG]));
         }
         else
@@ -539,6 +450,7 @@ gui_bar_window_draw (struct t_gui_bar_window *bar_window,
             }
             
             gui_window_clear (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
+                              CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_FG]),
                               CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_BG]));
             x = 0;
             y = 0;
@@ -585,13 +497,12 @@ gui_bar_window_draw (struct t_gui_bar_window *bar_window,
                         diff = length_screen_before_cursor - chars_available;
                         if (diff > 0)
                         {
-                            /*
-                             * FIXME: keep some spaces after end of input
-                             * for example by adding something to diff, like:
-                             * diff += (9 - (diff % 10));
-                             * but then scroll position has to be saved
-                             *  (in bar window ?)
-                             */
+                            if (CONFIG_INTEGER(config_look_input_cursor_scroll) > 0)
+                            {
+                                diff += (CONFIG_INTEGER(config_look_input_cursor_scroll)
+                                         - 1
+                                         - (diff % CONFIG_INTEGER(config_look_input_cursor_scroll)));
+                            }
                             
                             /* compute new start for displaying input */
                             new_start_input = pos_after_start_input + gui_chat_string_real_pos (pos_after_start_input, diff);
@@ -652,6 +563,11 @@ gui_bar_window_draw (struct t_gui_bar_window *bar_window,
                                                            A_BOLD | A_UNDERLINE | A_REVERSE);
                             wclrtobot (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar);
                         }
+                        else
+                        {
+                            gui_window_remove_color_style (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
+                                                           A_BOLD | A_UNDERLINE | A_REVERSE);
+                        }
                         while (x < bar_window->width)
                         {
                             gui_bar_window_print_string (bar_window, filling,
@@ -699,6 +615,7 @@ gui_bar_window_draw (struct t_gui_bar_window *bar_window,
         if (CONFIG_INTEGER(bar_window->bar->options[GUI_BAR_OPTION_SIZE]) == 0)
             gui_bar_window_set_current_size (bar_window, window, 1);
         gui_window_clear (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_bar,
+                          CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_FG]),
                           CONFIG_COLOR(bar_window->bar->options[GUI_BAR_OPTION_COLOR_BG]));
     }
 
@@ -722,21 +639,28 @@ gui_bar_window_draw (struct t_gui_bar_window *bar_window,
     
     if (CONFIG_INTEGER(bar_window->bar->options[GUI_BAR_OPTION_SEPARATOR]))
     {
+        if (CONFIG_STRING(config_look_hline_char)
+            && CONFIG_STRING(config_look_hline_char)[0])
+        {
+            hline_char = utf8_char_int (CONFIG_STRING(config_look_hline_char));
+            if (hline_char > 127)
+                hline_char = ACS_HLINE;
+        }
+        else
+            hline_char = ACS_HLINE;
         switch (CONFIG_INTEGER(bar_window->bar->options[GUI_BAR_OPTION_POSITION]))
         {
             case GUI_BAR_POSITION_BOTTOM:
                 gui_window_set_weechat_color (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_separator,
                                               GUI_COLOR_SEPARATOR);
                 mvwhline (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_separator,
-                          0, 0, gui_window_get_hline_char (),
-                          bar_window->width);
+                          0, 0, hline_char, bar_window->width);
                 break;
             case GUI_BAR_POSITION_TOP:
                 gui_window_set_weechat_color (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_separator,
                                               GUI_COLOR_SEPARATOR);
                 mvwhline (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_separator,
-                          0, 0, gui_window_get_hline_char (),
-                          bar_window->width);
+                          0, 0, hline_char, bar_window->width);
                 break;
             case GUI_BAR_POSITION_LEFT:
                 gui_window_set_weechat_color (GUI_BAR_WINDOW_OBJECTS(bar_window)->win_separator,
