@@ -27,7 +27,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
 #include "../../core/weechat.h"
 #include "../../core/wee-config.h"
@@ -70,8 +69,7 @@ gui_chat_marker_for_line (struct t_gui_buffer *buffer, struct t_gui_line *line)
     struct t_gui_line *last_read_line;
     
     /* marker is disabled in config? */
-    if ((CONFIG_INTEGER(config_look_read_marker) != CONFIG_LOOK_READ_MARKER_LINE)
-        && (CONFIG_INTEGER(config_look_read_marker) != CONFIG_LOOK_READ_MARKER_DOTTED_LINE))
+    if (CONFIG_INTEGER(config_look_read_marker) != CONFIG_LOOK_READ_MARKER_LINE)
         return 0;
     
     /* marker is not set for buffer? */
@@ -134,36 +132,30 @@ gui_chat_display_new_line (struct t_gui_window *window, int num_lines, int count
 void
 gui_chat_display_horizontal_line (struct t_gui_window *window, int simulate)
 {
-    int i, n, hline_char;
-    
-    hline_char = gui_window_get_hline_char ();
+    int x, size_on_screen;
+    char *read_marker_string, *default_string = "- ";
     
     if (!simulate)
     {
-        gui_window_set_weechat_color (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                      GUI_COLOR_CHAT_READ_MARKER);
-        switch (CONFIG_INTEGER(config_look_read_marker))
+        if (CONFIG_INTEGER(config_look_read_marker) == CONFIG_LOOK_READ_MARKER_LINE)
         {
-            case CONFIG_LOOK_READ_MARKER_LINE:
-                mvwhline (GUI_WINDOW_OBJECTS(window)->win_chat,
-                          window->win_chat_cursor_y, window->win_chat_cursor_x,
-                          hline_char, window->win_chat_width - 1);
-                break;
-            case CONFIG_LOOK_READ_MARKER_DOTTED_LINE:
-                wmove (GUI_WINDOW_OBJECTS(window)->win_chat,
-                       window->win_chat_cursor_y, window->win_chat_cursor_x);
-                wclrtoeol (GUI_WINDOW_OBJECTS(window)->win_chat);
-                n = (window->win_chat_width % 2);
-                for (i = 0; i < window->win_chat_width - 1; i++)
-                {
-                    if (i % 2 == n)
-                        mvwhline (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                  window->win_chat_cursor_y, i,
-                                  hline_char, 1);
-                }
-                break;
-            default:
-                break;
+            read_marker_string = CONFIG_STRING(config_look_read_marker_string);
+            if (!read_marker_string || !read_marker_string[0])
+                read_marker_string = default_string;
+            size_on_screen = utf8_strlen_screen (read_marker_string);
+            gui_window_set_weechat_color (GUI_WINDOW_OBJECTS(window)->win_chat,
+                                          GUI_COLOR_CHAT_READ_MARKER);
+            wmove (GUI_WINDOW_OBJECTS(window)->win_chat,
+                   window->win_chat_cursor_y, window->win_chat_cursor_x);
+            wclrtoeol (GUI_WINDOW_OBJECTS(window)->win_chat);
+            x = 0;
+            while (x < window->win_chat_width - 1)
+            {
+                mvwprintw (GUI_WINDOW_OBJECTS(window)->win_chat,
+                           window->win_chat_cursor_y, x,
+                           "%s", read_marker_string);
+                x += size_on_screen;
+            }
         }
         window->win_chat_cursor_x = window->win_chat_width;
     }
@@ -179,94 +171,37 @@ char *
 gui_chat_string_next_char (struct t_gui_window *window,
                            const unsigned char *string, int apply_style)
 {
-    char str_fg[3], str_bg[3], str_pair[6], *error;
-    int weechat_color, fg, bg, pair;
-    
     while (string[0])
     {
         switch (string[0])
         {
-            case GUI_COLOR_RESET_CHAR:
-                string++;
-                if (apply_style)
-                    gui_window_reset_style (GUI_WINDOW_OBJECTS(window)->win_chat, GUI_COLOR_CHAT);
-                break;
             case GUI_COLOR_COLOR_CHAR:
                 string++;
                 switch (string[0])
                 {
                     case GUI_COLOR_FG_CHAR: /* fg color */
-                        if (string[1] && string[2])
-                        {
-                            if (apply_style)
-                            {
-                                str_fg[0] = string[1];
-                                str_fg[1] = string[2];
-                                str_fg[2] = '\0';
-                                sscanf (str_fg, "%d", &fg);
-                                gui_window_set_custom_color_fg (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                                fg);
-                            }
-                            string += 3;
-                        }
+                        string++;
+                        gui_window_string_apply_color_fg ((unsigned char **)&string,
+                                                          (apply_style) ? GUI_WINDOW_OBJECTS(window)->win_chat : NULL);
                         break;
                     case GUI_COLOR_BG_CHAR: /* bg color */
-                        if (string[1] && string[2])
-                        {
-                            if (apply_style)
-                            {
-                                str_bg[0] = string[1];
-                                str_bg[1] = string[2];
-                                str_bg[2] = '\0';
-                                sscanf (str_bg, "%d", &bg);
-                                gui_window_set_custom_color_bg (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                                bg);
-                            }
-                            string += 3;
-                        }
+                        string++;
+                        gui_window_string_apply_color_bg ((unsigned char **)&string,
+                                                          (apply_style) ? GUI_WINDOW_OBJECTS(window)->win_chat : NULL);
                         break;
                     case GUI_COLOR_FG_BG_CHAR: /* fg + bg color */
-                        if (string[1] && string[2] && (string[3] == ',')
-                            && string[4] && string[5])
-                        {
-                            if (apply_style)
-                            {
-                                str_fg[0] = string[1];
-                                str_fg[1] = string[2];
-                                str_fg[2] = '\0';
-                                str_bg[0] = string[4];
-                                str_bg[1] = string[5];
-                                str_bg[2] = '\0';
-                                sscanf (str_fg, "%d", &fg);
-                                sscanf (str_bg, "%d", &bg);
-                                gui_window_set_custom_color_fg_bg (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                                   fg, bg);
-                            }
-                            string += 6;
-                        }
+                        string++;
+                        gui_window_string_apply_color_fg_bg ((unsigned char **)&string,
+                                                             (apply_style) ? GUI_WINDOW_OBJECTS(window)->win_chat : NULL);
                         break;
-                    case GUI_COLOR_PAIR_CHAR: /* pair number */
-                        if ((isdigit (string[1])) && (isdigit (string[2]))
-                            && (isdigit (string[3])) && (isdigit (string[4]))
-                            && (isdigit (string[5])))
-                        {
-                            if (apply_style)
-                            {
-                                memcpy (str_pair, string + 1, 5);
-                                str_pair[5] = '\0';
-                                error = NULL;
-                                pair = (int)strtol (str_pair, &error, 10);
-                                if (error && !error[0])
-                                {
-                                    gui_window_set_custom_color_pair (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                                      pair);
-                                }
-                            }
-                            string += 6;
-                        }
+                    case GUI_COLOR_EXTENDED_CHAR: /* pair number */
+                        string++;
+                        gui_window_string_apply_color_pair ((unsigned char **)&string,
+                                                            (apply_style) ? GUI_WINDOW_OBJECTS(window)->win_chat : NULL);
                         break;
                     case GUI_COLOR_BAR_CHAR: /* bar color */
-                        switch (string[1])
+                        string++;
+                        switch (string[0])
                         {
                             case GUI_COLOR_BAR_FG_CHAR:
                             case GUI_COLOR_BAR_DELIM_CHAR:
@@ -274,91 +209,35 @@ gui_chat_string_next_char (struct t_gui_window *window,
                             case GUI_COLOR_BAR_START_INPUT_CHAR:
                             case GUI_COLOR_BAR_START_INPUT_HIDDEN_CHAR:
                             case GUI_COLOR_BAR_MOVE_CURSOR_CHAR:
-                                string += 2;
-                                break;
-                            default:
                                 string++;
                                 break;
                         }
                         break;
                     default:
-                        if (isdigit (string[0]) && isdigit (string[1]))
-                        {
-                            if (apply_style)
-                            {
-                                str_fg[0] = string[0];
-                                str_fg[1] = string[1];
-                                str_fg[2] = '\0';
-                                sscanf (str_fg, "%d", &weechat_color);
-                                gui_window_set_weechat_color (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                              weechat_color);
-                            }
-                            string += 2;
-                        }
+                        gui_window_string_apply_color_weechat ((unsigned char **)&string,
+                                                               (apply_style) ? GUI_WINDOW_OBJECTS(window)->win_chat : NULL);
                         break;
                 }
                 break;
-            case GUI_COLOR_SET_WEECHAT_CHAR:
+            case GUI_COLOR_SET_ATTR_CHAR:
                 string++;
-                switch (string[0])
-                {
-                    case GUI_COLOR_ATTR_BOLD_CHAR:
-                        string++;
-                        if (apply_style)
-                            gui_window_set_color_style (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                        A_BOLD);
-                        break;
-                    case GUI_COLOR_ATTR_REVERSE_CHAR:
-                        string++;
-                        if (apply_style)
-                            gui_window_set_color_style (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                        A_REVERSE);
-                        break;
-                    case GUI_COLOR_ATTR_ITALIC_CHAR:
-                        /* not available in Curses GUI */
-                        string++;
-                        break;
-                    case GUI_COLOR_ATTR_UNDERLINE_CHAR:
-                        string++;
-                        if (apply_style)
-                            gui_window_set_color_style (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                        A_UNDERLINE);
-                        break;
-                }
+                gui_window_string_apply_color_set_attr ((unsigned char **)&string,
+                                                        (apply_style) ? GUI_WINDOW_OBJECTS(window)->win_chat : NULL);
                 break;
-            case GUI_COLOR_REMOVE_WEECHAT_CHAR:
+            case GUI_COLOR_REMOVE_ATTR_CHAR:
                 string++;
-                switch (string[0])
-                {
-                    case GUI_COLOR_ATTR_BOLD_CHAR:
-                        string++;
-                        if (apply_style)
-                            gui_window_remove_color_style (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                           A_BOLD);
-                        break;
-                    case GUI_COLOR_ATTR_REVERSE_CHAR:
-                        string++;
-                        if (apply_style)
-                            gui_window_remove_color_style (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                           A_REVERSE);
-                        break;
-                    case GUI_COLOR_ATTR_ITALIC_CHAR:
-                        /* not available in Curses GUI */
-                        string++;
-                        break;
-                    case GUI_COLOR_ATTR_UNDERLINE_CHAR:
-                        string++;
-                        if (apply_style)
-                            gui_window_remove_color_style (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                                           A_UNDERLINE);
-                        break;
-                }
+                gui_window_string_apply_color_remove_attr ((unsigned char **)&string,
+                                                           (apply_style) ? GUI_WINDOW_OBJECTS(window)->win_chat : NULL);
+                break;
+            case GUI_COLOR_RESET_CHAR:
+                string++;
+                if (apply_style)
+                    gui_window_reset_style (GUI_WINDOW_OBJECTS(window)->win_chat, GUI_COLOR_CHAT);
                 break;
             default:
                 return (char *)string;
                 break;
         }
-            
     }
     
     /* nothing found except color/attrib codes, so return NULL */
@@ -438,7 +317,9 @@ gui_chat_display_word (struct t_gui_window *window,
     int pos_saved_char, chars_to_display, num_displayed;
     int length_align;
     attr_t attrs;
+    attr_t *ptr_attrs;
     short pair;
+    short *ptr_pair;
     
     if (!data ||
         ((!simulate) && (window->win_chat_cursor_y >= window->win_chat_height)))
@@ -461,8 +342,7 @@ gui_chat_display_word (struct t_gui_window *window,
     {
         /* insert spaces for aligning text under time/nick */
         length_align = gui_line_get_align (window->buffer, line, 0, 0);
-        if ((length_align > 0)
-            && (window->win_chat_cursor_x == 0)
+        if ((window->win_chat_cursor_x == 0)
             && (*lines_displayed > 0)
             /* FIXME: modify arbitraty value for non aligning messages on time/nick? */
             && (length_align < (window->win_chat_width - 5)))
@@ -477,12 +357,15 @@ gui_chat_display_word (struct t_gui_window *window,
             window->win_chat_cursor_x += length_align;
             if ((CONFIG_INTEGER(config_look_align_end_of_lines) == CONFIG_LOOK_ALIGN_END_OF_LINES_MESSAGE)
                 && (CONFIG_INTEGER(config_look_prefix_align) != CONFIG_LOOK_PREFIX_ALIGN_NONE)
-                && (CONFIG_STRING(config_look_prefix_suffix)
-                    && CONFIG_STRING(config_look_prefix_suffix)[0]))
+                && CONFIG_STRING(config_look_prefix_suffix)
+                && CONFIG_STRING(config_look_prefix_suffix)[0])
             {
                 if (!simulate)
                 {
-                    wattr_get (GUI_WINDOW_OBJECTS(window)->win_chat, &attrs, &pair, NULL);
+                    ptr_attrs = &attrs;
+                    ptr_pair = &pair;
+                    wattr_get (GUI_WINDOW_OBJECTS(window)->win_chat,
+                               ptr_attrs, ptr_pair, NULL);
                     gui_window_set_weechat_color (GUI_WINDOW_OBJECTS(window)->win_chat,
                                                   GUI_COLOR_CHAT_PREFIX_SUFFIX);
                     gui_chat_display_word_raw (window,
@@ -564,36 +447,28 @@ gui_chat_display_time_and_prefix (struct t_gui_window *window,
     int i, length, length_allowed, num_spaces;
     struct t_gui_lines *mixed_lines;
     
+    if (!simulate)
+        gui_window_reset_style (GUI_WINDOW_OBJECTS(window)->win_chat, GUI_COLOR_CHAT);
+    
     /* display time */
     if (window->buffer->time_for_each_line
         && (line->data->str_time && line->data->str_time[0]))
     {
-        if (!simulate)
-            gui_window_reset_style (GUI_WINDOW_OBJECTS(window)->win_chat, GUI_COLOR_CHAT);
-        
         gui_chat_display_word (window, line, line->data->str_time,
                                NULL, 1, num_lines, count, lines_displayed,
                                simulate);
+        
+        if (!simulate)
+            gui_window_reset_style (GUI_WINDOW_OBJECTS(window)->win_chat, GUI_COLOR_CHAT);
         gui_chat_display_word (window, line, str_space,
                                NULL, 1, num_lines, count, lines_displayed,
                                simulate);
     }
-    else
-    {
-        if (!simulate)
-            gui_window_reset_style (GUI_WINDOW_OBJECTS(window)->win_chat, GUI_COLOR_CHAT);
-    }
     
     /* display buffer name (if many buffers are merged) */
     mixed_lines = line->data->buffer->mixed_lines;
     if (mixed_lines)
     {
-        if (!simulate)
-        {
-            gui_window_set_weechat_color (GUI_WINDOW_OBJECTS(window)->win_chat,
-                                          GUI_COLOR_CHAT_PREFIX_BUFFER);
-        }
-        
         if ((CONFIG_INTEGER(config_look_prefix_buffer_align_max) > 0)
             && (CONFIG_INTEGER(config_look_prefix_buffer_align) != CONFIG_LOOK_PREFIX_BUFFER_ALIGN_NONE))
         {
@@ -609,6 +484,11 @@ gui_chat_display_time_and_prefix (struct t_gui_window *window,
         
         if (CONFIG_INTEGER(config_look_prefix_buffer_align) == CONFIG_LOOK_PREFIX_BUFFER_ALIGN_RIGHT)
         {
+            if (!simulate)
+            {
+                gui_window_reset_style (GUI_WINDOW_OBJECTS(window)->win_chat,
+                                        GUI_COLOR_CHAT);
+            }
             for (i = 0; i < num_spaces; i++)
             {
                 gui_chat_display_word (window, line, str_space,
@@ -617,6 +497,12 @@ gui_chat_display_time_and_prefix (struct t_gui_window *window,
             }
         }
         
+        if (!simulate)
+        {
+            gui_window_set_weechat_color (GUI_WINDOW_OBJECTS(window)->win_chat,
+                                          GUI_COLOR_CHAT_PREFIX_BUFFER);
+        }
+        
         /* not enough space to display full buffer name? => truncate it! */
         if ((CONFIG_INTEGER(config_look_prefix_buffer_align) != CONFIG_LOOK_PREFIX_BUFFER_ALIGN_NONE)
             && (num_spaces < 0))
@@ -653,6 +539,11 @@ gui_chat_display_time_and_prefix (struct t_gui_window *window,
         }
         else
         {
+            if (!simulate)
+            {
+                gui_window_reset_style (GUI_WINDOW_OBJECTS(window)->win_chat,
+                                        GUI_COLOR_CHAT);
+            }
             if ((CONFIG_INTEGER(config_look_prefix_buffer_align) == CONFIG_LOOK_PREFIX_BUFFER_ALIGN_LEFT)
                 || ((CONFIG_INTEGER(config_look_prefix_buffer_align) == CONFIG_LOOK_PREFIX_BUFFER_ALIGN_NONE)
                 && (CONFIG_INTEGER(config_look_prefix_align) != CONFIG_LOOK_PREFIX_ALIGN_NONE)))
@@ -1013,7 +904,7 @@ gui_chat_display_line_y (struct t_gui_window *window, struct t_gui_line *line,
     if (gui_chat_display_word_raw (window, line->data->message,
                                    window->win_chat_width, 1) < window->win_chat_width)
     {
-        gui_window_clrtoeol_with_current_bg (GUI_WINDOW_OBJECTS(window)->win_chat);
+        gui_window_clrtoeol (GUI_WINDOW_OBJECTS(window)->win_chat);
     }
 }
 
@@ -1127,7 +1018,8 @@ gui_chat_draw (struct t_gui_buffer *buffer, int erase)
     struct t_gui_window *ptr_win;
     struct t_gui_line *ptr_line;
     char format_empty[32];
-    int i, line_pos, count, old_scroll, old_scroll_lines_after, y_start, y_end;
+    int i, line_pos, count, old_scrolling, old_lines_after;
+    int y_start, y_end;
     
     if (!gui_ok)
         return;
@@ -1159,10 +1051,10 @@ gui_chat_draw (struct t_gui_buffer *buffer, int erase)
             {
                 case GUI_BUFFER_TYPE_FORMATTED:
                     /* display at position of scrolling */
-                    if (ptr_win->start_line)
+                    if (ptr_win->scroll->start_line)
                     {
-                        ptr_line = ptr_win->start_line;
-                        line_pos = ptr_win->start_line_pos;
+                        ptr_line = ptr_win->scroll->start_line;
+                        line_pos = ptr_win->scroll->start_line_pos;
                     }
                     else
                     {
@@ -1184,10 +1076,10 @@ gui_chat_draw (struct t_gui_buffer *buffer, int erase)
                                                                               0, 1) -
                                                        line_pos, 0);
                         ptr_line = gui_line_get_next_displayed (ptr_line);
-                        ptr_win->first_line_displayed = 0;
+                        ptr_win->scroll->first_line_displayed = 0;
                     }
                     else
-                        ptr_win->first_line_displayed =
+                        ptr_win->scroll->first_line_displayed =
                             (ptr_line == gui_line_get_first_displayed (ptr_win->buffer));
                     
                     /* display lines */
@@ -1197,52 +1089,52 @@ gui_chat_draw (struct t_gui_buffer *buffer, int erase)
                         ptr_line = gui_line_get_next_displayed (ptr_line);
                     }
                     
-                    old_scroll = ptr_win->scroll;
-                    old_scroll_lines_after = ptr_win->scroll_lines_after;
+                    old_scrolling = ptr_win->scroll->scrolling;
+                    old_lines_after = ptr_win->scroll->lines_after;
                     
-                    ptr_win->scroll = (ptr_win->win_chat_cursor_y > ptr_win->win_chat_height - 1);
+                    ptr_win->scroll->scrolling = (ptr_win->win_chat_cursor_y > ptr_win->win_chat_height - 1);
                     
                     /* check if last line of buffer is entirely displayed and scrolling */
                     /* if so, disable scroll indicator */
-                    if (!ptr_line && ptr_win->scroll)
+                    if (!ptr_line && ptr_win->scroll->scrolling)
                     {
                         if ((count == gui_chat_display_line (ptr_win, gui_line_get_last_displayed (ptr_win->buffer), 0, 1))
                             || (count == ptr_win->win_chat_height))
-                            ptr_win->scroll = 0;
+                            ptr_win->scroll->scrolling = 0;
                     }
                     
-                    if (!ptr_win->scroll
-                        && (ptr_win->start_line == gui_line_get_first_displayed (ptr_win->buffer)))
+                    if (!ptr_win->scroll->scrolling
+                        && (ptr_win->scroll->start_line == gui_line_get_first_displayed (ptr_win->buffer)))
                     {
-                        ptr_win->start_line = NULL;
-                        ptr_win->start_line_pos = 0;
+                        ptr_win->scroll->start_line = NULL;
+                        ptr_win->scroll->start_line_pos = 0;
                     }
                     
-                    ptr_win->scroll_lines_after = 0;
-                    if (ptr_win->scroll && ptr_line)
+                    ptr_win->scroll->lines_after = 0;
+                    if (ptr_win->scroll->scrolling && ptr_line)
                     {
                         /* count number of lines after last line displayed */
                         while (ptr_line)
                         {
                             ptr_line = gui_line_get_next_displayed (ptr_line);
                             if (ptr_line)
-                                ptr_win->scroll_lines_after++;
+                                ptr_win->scroll->lines_after++;
                         }
-                        ptr_win->scroll_lines_after++;
+                        ptr_win->scroll->lines_after++;
                     }
                     
-                    if ((ptr_win->scroll != old_scroll)
-                        || (ptr_win->scroll_lines_after != old_scroll_lines_after))
+                    if ((ptr_win->scroll->scrolling != old_scrolling)
+                        || (ptr_win->scroll->lines_after != old_lines_after))
                     {
                         hook_signal_send ("window_scrolled",
                                           WEECHAT_HOOK_SIGNAL_POINTER, ptr_win);
                     }
                     
-                    if (!ptr_win->scroll && ptr_win->scroll_reset_allowed)
+                    if (!ptr_win->scroll->scrolling
+                        && ptr_win->scroll->reset_allowed)
                     {
-                        ptr_win->start_line = NULL;
-                        ptr_win->start_line_pos = 0;
-                        gui_hotlist_remove_buffer (ptr_win->buffer);
+                        ptr_win->scroll->start_line = NULL;
+                        ptr_win->scroll->start_line_pos = 0;
                     }
                     
                     /* cursor is below end line of chat window? */
@@ -1252,20 +1144,20 @@ gui_chat_draw (struct t_gui_buffer *buffer, int erase)
                         ptr_win->win_chat_cursor_y = ptr_win->win_chat_height - 1;
                     }
                     
-                    ptr_win->scroll_reset_allowed = 0;
+                    ptr_win->scroll->reset_allowed = 0;
                     
                     break;
                 case GUI_BUFFER_TYPE_FREE:
                     /* display at position of scrolling */
-                    ptr_line = (ptr_win->start_line) ?
-                        ptr_win->start_line : buffer->lines->first_line;
+                    ptr_line = (ptr_win->scroll->start_line) ?
+                        ptr_win->scroll->start_line : buffer->lines->first_line;
                     if (ptr_line)
                     {
                         if (!ptr_line->data->displayed)
                             ptr_line = gui_line_get_next_displayed (ptr_line);
                         if (ptr_line)
                         {
-                            y_start = (ptr_win->start_line) ? ptr_line->data->y : 0;
+                            y_start = (ptr_win->scroll->start_line) ? ptr_line->data->y : 0;
                             y_end = y_start + ptr_win->win_chat_height - 1;
                             while (ptr_line && (ptr_line->data->y <= y_end))
                             {
