@@ -81,114 +81,126 @@ gui_keyboard_default_bindings ()
     int i;
     char key_str[32], command[32];
     
-    BIND(/* RC              */ "ctrl-M",             "/input return");
-    BIND(/* RC              */ "ctrl-J",             "/input return");
-    BIND(/* tab             */ "ctrl-I",             "/input complete_next");
-    BIND(/* s-tab           */ "meta2-Z",            "/input complete_previous");
-    BIND(/* ^R              */ "ctrl-R",             "/input search_text");
-    BIND(/* basckpace       */ "ctrl-H",             "/input delete_previous_char");
-    BIND(/* basckpace       */ "ctrl-?",             "/input delete_previous_char");
-    BIND(/* ^_              */ "ctrl-_",             "/input undo");
-    BIND(/* m-_             */ "meta-_",             "/input redo");
-    BIND(/* del             */ "meta2-3~",           "/input delete_next_char");
-    BIND(/* ^D              */ "ctrl-D",             "/input delete_next_char");
-    BIND(/* ^W              */ "ctrl-W",             "/input delete_previous_word");
-    BIND(/* ^X              */ "ctrl-X",             "/input switch_active_buffer");
-    BIND(/* m-d             */ "meta-d",             "/input delete_next_word");
-    BIND(/* ^K              */ "ctrl-K",             "/input delete_end_of_line");
-    BIND(/* m-r             */ "meta-r",             "/input delete_line");
-    BIND(/* ^T              */ "ctrl-T",             "/input transpose_chars");
-    BIND(/* ^U              */ "ctrl-U",             "/input delete_beginning_of_line");
-    BIND(/* ^Y              */ "ctrl-Y",             "/input clipboard_paste");
-    BIND(/* home            */ "meta2-1~",           "/input move_beginning_of_line");
-    BIND(/* home            */ "meta2-H",            "/input move_beginning_of_line");
-    BIND(/* home            */ "meta2-7~",           "/input move_beginning_of_line");
-    BIND(/* ^A              */ "ctrl-A",             "/input move_beginning_of_line");
-    BIND(/* end             */ "meta2-4~",           "/input move_end_of_line");
-    BIND(/* end             */ "meta2-F",            "/input move_end_of_line");
-    BIND(/* end             */ "meta2-8~",           "/input move_end_of_line");
-    BIND(/* ^E              */ "ctrl-E",             "/input move_end_of_line");
-    BIND(/* left            */ "meta2-D",            "/input move_previous_char");
-    BIND(/* ^B              */ "ctrl-B",             "/input move_previous_char");
-    BIND(/* right           */ "meta2-C",            "/input move_next_char");
-    BIND(/* ^F              */ "ctrl-F",             "/input move_next_char");
-    BIND(/* m-b             */ "meta-b",             "/input move_previous_word");
-    BIND(/* ^left           */ "meta-Od",            "/input move_previous_word");
-    BIND(/* m-f             */ "meta-f",             "/input move_next_word");
-    BIND(/* ^right          */ "meta-Oc",            "/input move_next_word");
-    BIND(/* up              */ "meta2-A",            "/input history_previous");
-    BIND(/* down            */ "meta2-B",            "/input history_next");
-    BIND(/* ^up             */ "meta-Oa",            "/input history_global_previous");
-    BIND(/* ^down           */ "meta-Ob",            "/input history_global_next");
-    BIND(/* m-a             */ "meta-a",             "/input jump_smart");
-    BIND(/* m-j,m-l         */ "meta-jmeta-l",       "/input jump_last_buffer");
-    BIND(/* m-j,m-r         */ "meta-jmeta-r",       "/server raw");
-    BIND(/* m-j,m-s         */ "meta-jmeta-s",       "/server jump");
-    BIND(/* m-h             */ "meta-h",             "/input hotlist_clear");
-    BIND(/* m-k             */ "meta-k",             "/input grab_key_command");
-    BIND(/* m-u             */ "meta-u",             "/input scroll_unread");
-    BIND(/* ^S^U            */ "ctrl-Sctrl-U",       "/input set_unread");
-    BIND(/* ^Cb             */ "ctrl-Cb",            "/input insert \\x02");
-    BIND(/* ^Cc             */ "ctrl-Cc",            "/input insert \\x03");
-    BIND(/* ^Ci             */ "ctrl-Ci",            "/input insert \\x1D");
-    BIND(/* ^Co             */ "ctrl-Co",            "/input insert \\x0F");
-    BIND(/* ^Cr             */ "ctrl-Cr",            "/input insert \\x12");
-    BIND(/* ^Cu             */ "ctrl-Cu",            "/input insert \\x15");
-    BIND(/* m-right         */ "meta-meta2-C",       "/buffer +1");
-    BIND(/* m-right (K)     */ "meta2-1;3C",         "/buffer +1");
-    BIND(/* m-down          */ "meta-meta2-B",       "/buffer +1");
-    BIND(/* m-down (K)      */ "meta2-1;3B",         "/buffer +1");
-    BIND(/* F6              */ "meta2-17~",          "/buffer +1");
-    BIND(/* ^N              */ "ctrl-N",             "/buffer +1");
-    BIND(/* m-left          */ "meta-meta2-D",       "/buffer -1");
-    BIND(/* m-left (K)      */ "meta2-1;3D",         "/buffer -1");
-    BIND(/* m-up            */ "meta-meta2-A",       "/buffer -1");
-    BIND(/* m-up (K)        */ "meta2-1;3A",         "/buffer -1");
-    BIND(/* F5              */ "meta2-15~",          "/buffer -1");
-    BIND(/* ^P              */ "ctrl-P",             "/buffer -1");
-    BIND(/* pgup            */ "meta2-5~",           "/window page_up");
-    BIND(/* pgup            */ "meta2-I",            "/window page_up");
-    BIND(/* pgdn            */ "meta2-6~",           "/window page_down");
-    BIND(/* pgdn            */ "meta2-G",            "/window page_down");
-    BIND(/* m-pgup          */ "meta-meta2-5~",      "/window scroll_up");
-    BIND(/* m-pgdn          */ "meta-meta2-6~",      "/window scroll_down");
-    BIND(/* m-home          */ "meta-meta2-1~",      "/window scroll_top");
-    BIND(/* m-home          */ "meta-meta2-7~",      "/window scroll_top");
-    BIND(/* m-end           */ "meta-meta2-4~",      "/window scroll_bottom");
-    BIND(/* m-end           */ "meta-meta2-8~",      "/window scroll_bottom");
-    BIND(/* m-n             */ "meta-n",             "/window scroll_next_highlight");
-    BIND(/* m-p             */ "meta-p",             "/window scroll_previous_highlight");
-    BIND(/* F9              */ "meta2-20~",          "/bar scroll title * x-50%");
-    BIND(/* F10             */ "meta2-21~",          "/bar scroll title * x+50%");
-    BIND(/* F11             */ "meta2-23~",          "/bar scroll nicklist * y-100%");
-    BIND(/* F12             */ "meta2-24~",          "/bar scroll nicklist * y+100%");
-    BIND(/* m-F11           */ "meta-meta2-23~",     "/bar scroll nicklist * yb");
-    BIND(/* m-F12           */ "meta-meta2-24~",     "/bar scroll nicklist * ye");
-    BIND(/* ^L              */ "ctrl-L",             "/window refresh");
-    BIND(/* F7              */ "meta2-18~",          "/window -1");
-    BIND(/* F8              */ "meta2-19~",          "/window +1");
-    BIND(/* m-w,m-up        */ "meta-wmeta-meta2-A", "/window up");
-    BIND(/* m-w,m-up (K)    */ "meta-wmeta2-1;3A",   "/window up");
-    BIND(/* m-w,m-down      */ "meta-wmeta-meta2-B", "/window down");
-    BIND(/* m-w,m-down (K)  */ "meta-wmeta2-1;3B",   "/window down");
-    BIND(/* m-w,m-right     */ "meta-wmeta-meta2-C", "/window right");
-    BIND(/* m-w,m-right (K) */ "meta-wmeta2-1;3C",   "/window right");
-    BIND(/* m-w,m-left      */ "meta-wmeta-meta2-D", "/window left");
-    BIND(/* m-w,m-left (K)  */ "meta-wmeta2-1;3D",   "/window left");
-    BIND(/* m-z             */ "meta-z",             "/window zoom");
-    BIND(/* m-=             */ "meta-=",             "/filter toggle");
-    BIND(/* m-0             */ "meta-0",             "/buffer *10");
-    BIND(/* m-1             */ "meta-1",             "/buffer *1");
-    BIND(/* m-2             */ "meta-2",             "/buffer *2");
-    BIND(/* m-3             */ "meta-3",             "/buffer *3");
-    BIND(/* m-4             */ "meta-4",             "/buffer *4");
-    BIND(/* m-5             */ "meta-5",             "/buffer *5");
-    BIND(/* m-6             */ "meta-6",             "/buffer *6");
-    BIND(/* m-7             */ "meta-7",             "/buffer *7");
-    BIND(/* m-8             */ "meta-8",             "/buffer *8");
-    BIND(/* m-9             */ "meta-9",             "/buffer *9");
-    BIND(/* m-<             */ "meta-<",             "/input jump_previously_visited_buffer");
-    BIND(/* m->             */ "meta->",             "/input jump_next_visited_buffer");
+    BIND(/* RC          */ "ctrl-M",             "/input return");
+    BIND(/* RC          */ "ctrl-J",             "/input return");
+    BIND(/* tab         */ "ctrl-I",             "/input complete_next");
+    BIND(/* s-tab       */ "meta2-Z",            "/input complete_previous");
+    BIND(/* ^R          */ "ctrl-R",             "/input search_text");
+    BIND(/* basckpace   */ "ctrl-H",             "/input delete_previous_char");
+    BIND(/* basckpace   */ "ctrl-?",             "/input delete_previous_char");
+    BIND(/* ^_          */ "ctrl-_",             "/input undo");
+    BIND(/* m-_         */ "meta-_",             "/input redo");
+    BIND(/* del         */ "meta2-3~",           "/input delete_next_char");
+    BIND(/* ^D          */ "ctrl-D",             "/input delete_next_char");
+    BIND(/* ^W          */ "ctrl-W",             "/input delete_previous_word");
+    BIND(/* ^X          */ "ctrl-X",             "/input switch_active_buffer");
+    BIND(/* m-d         */ "meta-d",             "/input delete_next_word");
+    BIND(/* ^K          */ "ctrl-K",             "/input delete_end_of_line");
+    BIND(/* m-r         */ "meta-r",             "/input delete_line");
+    BIND(/* ^T          */ "ctrl-T",             "/input transpose_chars");
+    BIND(/* ^U          */ "ctrl-U",             "/input delete_beginning_of_line");
+    BIND(/* ^Y          */ "ctrl-Y",             "/input clipboard_paste");
+    BIND(/* home        */ "meta2-1~",           "/input move_beginning_of_line");
+    BIND(/* home        */ "meta2-H",            "/input move_beginning_of_line");
+    BIND(/* home        */ "meta2-7~",           "/input move_beginning_of_line");
+    BIND(/* home        */ "meta-OH",            "/input move_beginning_of_line");
+    BIND(/* ^A          */ "ctrl-A",             "/input move_beginning_of_line");
+    BIND(/* end         */ "meta2-4~",           "/input move_end_of_line");
+    BIND(/* end         */ "meta2-F",            "/input move_end_of_line");
+    BIND(/* end         */ "meta2-8~",           "/input move_end_of_line");
+    BIND(/* end         */ "meta-OF",            "/input move_end_of_line");
+    BIND(/* ^E          */ "ctrl-E",             "/input move_end_of_line");
+    BIND(/* left        */ "meta2-D",            "/input move_previous_char");
+    BIND(/* ^B          */ "ctrl-B",             "/input move_previous_char");
+    BIND(/* right       */ "meta2-C",            "/input move_next_char");
+    BIND(/* ^F          */ "ctrl-F",             "/input move_next_char");
+    BIND(/* m-b         */ "meta-b",             "/input move_previous_word");
+    BIND(/* ^left       */ "meta-Od",            "/input move_previous_word");
+    BIND(/* ^left       */ "meta-OD",            "/input move_previous_word");
+    BIND(/* m-f         */ "meta-f",             "/input move_next_word");
+    BIND(/* ^right      */ "meta-Oc",            "/input move_next_word");
+    BIND(/* ^right      */ "meta-OC",            "/input move_next_word");
+    BIND(/* up          */ "meta2-A",            "/input history_previous");
+    BIND(/* down        */ "meta2-B",            "/input history_next");
+    BIND(/* ^up         */ "meta-Oa",            "/input history_global_previous");
+    BIND(/* ^up         */ "meta-OA",            "/input history_global_previous");
+    BIND(/* ^up         */ "meta2-1;5A",         "/input history_global_previous");
+    BIND(/* ^down       */ "meta-Ob",            "/input history_global_next");
+    BIND(/* ^down       */ "meta-OB",            "/input history_global_next");
+    BIND(/* ^down       */ "meta2-1;5B",         "/input history_global_next");
+    BIND(/* m-a         */ "meta-a",             "/input jump_smart");
+    BIND(/* m-j,m-l     */ "meta-jmeta-l",       "/input jump_last_buffer");
+    BIND(/* m-j,m-r     */ "meta-jmeta-r",       "/server raw");
+    BIND(/* m-j,m-s     */ "meta-jmeta-s",       "/server jump");
+    BIND(/* m-h         */ "meta-h",             "/input hotlist_clear");
+    BIND(/* m-k         */ "meta-k",             "/input grab_key_command");
+    BIND(/* m-u         */ "meta-u",             "/input scroll_unread");
+    BIND(/* ^S^U        */ "ctrl-Sctrl-U",       "/input set_unread");
+    BIND(/* ^Cb         */ "ctrl-Cb",            "/input insert \\x02");
+    BIND(/* ^Cc         */ "ctrl-Cc",            "/input insert \\x03");
+    BIND(/* ^Ci         */ "ctrl-Ci",            "/input insert \\x1D");
+    BIND(/* ^Co         */ "ctrl-Co",            "/input insert \\x0F");
+    BIND(/* ^Cr         */ "ctrl-Cr",            "/input insert \\x12");
+    BIND(/* ^Cu         */ "ctrl-Cu",            "/input insert \\x15");
+    BIND(/* m-right     */ "meta-meta2-C",       "/buffer +1");
+    BIND(/* m-right     */ "meta2-1;3C",         "/buffer +1");
+    BIND(/* m-down      */ "meta-meta2-B",       "/buffer +1");
+    BIND(/* m-down      */ "meta2-1;3B",         "/buffer +1");
+    BIND(/* F6          */ "meta2-17~",          "/buffer +1");
+    BIND(/* ^N          */ "ctrl-N",             "/buffer +1");
+    BIND(/* m-left      */ "meta-meta2-D",       "/buffer -1");
+    BIND(/* m-left      */ "meta2-1;3D",         "/buffer -1");
+    BIND(/* m-up        */ "meta-meta2-A",       "/buffer -1");
+    BIND(/* m-up        */ "meta2-1;3A",         "/buffer -1");
+    BIND(/* F5          */ "meta2-15~",          "/buffer -1");
+    BIND(/* ^P          */ "ctrl-P",             "/buffer -1");
+    BIND(/* pgup        */ "meta2-5~",           "/window page_up");
+    BIND(/* pgup        */ "meta2-I",            "/window page_up");
+    BIND(/* pgdn        */ "meta2-6~",           "/window page_down");
+    BIND(/* pgdn        */ "meta2-G",            "/window page_down");
+    BIND(/* m-pgup      */ "meta-meta2-5~",      "/window scroll_up");
+    BIND(/* m-pgup      */ "meta2-5;3~",         "/window scroll_up");
+    BIND(/* m-pgdn      */ "meta-meta2-6~",      "/window scroll_down");
+    BIND(/* m-pgdn      */ "meta2-6;3~",         "/window scroll_down");
+    BIND(/* m-home      */ "meta-meta2-1~",      "/window scroll_top");
+    BIND(/* m-home      */ "meta-meta2-7~",      "/window scroll_top");
+    BIND(/* m-end       */ "meta-meta2-4~",      "/window scroll_bottom");
+    BIND(/* m-end       */ "meta-meta2-8~",      "/window scroll_bottom");
+    BIND(/* m-n         */ "meta-n",             "/window scroll_next_highlight");
+    BIND(/* m-p         */ "meta-p",             "/window scroll_previous_highlight");
+    BIND(/* F9          */ "meta2-20~",          "/bar scroll title * x-50%");
+    BIND(/* F10         */ "meta2-21~",          "/bar scroll title * x+50%");
+    BIND(/* F11         */ "meta2-23~",          "/bar scroll nicklist * y-100%");
+    BIND(/* F12         */ "meta2-24~",          "/bar scroll nicklist * y+100%");
+    BIND(/* m-F11       */ "meta-meta2-23~",     "/bar scroll nicklist * yb");
+    BIND(/* m-F12       */ "meta-meta2-24~",     "/bar scroll nicklist * ye");
+    BIND(/* ^L          */ "ctrl-L",             "/window refresh");
+    BIND(/* F7          */ "meta2-18~",          "/window -1");
+    BIND(/* F8          */ "meta2-19~",          "/window +1");
+    BIND(/* m-w,m-up    */ "meta-wmeta-meta2-A", "/window up");
+    BIND(/* m-w,m-up    */ "meta-wmeta2-1;3A",   "/window up");
+    BIND(/* m-w,m-down  */ "meta-wmeta-meta2-B", "/window down");
+    BIND(/* m-w,m-down  */ "meta-wmeta2-1;3B",   "/window down");
+    BIND(/* m-w,m-right */ "meta-wmeta-meta2-C", "/window right");
+    BIND(/* m-w,m-right */ "meta-wmeta2-1;3C",   "/window right");
+    BIND(/* m-w,m-left  */ "meta-wmeta-meta2-D", "/window left");
+    BIND(/* m-w,m-left  */ "meta-wmeta2-1;3D",   "/window left");
+    BIND(/* m-w,m-b     */ "meta-wmeta-b",       "/window balance");
+    BIND(/* m-w,m-s     */ "meta-wmeta-s",       "/window swap");
+    BIND(/* m-z         */ "meta-z",             "/window zoom");
+    BIND(/* m-=         */ "meta-=",             "/filter toggle");
+    BIND(/* m-0         */ "meta-0",             "/buffer *10");
+    BIND(/* m-1         */ "meta-1",             "/buffer *1");
+    BIND(/* m-2         */ "meta-2",             "/buffer *2");
+    BIND(/* m-3         */ "meta-3",             "/buffer *3");
+    BIND(/* m-4         */ "meta-4",             "/buffer *4");
+    BIND(/* m-5         */ "meta-5",             "/buffer *5");
+    BIND(/* m-6         */ "meta-6",             "/buffer *6");
+    BIND(/* m-7         */ "meta-7",             "/buffer *7");
+    BIND(/* m-8         */ "meta-8",             "/buffer *8");
+    BIND(/* m-9         */ "meta-9",             "/buffer *9");
+    BIND(/* m-<         */ "meta-<",             "/input jump_previously_visited_buffer");
+    BIND(/* m->         */ "meta->",             "/input jump_next_visited_buffer");
     
     /* bind meta-j + {01..99} to switch to buffers # > 10 */
     for (i = 1; i < 100; i++)
