@@ -30,6 +30,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
+#include <time.h>
 
 #include "../../core/weechat.h"
 #include "../../core/wee-command.h"
@@ -126,6 +127,8 @@ gui_main_init ()
     {
         gui_init_ok = 1;
         
+        ptr_buffer->num_displayed = 1;
+        
         /* set title for core buffer */
         gui_buffer_set_title (ptr_buffer,
                               "WeeChat " PACKAGE_VERSION " "
@@ -225,6 +228,13 @@ gui_main_refreshs ()
     struct t_gui_buffer *ptr_buffer;
     struct t_gui_bar *ptr_bar;
     
+    /* refresh color buffer if needed */
+    if (gui_color_buffer_refresh_needed)
+    {
+        gui_color_buffer_display ();
+        gui_color_buffer_refresh_needed = 0;
+    }
+    
     /* refresh window if needed */
     if (gui_window_refresh_needed)
     {
@@ -318,11 +328,22 @@ gui_main_loop ()
         
         /* execute hook timers */
         hook_timer_exec ();
+
+        /* auto reset of color pairs */
+        if (gui_color_pairs_auto_reset)
+        {
+            gui_color_reset_pairs ();
+            gui_color_pairs_auto_reset_last = time (NULL);
+            gui_color_pairs_auto_reset = 0;
+            gui_color_pairs_auto_reset_pending = 1;
+        }
         
         gui_main_refreshs ();
         if (gui_window_refresh_needed)
             gui_main_refreshs ();
         
+        gui_color_pairs_auto_reset_pending = 0;
+        
         /* wait for keyboard or network activity */
         FD_ZERO (&read_fds);
         FD_ZERO (&write_fds);
