@@ -20,6 +20,8 @@
 #ifndef __WEECHAT_GUI_CURSES_H
 #define __WEECHAT_GUI_CURSES_H 1
 
+#include <time.h>
+
 #ifdef HAVE_NCURSESW_CURSES_H
 #include <ncursesw/ncurses.h>
 #elif HAVE_NCURSES_H
@@ -36,7 +38,7 @@ struct t_gui_buffer;
 struct t_gui_window;
 struct t_gui_bar_window;
 
-#define GUI_CURSES_NUM_WEECHAT_COLORS 16
+#define GUI_CURSES_NUM_WEECHAT_COLORS 17
 
 #define GUI_WINDOW_OBJECTS(window)                                      \
     ((struct t_gui_window_curses_objects *)(window->gui_objects))
@@ -57,11 +59,16 @@ struct t_gui_bar_window_curses_objects
 
 extern int gui_term_cols, gui_term_lines;
 extern struct t_gui_color gui_weechat_colors[];
-extern int gui_color_last_pair;
-extern int gui_color_num_bg;
+extern int gui_color_term_colors;
+extern int gui_color_num_pairs;
+extern int gui_color_pairs_auto_reset;
+extern int gui_color_pairs_auto_reset_pending;
+extern time_t gui_color_pairs_auto_reset_last;
+extern int gui_color_buffer_refresh_needed;
 
 /* color functions */
-extern int gui_color_get_pair (int num_color);
+extern int gui_color_get_pair (int fg, int bg);
+extern int gui_color_weechat_get_pair (int weechat_color);
 extern void gui_color_pre_init ();
 extern void gui_color_init ();
 extern void gui_color_end ();
@@ -78,8 +85,8 @@ extern int gui_keyboard_read_cb (void *data, int fd);
 /* window functions */
 extern void gui_window_read_terminal_size ();
 extern void gui_window_redraw_buffer (struct t_gui_buffer *buffer);
-extern int gui_window_get_hline_char ();
-extern void gui_window_clear (WINDOW *window, int bg);
+extern void gui_window_clear (WINDOW *window, int fg, int bg);
+extern void gui_window_clrtoeol (WINDOW *window);
 extern void gui_window_reset_style (WINDOW *window, int num_color);
 extern void gui_window_set_color_style (WINDOW *window, int style);
 extern void gui_window_remove_color_style (WINDOW *window, int style);
@@ -89,7 +96,22 @@ extern void gui_window_set_custom_color_fg_bg (WINDOW *window, int fg, int bg);
 extern void gui_window_set_custom_color_pair (WINDOW *window, int pair);
 extern void gui_window_set_custom_color_fg (WINDOW *window, int fg);
 extern void gui_window_set_custom_color_bg (WINDOW *window, int bg);
-extern void gui_window_clrtoeol_with_current_bg (WINDOW *window);
+extern void gui_window_string_apply_color_fg (unsigned char **str,
+                                              WINDOW *window);
+extern void gui_window_string_apply_color_bg (unsigned char **str,
+                                              WINDOW *window);
+extern void gui_window_string_apply_color_fg_bg (unsigned char **str,
+                                                 WINDOW *window);
+extern void gui_window_string_apply_color_pair (unsigned char **str,
+                                                WINDOW *window);
+extern void gui_window_string_apply_color_weechat (unsigned char **str,
+                                                   WINDOW *window);
+extern void gui_window_string_apply_color_set_attr (unsigned char **str,
+                                                    WINDOW *window);
+extern void gui_window_string_apply_color_remove_attr (unsigned char **str,
+                                                       WINDOW *window);
+extern void gui_window_apply_color (unsigned char **str, WINDOW *window,
+                                    int apply_bar_colors);
 extern void gui_window_set_title (const char *title);
 
 #endif /* __WEECHAT_GUI_CURSES_H */
