@@ -202,8 +202,8 @@ gui_window_switch_to_buffer (struct t_gui_window *window,
     
     if (window->buffer != buffer)
     {
-        window->start_line = NULL;
-        window->start_line_pos = 0;
+        window->scroll->start_line = NULL;
+        window->scroll->start_line_pos = 0;
         if (!gui_buffers_visited_frozen)
         {
             gui_buffer_visited_add (window->buffer);
@@ -244,8 +244,8 @@ gui_window_switch_to_buffer (struct t_gui_window *window,
         gtk_text_buffer_apply_tag (GUI_WINDOW_OBJECTS(window)->textbuffer_chat, GUI_WINDOW_OBJECTS(window)->texttag_chat, &start, &end);
     }
     
-    window->start_line = NULL;
-    window->start_line_pos = 0;
+    window->scroll->start_line = NULL;
+    window->scroll->start_line_pos = 0;
     
     gui_buffer_add_value_num_displayed (buffer, 1);
     
@@ -262,11 +262,11 @@ gui_window_page_up (struct t_gui_window *window)
     if (!gui_ok)
         return;
     
-    if (!window->first_line_displayed)
+    if (!window->scroll->first_line_displayed)
     {
-        gui_chat_calculate_line_diff (window, &window->start_line,
-                                      &window->start_line_pos,
-                                      (window->start_line) ?
+        gui_chat_calculate_line_diff (window, &window->scroll->start_line,
+                                      &window->scroll->start_line_pos,
+                                      (window->scroll->start_line) ?
                                       (-1) * (window->win_chat_height - 1) :
                                       (-1) * ((window->win_chat_height - 1) * 2));
         gui_chat_draw (window->buffer, 0);
@@ -286,22 +286,22 @@ gui_window_page_down (struct t_gui_window *window)
     if (!gui_ok)
         return;
     
-    if (window->start_line)
+    if (window->scroll->start_line)
     {
-        gui_chat_calculate_line_diff (window, &window->start_line,
-                                      &window->start_line_pos,
+        gui_chat_calculate_line_diff (window, &window->scroll->start_line,
+                                      &window->scroll->start_line_pos,
                                       window->win_chat_height - 1);
         
         /* check if we can display all */
-        ptr_line = window->start_line;
-        line_pos = window->start_line_pos;
+        ptr_line = window->scroll->start_line;
+        line_pos = window->scroll->start_line_pos;
         gui_chat_calculate_line_diff (window, &ptr_line,
                                       &line_pos,
                                       window->win_chat_height - 1);
         if (!ptr_line)
         {
-            window->start_line = NULL;
-            window->start_line_pos = 0;
+            window->scroll->start_line = NULL;
+            window->scroll->start_line_pos = 0;
         }
         
         gui_chat_draw (window->buffer, 0);
@@ -318,11 +318,11 @@ gui_window_scroll_up (struct t_gui_window *window)
     if (!gui_ok)
         return;
     
-    if (!window->first_line_displayed)
+    if (!window->scroll->first_line_displayed)
     {
-        gui_chat_calculate_line_diff (window, &window->start_line,
-                                      &window->start_line_pos,
-                                      (window->start_line) ?
+        gui_chat_calculate_line_diff (window, &window->scroll->start_line,
+                                      &window->scroll->start_line_pos,
+                                      (window->scroll->start_line) ?
                                       (-1) * CONFIG_INTEGER(config_look_scroll_amount) :
                                       (-1) * ( (window->win_chat_height - 1) +
                                                CONFIG_INTEGER(config_look_scroll_amount)));
@@ -343,23 +343,23 @@ gui_window_scroll_down (struct t_gui_window *window)
     if (!gui_ok)
         return;
     
-    if (window->start_line)
+    if (window->scroll->start_line)
     {
-        gui_chat_calculate_line_diff (window, &window->start_line,
-                                      &window->start_line_pos,
+        gui_chat_calculate_line_diff (window, &window->scroll->start_line,
+                                      &window->scroll->start_line_pos,
                                       CONFIG_INTEGER(config_look_scroll_amount));
         
         /* check if we can display all */
-        ptr_line = window->start_line;
-        line_pos = window->start_line_pos;
+        ptr_line = window->scroll->start_line;
+        line_pos = window->scroll->start_line_pos;
         gui_chat_calculate_line_diff (window, &ptr_line,
                                       &line_pos,
                                       window->win_chat_height - 1);
         
         if (!ptr_line)
         {
-            window->start_line = NULL;
-            window->start_line_pos = 0;
+            window->scroll->start_line = NULL;
+            window->scroll->start_line_pos = 0;
         }
         
         gui_chat_draw (window->buffer, 0);
@@ -376,10 +376,10 @@ gui_window_scroll_top (struct t_gui_window *window)
     if (!gui_ok)
         return;
     
-    if (!window->first_line_displayed)
+    if (!window->scroll->first_line_displayed)
     {
-        window->start_line = window->buffer->lines->first_line;
-        window->start_line_pos = 0;
+        window->scroll->start_line = window->buffer->lines->first_line;
+        window->scroll->start_line_pos = 0;
         gui_chat_draw (window->buffer, 0);
     }
 }
@@ -394,10 +394,10 @@ gui_window_scroll_bottom (struct t_gui_window *window)
     if (!gui_ok)
         return;
     
-    if (window->start_line)
+    if (window->scroll->start_line)
     {
-        window->start_line = NULL;
-        window->start_line_pos = 0;
+        window->scroll->start_line = NULL;
+        window->scroll->start_line_pos = 0;
         gui_chat_draw (window->buffer, 0);
     }
 }
@@ -576,6 +576,18 @@ gui_window_resize (struct t_gui_window *window, int percentage)
     (void) percentage;
 }
 
+/*
+ * gui_window_resize_delta: resize window using delta percentage
+ */
+
+void
+gui_window_resize_delta (struct t_gui_window *window, int delta_percentage)
+{
+    /* TODO: write this function for Gtk */
+    (void) window;
+    (void) delta_percentage;
+}
+
 /*
  * gui_window_merge: merge window with its sister
  */
@@ -779,6 +791,39 @@ gui_window_switch_right (struct t_gui_window *window)
     }
 }
 
+/*
+ * gui_window_balance: balance windows (set all splits to 50%)
+ *                     return 1 if some windows have been balanced
+ *                            0 if nothing was changed
+ */
+
+int
+gui_window_balance (struct t_gui_window_tree *tree)
+{
+    (void) tree;
+    
+    /* TODO: write this function for Gtk */
+    return 0;
+}
+
+/*
+ * gui_window_swap: swap buffer of two windows
+ *                  direction can be: 0 = auto (swap with sister)
+ *                                    1 = window above
+ *                                    2 = window on the right
+ *                                    3 = window below
+ *                                    4 = window on the left
+ */
+
+void
+gui_window_swap (struct t_gui_window *window, int direction)
+{
+    (void) window;
+    (void) direction;
+    
+    /* TODO: write this function for Gtk */
+}
+
 /*
  * gui_window_refresh_screen: called when term size is modified
  */
