@@ -291,7 +291,7 @@ gui_bar_item_get_value (const char *name, struct t_gui_bar *bar,
     const char *ptr, *start, *end;
     char *prefix, *item_name, *suffix;
     char *item_value, delimiter_color[32], bar_color[32];
-    char *result;
+    char *result, str_attr[8];
     int valid_char, length;
     struct t_gui_bar_item *ptr_item;
     struct t_weechat_plugin *ptr_plugin;
@@ -377,37 +377,50 @@ gui_bar_item_get_value (const char *name, struct t_gui_bar *bar,
             bar_color[0] = '\0';
             if (prefix || suffix)
             {
-                if (CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_DELIM]) & GUI_COLOR_PAIR_FLAG)
-                {
-                    snprintf (delimiter_color, sizeof (delimiter_color),
-                              "%c%c%05d",
-                              GUI_COLOR_COLOR_CHAR,
-                              GUI_COLOR_PAIR_CHAR,
-                              CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_DELIM]) & GUI_COLOR_PAIR_MASK);
-                }
-                else
-                {
-                    snprintf (delimiter_color, sizeof (delimiter_color),
-                              "%c%c%02d",
-                              GUI_COLOR_COLOR_CHAR,
-                              GUI_COLOR_FG_CHAR,
-                              CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_DELIM]));
-                }
-                if (CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_FG]) & GUI_COLOR_PAIR_FLAG)
+                /* color for text in bar */
+                gui_color_attr_build_string (CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_FG]),
+                                             str_attr);
+                if (CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_FG]) & GUI_COLOR_EXTENDED_FLAG)
                 {
                     snprintf (bar_color, sizeof (bar_color),
-                              "%c%c%05d",
+                              "%c%c%c%s%05d",
                               GUI_COLOR_COLOR_CHAR,
-                              GUI_COLOR_PAIR_CHAR,
-                              CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_FG]) & GUI_COLOR_PAIR_MASK);
+                              GUI_COLOR_FG_CHAR,
+                              GUI_COLOR_EXTENDED_CHAR,
+                              str_attr,
+                              CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_FG]) & GUI_COLOR_EXTENDED_MASK);
                 }
                 else
                 {
                     snprintf (bar_color, sizeof (bar_color),
-                              "%c%c%02d",
+                              "%c%c%s%02d",
                               GUI_COLOR_COLOR_CHAR,
                               GUI_COLOR_FG_CHAR,
-                              CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_FG]));
+                              str_attr,
+                              CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_FG]) & GUI_COLOR_EXTENDED_MASK);
+                }
+                
+                /* color for bar delimiters */
+                gui_color_attr_build_string (CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_DELIM]),
+                                             str_attr);
+                if (CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_DELIM]) & GUI_COLOR_EXTENDED_FLAG)
+                {
+                    snprintf (delimiter_color, sizeof (delimiter_color),
+                              "%c%c%c%s%05d",
+                              GUI_COLOR_COLOR_CHAR,
+                              GUI_COLOR_FG_CHAR,
+                              GUI_COLOR_EXTENDED_CHAR,
+                              str_attr,
+                              CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_DELIM]) & GUI_COLOR_EXTENDED_MASK);
+                }
+                else
+                {
+                    snprintf (delimiter_color, sizeof (delimiter_color),
+                              "%c%c%s%02d",
+                              GUI_COLOR_COLOR_CHAR,
+                              GUI_COLOR_FG_CHAR,
+                              str_attr,
+                              CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_DELIM]) & GUI_COLOR_EXTENDED_MASK);
                 }
             }
             snprintf (result, length,
@@ -1000,12 +1013,12 @@ gui_bar_item_default_scroll (void *data, struct t_gui_bar_item *item,
     if (!window)
         window = gui_current_window;
     
-    if (!window->scroll)
+    if (!window->scroll->scrolling)
         return NULL;
     
     snprintf (buf, sizeof (buf), _("%s-MORE(%d)-"),
               gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(config_color_status_more))),
-              window->scroll_lines_after);
+              window->scroll->lines_after);
     
     return strdup (buf);
 }
@@ -1021,6 +1034,7 @@ gui_bar_item_default_hotlist (void *data, struct t_gui_bar_item *item,
     char buf[2048], format[32], *buffer_without_name_displayed;
     struct t_gui_hotlist *ptr_hotlist;
     int numbers_count, names_count, display_name;
+    int priority, priority_min, priority_min_displayed, private;
     
     /* make C compiler happy */
     (void) data;
@@ -1032,6 +1046,7 @@ gui_bar_item_default_hotlist (void *data, struct t_gui_bar_item *item,
     
     buf[0] = '\0';
     
+    /* TRANSLATORS: if possible use short word or abbreviation here ("Act" is abbreviation of "Activity" in english) */
     strcat (buf, _("Act: "));
     
     buffer_without_name_displayed = NULL;
@@ -1075,10 +1090,12 @@ gui_bar_item_default_hotlist (void *data, struct t_gui_bar_item *item,
         if (display_name || !buffer_without_name_displayed
             || (buffer_without_name_displayed[ptr_hotlist->buffer->number - 1] == 0))
         {
-            if (numbers_count > 0)
+            if ((numbers_count > 0)
+                && (CONFIG_STRING(config_look_hotlist_buffer_separator))
+                && (CONFIG_STRING(config_look_hotlist_buffer_separator)[0]))
             {
                 strcat (buf, GUI_COLOR_CUSTOM_BAR_DELIM);
-                strcat (buf, ",");
+                strcat (buf, CONFIG_STRING(config_look_hotlist_buffer_separator));
             }
             
             switch (ptr_hotlist->priority)
@@ -1095,6 +1112,12 @@ gui_bar_item_default_hotlist (void *data, struct t_gui_bar_item *item,
                 case GUI_HOTLIST_HIGHLIGHT:
                     strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(config_color_status_data_highlight))));
                     break;
+                case GUI_HOTLIST_NUM_PRIORITIES:
+                    /*
+                     * this constant is used to count hotlist priorities only,
+                     * it is never used as priority
+                     */
+                    break;
             }
             sprintf (buf + strlen (buf), "%d", ptr_hotlist->buffer->number);
             numbers_count++;
@@ -1122,6 +1145,71 @@ gui_bar_item_default_hotlist (void *data, struct t_gui_bar_item *item,
                     buffer_without_name_displayed[ptr_hotlist->buffer->number - 1] = 1;
             }
             
+            /* display messages count by priority */
+            if (CONFIG_INTEGER(config_look_hotlist_count_max) > 0)
+            {
+                private = (ptr_hotlist->count[GUI_HOTLIST_PRIVATE] > 0) ? 1 : 0;
+                priority_min_displayed = ptr_hotlist->priority + 1;
+                priority_min = ptr_hotlist->priority - CONFIG_INTEGER(config_look_hotlist_count_max);
+                if (priority_min < 0)
+                    priority_min = 0;
+                for (priority = ptr_hotlist->priority;
+                     priority >= priority_min;
+                     priority--)
+                {
+                    if (!private && (priority == GUI_HOTLIST_PRIVATE))
+                        continue;
+                    if (private && (priority == GUI_HOTLIST_MESSAGE))
+                        continue;
+                    if (((priority == (int)ptr_hotlist->priority)
+                         && (ptr_hotlist->count[priority] >= CONFIG_INTEGER(config_look_hotlist_count_min_msg)))
+                        || ((priority != (int)ptr_hotlist->priority)
+                            && (ptr_hotlist->count[priority] > 0)))
+                    {
+                        priority_min_displayed = priority;
+                    }
+                }
+                if (priority_min_displayed <= (int)ptr_hotlist->priority)
+                {
+                    for (priority = ptr_hotlist->priority;
+                         priority >= priority_min_displayed;
+                         priority--)
+                    {
+                        if (!private && (priority == GUI_HOTLIST_PRIVATE))
+                            continue;
+                        if (private && (priority == GUI_HOTLIST_MESSAGE))
+                            continue;
+                        strcat (buf, GUI_COLOR_CUSTOM_BAR_DELIM);
+                        strcat (buf, (priority == (int)ptr_hotlist->priority) ? "(" : ",");
+                        switch (priority)
+                        {
+                            case GUI_HOTLIST_LOW:
+                                strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(config_color_status_count_other))));
+                                break;
+                            case GUI_HOTLIST_MESSAGE:
+                                strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(config_color_status_count_msg))));
+                                break;
+                            case GUI_HOTLIST_PRIVATE:
+                                strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(config_color_status_count_private))));
+                                break;
+                            case GUI_HOTLIST_HIGHLIGHT:
+                                strcat (buf, gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(config_color_status_count_highlight))));
+                                break;
+                            case GUI_HOTLIST_NUM_PRIORITIES:
+                                /*
+                                 * this constant is used to count hotlist priorities only,
+                                 * it is never used as priority
+                                 */
+                                break;
+                        }
+                        sprintf (buf + strlen (buf),
+                                 "%d", ptr_hotlist->count[priority]);
+                    }
+                    strcat (buf, GUI_COLOR_CUSTOM_BAR_DELIM);
+                    strcat (buf, ")");
+                }
+            }
+            
             if (strlen (buf) > sizeof (buf) - 64)
                 break;
         }
