@@ -84,6 +84,54 @@ gui_color_search_config (const char *color_name)
     return NULL;
 }
 
+/*
+ * gui_color_attr_get_flag: return flag for attribute char of a color
+ *                          If char is unknown, 0 is returned
+ */
+
+int
+gui_color_attr_get_flag (char c)
+{
+    if (c == GUI_COLOR_EXTENDED_BOLD_CHAR)
+        return GUI_COLOR_EXTENDED_BOLD_FLAG;
+    
+    if (c == GUI_COLOR_EXTENDED_REVERSE_CHAR)
+        return GUI_COLOR_EXTENDED_REVERSE_FLAG;
+    
+    if (c == GUI_COLOR_EXTENDED_ITALIC_CHAR)
+        return GUI_COLOR_EXTENDED_ITALIC_FLAG;
+    
+    if (c == GUI_COLOR_EXTENDED_UNDERLINE_CHAR)
+        return GUI_COLOR_EXTENDED_UNDERLINE_FLAG;
+    
+    return 0;
+}
+
+/*
+ * gui_color_attr_build_string: build string with attributes of color
+ *                              The str_attr must be at least 5 bytes long
+ *                              (4 for attributes + final '\0')
+ */
+
+void
+gui_color_attr_build_string (int color, char *str_attr)
+{
+    int i;
+    
+    i = 0;
+    
+    if (color & GUI_COLOR_EXTENDED_BOLD_FLAG)
+        str_attr[i++] = GUI_COLOR_EXTENDED_BOLD_CHAR;
+    if (color & GUI_COLOR_EXTENDED_REVERSE_FLAG)
+        str_attr[i++] = GUI_COLOR_EXTENDED_REVERSE_CHAR;
+    if (color & GUI_COLOR_EXTENDED_ITALIC_FLAG)
+        str_attr[i++] = GUI_COLOR_EXTENDED_ITALIC_CHAR;
+    if (color & GUI_COLOR_EXTENDED_UNDERLINE_FLAG)
+        str_attr[i++] = GUI_COLOR_EXTENDED_UNDERLINE_CHAR;
+    
+    str_attr[i] = '\0';
+}
+
 /*
  * gui_color_get_custom: get a custom color with a name (GUI dependent)
  */
@@ -91,10 +139,12 @@ gui_color_search_config (const char *color_name)
 const char *
 gui_color_get_custom (const char *color_name)
 {
-    int fg, bg, pair;
-    static char color[32][16];
+    int fg, bg, fg_term, bg_term, term_color;
+    static char color[32][32];
     static int index_color = 0;
-    char *pos_comma, *str_fg, *pos_bg, *error;
+    char color_fg[32], color_bg[32];
+    char *pos_delim, *str_fg, *pos_bg, *error, *color_attr;
+    const char *ptr_color_name;
     
     /* attribute or other color name (GUI dependent) */
     index_color = (index_color + 1) % 32;
@@ -106,64 +156,64 @@ gui_color_get_custom (const char *color_name)
     if (string_strcasecmp (color_name, "reset") == 0)
     {
         snprintf (color[index_color], sizeof (color[index_color]),
-                  "%s",
-                  GUI_COLOR_RESET_STR);
+                  "%c",
+                  GUI_COLOR_RESET_CHAR);
     }
     else if (string_strcasecmp (color_name, "bold") == 0)
     {
         snprintf (color[index_color], sizeof (color[index_color]),
-                  "%s%s",
-                  GUI_COLOR_SET_WEECHAT_STR,
-                  GUI_COLOR_ATTR_BOLD_STR);
+                  "%c%c",
+                  GUI_COLOR_SET_ATTR_CHAR,
+                  GUI_COLOR_ATTR_BOLD_CHAR);
     }
     else if (string_strcasecmp (color_name, "-bold") == 0)
     {
         snprintf (color[index_color], sizeof (color[index_color]),
-                  "%s%s",
-                  GUI_COLOR_REMOVE_WEECHAT_STR,
-                  GUI_COLOR_ATTR_BOLD_STR);
+                  "%c%c",
+                  GUI_COLOR_REMOVE_ATTR_CHAR,
+                  GUI_COLOR_ATTR_BOLD_CHAR);
     }
     else if (string_strcasecmp (color_name, "reverse") == 0)
     {
         snprintf (color[index_color], sizeof (color[index_color]),
-                  "%s%s",
-                  GUI_COLOR_SET_WEECHAT_STR,
-                  GUI_COLOR_ATTR_REVERSE_STR);
+                  "%c%c",
+                  GUI_COLOR_SET_ATTR_CHAR,
+                  GUI_COLOR_ATTR_REVERSE_CHAR);
     }
     else if (string_strcasecmp (color_name, "-reverse") == 0)
     {
         snprintf (color[index_color], sizeof (color[index_color]),
-                  "%s%s",
-                  GUI_COLOR_REMOVE_WEECHAT_STR,
-                  GUI_COLOR_ATTR_REVERSE_STR);
+                  "%c%c",
+                  GUI_COLOR_REMOVE_ATTR_CHAR,
+                  GUI_COLOR_ATTR_REVERSE_CHAR);
     }
     else if (string_strcasecmp (color_name, "italic") == 0)
     {
         snprintf (color[index_color], sizeof (color[index_color]),
-                  "%s%s",
-                  GUI_COLOR_SET_WEECHAT_STR,
-                  GUI_COLOR_ATTR_ITALIC_STR);
+                  "%c%c",
+                  GUI_COLOR_SET_ATTR_CHAR,
+                  GUI_COLOR_ATTR_ITALIC_CHAR);
     }
     else if (string_strcasecmp (color_name, "-italic") == 0)
     {
         snprintf (color[index_color], sizeof (color[index_color]),
-                  "%s%s",
-                  GUI_COLOR_REMOVE_WEECHAT_STR,
-                  GUI_COLOR_ATTR_ITALIC_STR);
+                  "%c%c",
+                  GUI_COLOR_REMOVE_ATTR_CHAR,
+                  GUI_COLOR_ATTR_ITALIC_CHAR);
     }
     else if (string_strcasecmp (color_name, "underline") == 0)
     {
         snprintf (color[index_color], sizeof (color[index_color]),
-                  "%s%s",
-                  GUI_COLOR_SET_WEECHAT_STR,
-                  GUI_COLOR_ATTR_UNDERLINE_STR);
+                  "%c%c",
+                  GUI_COLOR_SET_ATTR_CHAR,
+                  GUI_COLOR_ATTR_UNDERLINE_CHAR);
     }
     else if (string_strcasecmp (color_name, "-underline") == 0)
     {
         snprintf (color[index_color], sizeof (color[index_color]),
-                  "%s%s",
-                  GUI_COLOR_REMOVE_WEECHAT_STR,
-                  GUI_COLOR_ATTR_UNDERLINE_STR);
+                  "%c%c",
+                  GUI_COLOR_REMOVE_ATTR_CHAR,
+                  GUI_COLOR_ATTR_UNDERLINE_CHAR);
     }
     else if (string_strcasecmp (color_name, "bar_fg") == 0)
     {
@@ -192,86 +242,139 @@ gui_color_get_custom (const char *color_name)
     else
     {
         /* custom color name (GUI dependent) */
-        pair = gui_color_palette_get_alias (color_name);
-        if (pair >= 0)
+        fg_term = -1;
+        bg_term = -1;
+        fg = -1;
+        bg = -1;
+        color_attr = NULL;
+        color_fg[0] = '\0';
+        color_bg[0] = '\0';
+        
+        /* read extra attributes (bold, ..) */
+        ptr_color_name = color_name;
+        while (gui_color_attr_get_flag (ptr_color_name[0]) > 0)
         {
-            snprintf (color[index_color], sizeof (color[index_color]),
-                      "%s%s%05d",
-                      GUI_COLOR_COLOR_STR,
-                      GUI_COLOR_PAIR_STR,
-                      pair);
+            ptr_color_name++;
+        }
+        if (ptr_color_name != color_name)
+        {
+            color_attr = string_strndup (color_name,
+                                         ptr_color_name - color_name);
+        }
+        
+        pos_delim = strchr (ptr_color_name, ',');
+        if (!pos_delim)
+            pos_delim = strchr (ptr_color_name, ':');
+        if (pos_delim)
+        {
+            if (pos_delim == ptr_color_name)
+                str_fg = NULL;
+            else
+                str_fg = string_strndup (ptr_color_name,
+                                         pos_delim - ptr_color_name);
+            pos_bg = pos_delim + 1;
         }
         else
         {
-            error = NULL;
-            pair = (int)strtol (color_name, &error, 10);
-            if (error && !error[0])
+            str_fg = strdup (ptr_color_name);
+            pos_bg = NULL;
+        }
+        
+        if (str_fg)
+        {
+            fg_term = gui_color_palette_get_alias (str_fg);
+            if (fg_term < 0)
             {
-                snprintf (color[index_color], sizeof (color[index_color]),
-                          "%s%s%05d",
-                          GUI_COLOR_COLOR_STR,
-                          GUI_COLOR_PAIR_STR,
-                          pair);
-            }
-            else
-            {
-                pos_comma = strchr (color_name, ',');
-                if (pos_comma)
+                error = NULL;
+                term_color = (int)strtol (str_fg, &error, 10);
+                if (error && !error[0])
                 {
-                    if (pos_comma == color_name)
-                        str_fg = NULL;
-                    else
-                        str_fg = string_strndup (color_name, pos_comma - color_name);
-                    pos_bg = pos_comma + 1;
+                    fg_term = term_color;
+                    if (fg_term < 0)
+                        fg_term = 0;
+                    else if (fg_term > GUI_COLOR_EXTENDED_MAX)
+                        fg_term = GUI_COLOR_EXTENDED_MAX;
                 }
                 else
-                {
-                    str_fg = strdup (color_name);
-                    pos_bg = NULL;
-                }
-                
-                if (str_fg && pos_bg)
-                {
                     fg = gui_color_search (str_fg);
-                    bg = gui_color_search (pos_bg);
-                    if ((fg >= 0) && (bg >= 0))
-                    {
-                        snprintf (color[index_color], sizeof (color[index_color]),
-                                  "%s%s%02d,%02d",
-                                  GUI_COLOR_COLOR_STR,
-                                  GUI_COLOR_FG_BG_STR,
-                                  fg, bg);
-                    }
-                }
-                else if (str_fg && !pos_bg)
-                {
-                    fg = gui_color_search (str_fg);
-                    if (fg >= 0)
-                    {
-                        snprintf (color[index_color], sizeof (color[index_color]),
-                                  "%s%s%02d",
-                                  GUI_COLOR_COLOR_STR,
-                                  GUI_COLOR_FG_STR,
-                                  fg);
-                    }
-                }
-                else if (!str_fg && pos_bg)
-                {
-                    bg = gui_color_search (pos_bg);
-                    if (bg >= 0)
-                    {
-                        snprintf (color[index_color], sizeof (color[index_color]),
-                                  "%s%s%02d",
-                                  GUI_COLOR_COLOR_STR,
-                                  GUI_COLOR_BG_STR,
-                                  bg);
-                    }
-                }
-                
-                if (str_fg)
-                    free (str_fg);
             }
         }
+        if (pos_bg)
+        {
+            bg_term = gui_color_palette_get_alias (pos_bg);
+            if (bg_term < 0)
+            {
+                error = NULL;
+                term_color = (int)strtol (pos_bg, &error, 10);
+                if (error && !error[0])
+                {
+                    bg_term = term_color;
+                    if (bg_term < 0)
+                        bg_term = 0;
+                    else if (bg_term > GUI_COLOR_EXTENDED_MAX)
+                        bg_term = GUI_COLOR_EXTENDED_MAX;
+                }
+                else
+                    bg = gui_color_search (pos_bg);
+            }
+        }
+        
+        if (fg_term >= 0)
+        {
+            snprintf (color_fg, sizeof (color_fg), "%c%s%05d",
+                      GUI_COLOR_EXTENDED_CHAR,
+                      (color_attr) ? color_attr : "",
+                      fg_term);
+        }
+        else if (fg >= 0)
+        {
+            snprintf (color_fg, sizeof (color_fg), "%s%02d",
+                      (color_attr) ? color_attr : "",
+                      fg);
+        }
+        
+        if (bg_term >= 0)
+        {
+            snprintf (color_bg, sizeof (color_bg), "%c%05d",
+                      GUI_COLOR_EXTENDED_CHAR,
+                      bg_term);
+        }
+        else if (bg >= 0)
+        {
+            snprintf (color_bg, sizeof (color_bg), "%02d",
+                      bg);
+        }
+
+        if (color_fg[0] && color_bg[0])
+        {
+            snprintf (color[index_color], sizeof (color[index_color]),
+                      "%c%c%s,%s",
+                      GUI_COLOR_COLOR_CHAR,
+                      GUI_COLOR_FG_BG_CHAR,
+                      color_fg,
+                      color_bg);
+        }
+        else if (color_fg[0])
+        {
+            snprintf (color[index_color], sizeof (color[index_color]),
+                      "%c%c%s",
+                      GUI_COLOR_COLOR_CHAR,
+                      GUI_COLOR_FG_CHAR,
+                      color_fg);
+        }
+        else if (color_bg[0])
+        {
+            snprintf (color[index_color], sizeof (color[index_color]),
+                      "%c%c%s",
+                      GUI_COLOR_COLOR_CHAR,
+                      GUI_COLOR_BG_CHAR,
+                      color_bg);
+        }
+        
+        if (color_attr)
+            free (color_attr);
+        if (str_fg)
+            free (str_fg);
     }
     
     return color[index_color];
@@ -313,16 +416,90 @@ gui_color_decode (const char *string, const char *replacement)
                 switch (ptr_string[0])
                 {
                     case GUI_COLOR_FG_CHAR:
+                        ptr_string++;
+                        if (ptr_string[0] == GUI_COLOR_EXTENDED_CHAR)
+                        {
+                            ptr_string++;
+                            while (gui_color_attr_get_flag (ptr_string[0]) > 0)
+                            {
+                                ptr_string++;
+                            }
+                            if (ptr_string[0] && ptr_string[1] && ptr_string[2]
+                                && ptr_string[3] && ptr_string[4])
+                            {
+                                ptr_string += 5;
+                            }
+                        }
+                        else
+                        {
+                            while (gui_color_attr_get_flag (ptr_string[0]) > 0)
+                            {
+                                ptr_string++;
+                            }
+                            if (ptr_string[0] && ptr_string[1])
+                                ptr_string += 2;
+                        }
+                        break;
                     case GUI_COLOR_BG_CHAR:
-                        if (ptr_string[1] && ptr_string[2])
-                            ptr_string += 3;
+                        ptr_string++;
+                        if (ptr_string[0] == GUI_COLOR_EXTENDED_CHAR)
+                        {
+                            ptr_string++;
+                            if (ptr_string[0] && ptr_string[1] && ptr_string[2]
+                                && ptr_string[3] && ptr_string[4])
+                            {
+                                ptr_string += 5;
+                            }
+                        }
+                        else
+                        {
+                            if (ptr_string[0] && ptr_string[1])
+                                ptr_string += 2;
+                        }
                         break;
                     case GUI_COLOR_FG_BG_CHAR:
-                        if (ptr_string[1] && ptr_string[2] && (ptr_string[3] == ',')
-                            && ptr_string[4] && ptr_string[5])
-                            ptr_string += 6;
+                        ptr_string++;
+                        if (ptr_string[0] == GUI_COLOR_EXTENDED_CHAR)
+                        {
+                            ptr_string++;
+                            while (gui_color_attr_get_flag (ptr_string[0]) > 0)
+                            {
+                                ptr_string++;
+                            }
+                            if (ptr_string[0] && ptr_string[1] && ptr_string[2]
+                                && ptr_string[3] && ptr_string[4])
+                            {
+                                ptr_string += 5;
+                            }
+                        }
+                        else
+                        {
+                            while (gui_color_attr_get_flag (ptr_string[0]) > 0)
+                            {
+                                ptr_string++;
+                            }
+                            if (ptr_string[0] && ptr_string[1])
+                                ptr_string += 2;
+                        }
+                        if (ptr_string[0] == ',')
+                        {
+                            if (ptr_string[1] == GUI_COLOR_EXTENDED_CHAR)
+                            {
+                                if (ptr_string[2] && ptr_string[3]
+                                    && ptr_string[4] && ptr_string[5]
+                                    && ptr_string[6])
+                                {
+                                    ptr_string += 7;
+                                }
+                            }
+                            else
+                            {
+                                if (ptr_string[1] && ptr_string[2])
+                                    ptr_string += 3;
+                            }
+                        }
                         break;
-                    case GUI_COLOR_PAIR_CHAR:
+                    case GUI_COLOR_EXTENDED_CHAR:
                         if ((isdigit (ptr_string[1])) && (isdigit (ptr_string[2]))
                             && (isdigit (ptr_string[3])) && (isdigit (ptr_string[4]))
                             && (isdigit (ptr_string[5])))
@@ -353,8 +530,8 @@ gui_color_decode (const char *string, const char *replacement)
                     out_pos++;
                 }
                 break;
-            case GUI_COLOR_SET_WEECHAT_CHAR:
-            case GUI_COLOR_REMOVE_WEECHAT_CHAR:
+            case GUI_COLOR_SET_ATTR_CHAR:
+            case GUI_COLOR_REMOVE_ATTR_CHAR:
                 ptr_string++;
                 if (ptr_string[0])
                     ptr_string++;
@@ -582,17 +759,14 @@ gui_color_palette_add (int number, const char *value)
     new_color_palette = gui_color_palette_new (number, value);
     if (!new_color_palette)
         return;
-
+    
     snprintf (str_number, sizeof (str_number), "%d", number);
     hashtable_set (gui_color_hash_palette_color,
                    str_number, new_color_palette);
     gui_color_palette_build_aliases ();
     
     if (gui_init_ok)
-    {
-        gui_color_init_pair (number);
         gui_color_buffer_display ();
-    }
 }
 
 /*
@@ -614,11 +788,9 @@ gui_color_palette_remove (int number)
     {
         hashtable_remove (gui_color_hash_palette_color, str_number);
         gui_color_palette_build_aliases ();
+        
         if (gui_init_ok)
-        {
-            gui_color_init_pair (number);
             gui_color_buffer_display ();
-        }
     }
 }
 
