@@ -69,74 +69,65 @@ enum t_gui_color_enum
 /* WeeChat internal color attributes (should never be in protocol messages) */
 
 #define GUI_COLOR_COLOR_CHAR           '\x19'
-#define GUI_COLOR_COLOR_STR            "\x19"
-#define GUI_COLOR_SET_WEECHAT_CHAR     '\x1A'
-#define GUI_COLOR_SET_WEECHAT_STR      "\x1A"
-#define GUI_COLOR_REMOVE_WEECHAT_CHAR  '\x1B'
-#define GUI_COLOR_REMOVE_WEECHAT_STR   "\x1B"
+#define GUI_COLOR_SET_ATTR_CHAR        '\x1A'
+#define GUI_COLOR_REMOVE_ATTR_CHAR     '\x1B'
 #define GUI_COLOR_RESET_CHAR           '\x1C'
-#define GUI_COLOR_RESET_STR            "\x1C"
 
 #define GUI_COLOR_ATTR_BOLD_CHAR       '\x01'
-#define GUI_COLOR_ATTR_BOLD_STR        "\x01"
 #define GUI_COLOR_ATTR_REVERSE_CHAR    '\x02'
-#define GUI_COLOR_ATTR_REVERSE_STR     "\x02"
 #define GUI_COLOR_ATTR_ITALIC_CHAR     '\x03'
-#define GUI_COLOR_ATTR_ITALIC_STR      "\x03"
 #define GUI_COLOR_ATTR_UNDERLINE_CHAR  '\x04'
-#define GUI_COLOR_ATTR_UNDERLINE_STR   "\x04"
 
 #define GUI_COLOR(color) ((gui_color[color]) ? gui_color[color]->string : "")
-#define GUI_NO_COLOR     GUI_COLOR_RESET_STR
+#define GUI_NO_COLOR     "\x1C"
 
 #define GUI_COLOR_CUSTOM_BAR_FG    (gui_color_get_custom ("bar_fg"))
 #define GUI_COLOR_CUSTOM_BAR_DELIM (gui_color_get_custom ("bar_delim"))
 #define GUI_COLOR_CUSTOM_BAR_BG    (gui_color_get_custom ("bar_bg"))
 
+/* color codes for chat and bars */
 #define GUI_COLOR_FG_CHAR                     'F'
-#define GUI_COLOR_FG_STR                      "F"
 #define GUI_COLOR_BG_CHAR                     'B'
-#define GUI_COLOR_BG_STR                      "B"
 #define GUI_COLOR_FG_BG_CHAR                  '*'
-#define GUI_COLOR_FG_BG_STR                   "*"
-#define GUI_COLOR_PAIR_CHAR                   '@'
-#define GUI_COLOR_PAIR_STR                    "@"
-#define GUI_COLOR_BAR_CHAR                    'b'
-#define GUI_COLOR_BAR_STR                     "b"
-#define GUI_COLOR_BAR_FG_CHAR                 'F'
-#define GUI_COLOR_BAR_FG_STR                  "F"
-#define GUI_COLOR_BAR_DELIM_CHAR              'D'
-#define GUI_COLOR_BAR_DELIM_STR               "D"
-#define GUI_COLOR_BAR_BG_CHAR                 'B'
-#define GUI_COLOR_BAR_BG_STR                  "B"
-#define GUI_COLOR_BAR_START_INPUT_CHAR        '_'
-#define GUI_COLOR_BAR_START_INPUT_STR         "_"
-#define GUI_COLOR_BAR_START_INPUT_HIDDEN_CHAR '-'
-#define GUI_COLOR_BAR_START_INPUT_HIDDEN_STR  "-"
-#define GUI_COLOR_BAR_MOVE_CURSOR_CHAR        '#'
-#define GUI_COLOR_BAR_MOVE_CURSOR_STR         "#"
+#define GUI_COLOR_EXTENDED_CHAR               '@'
+#define GUI_COLOR_EXTENDED_BOLD_CHAR          '*'
+#define GUI_COLOR_EXTENDED_REVERSE_CHAR       '!'
+#define GUI_COLOR_EXTENDED_ITALIC_CHAR        '/'
+#define GUI_COLOR_EXTENDED_UNDERLINE_CHAR     '_'
 
-#define GUI_COLOR_PAIR_FLAG 0x10000
-#define GUI_COLOR_PAIR_MASK 0xFFFF
+/* color codes specific to bars */
+#define GUI_COLOR_BAR_CHAR                    'b'
+#define GUI_COLOR_BAR_FG_CHAR                 'F'
+#define GUI_COLOR_BAR_DELIM_CHAR              'D'
+#define GUI_COLOR_BAR_BG_CHAR                 'B'
+#define GUI_COLOR_BAR_START_INPUT_CHAR        '_'
+#define GUI_COLOR_BAR_START_INPUT_HIDDEN_CHAR '-'
+#define GUI_COLOR_BAR_MOVE_CURSOR_CHAR        '#'
+
+#define GUI_COLOR_EXTENDED_FLAG               0x0100000
+#define GUI_COLOR_EXTENDED_BOLD_FLAG          0x0200000
+#define GUI_COLOR_EXTENDED_REVERSE_FLAG       0x0400000
+#define GUI_COLOR_EXTENDED_ITALIC_FLAG        0x0800000
+#define GUI_COLOR_EXTENDED_UNDERLINE_FLAG     0x1000000
+#define GUI_COLOR_EXTENDED_MASK               0x00FFFFF
+#define GUI_COLOR_EXTENDED_MAX                99999
 
 /* color structure */
 
 struct t_gui_color
 {
-    int foreground;                 /* foreground color                     */
-    int background;                 /* background color                     */
-    int attributes;                 /* attributes (bold, ..)                */
-    char *string;                   /* WeeChat color: "\x19??", ?? is #color*/
+    int foreground;                /* foreground color                      */
+    int background;                /* background color                      */
+    int attributes;                /* attributes (bold, ..)                 */
+    char *string;                  /* WeeChat color: "\x19??", ?? is #color */
 };
 
 /* custom color in palette */
 
 struct t_gui_color_palette
 {
-    char *alias;                    /* alias name for this color pair       */
-    int foreground;                 /* foreground color                     */
-    int background;                 /* background color                     */
-    int r, g, b;                    /* red/green/blue values for color      */
+    char *alias;                   /* alias name for this color pair        */
+    int r, g, b;                   /* red/green/blue values for color       */
 };
 
 /* color variables */
@@ -149,6 +140,8 @@ extern struct t_weelist *gui_color_list_with_alias;
 /* color functions */
 
 extern const char *gui_color_search_config (const char *color_name);
+extern int gui_color_attr_get_flag (char c);
+extern void gui_color_attr_build_string (int color, char *str_attr);
 extern const char *gui_color_get_custom (const char *color_name);
 extern char *gui_color_decode (const char *string, const char *replacement);
 extern char *gui_color_string_replace_colors (const char *string);
@@ -167,7 +160,7 @@ extern int gui_color_assign (int *color, char const *color_name);
 extern int gui_color_assign_by_diff (int *color, const char *color_name,
                                      int diff);
 extern int gui_color_get_weechat_colors_number ();
-extern int gui_color_get_last_pair ();
+extern int gui_color_get_term_colors ();
 extern const char *gui_color_get_name (int num_color);
 extern void gui_color_init_pair (int number);
 extern void gui_color_init_pairs ();
@@ -175,11 +168,13 @@ extern void gui_color_init_weechat ();
 extern void gui_color_display_terminal_colors ();
 extern void gui_color_buffer_display ();
 extern void gui_color_switch_colors ();
+extern void gui_color_reset_pairs ();
 extern void gui_color_buffer_assign ();
 extern void gui_color_buffer_open ();
 extern void gui_color_palette_build_aliases ();
 extern struct t_gui_color_palette *gui_color_palette_new (int number,
                                                           const char *value);
 extern void gui_color_palette_free (struct t_gui_color_palette *color_palette);
+extern void gui_color_dump ();
 
 #endif /* __WEECHAT_GUI_COLOR_H */
