@@ -32,8 +32,9 @@ struct t_gui_filter
 {
     int enabled;                       /* 1 if filter enabled, otherwise 0  */
     char *name;                        /* filter name                       */
-    char *plugin_name;                 /* plugin name                       */
-    char *buffer_name;                 /* name of buffer                    */
+    char *buffer_name;                 /* name of buffer(s)                 */
+    int num_buffers;                   /* number of buffers in list         */
+    char **buffers;                    /* list of buffer names              */
     char *tags;                        /* tags                              */
     int tags_count;                    /* number of tags                    */
     char **tags_array;                 /* array of tags                     */
@@ -52,15 +53,12 @@ extern int gui_filters_enabled;
 
 /* filter functions */
 
-extern int gui_filter_check_line (struct t_gui_buffer *buffer,
-                                  struct t_gui_line *line);
+extern int gui_filter_check_line (struct t_gui_line *line,
+                                  const char *buffer_full_name);
 extern void gui_filter_global_enable ();
 extern void gui_filter_global_disable ();
 extern void gui_filter_enable (struct t_gui_filter *filter);
 extern void gui_filter_disable (struct t_gui_filter *filter);
-extern struct t_gui_filter *gui_filter_search (const char *buffer_name,
-                                               const char *tags,
-                                               const char *regex);
 extern struct t_gui_filter *gui_filter_search_by_name (const char *name);
 extern struct t_gui_filter *gui_filter_new (int enabled,
                                             const char *name,
