@@ -404,7 +404,7 @@ gui_line_get_nick_tag (struct t_gui_line *line)
 int
 gui_line_has_highlight (struct t_gui_line *line)
 {
-    int rc, i;
+    int rc, i, j, no_highlight;
     char *msg_no_color, *highlight_words;
     
     /*
@@ -415,12 +415,27 @@ gui_line_has_highlight (struct t_gui_line *line)
         && (strcmp (line->data->buffer->highlight_words, "-") == 0))
         return 0;
     
-    /* check if highlight is disabled for line */
+    /*
+     * check if highlight is forced by a tag (with option highlight_tags) or
+     * disabled for line
+     */
+    no_highlight = 0;
     for (i = 0; i < line->data->tags_count; i++)
     {
+        if (config_highlight_tags)
+        {
+            for (j = 0; j < config_num_highlight_tags; j++)
+            {
+                if (string_strcasecmp (line->data->tags_array[i],
+                                       config_highlight_tags[j]) == 0)
+                    return 1;
+            }
+        }
         if (strcmp (line->data->tags_array[i], GUI_CHAT_TAG_NO_HIGHLIGHT) == 0)
-            return 0;
+            no_highlight = 1;
     }
+    if (no_highlight)
+        return 0;
     
     /*
      * check that line matches highlight tags, if any (if no tag is specified,
@@ -550,16 +565,21 @@ gui_line_remove_from_list (struct t_gui_buffer *buffer,
                            int free_data)
 {
     struct t_gui_window *ptr_win;
+    struct t_gui_window_scroll *ptr_scroll;
     int update_prefix_max_length;
     
-    /* reset scroll for any window starting with this line */
+    /* reset scroll for any window scroll starting with this line */
     for (ptr_win = gui_windows; ptr_win; ptr_win = ptr_win->next_window)
     {
-        if (ptr_win->start_line == line)
+        for (ptr_scroll = ptr_win->scroll; ptr_scroll;
+             ptr_scroll = ptr_scroll->next_scroll)
         {
-            ptr_win->start_line = ptr_win->start_line->next_line;
-            ptr_win->start_line_pos = 0;
-            gui_buffer_ask_chat_refresh (buffer, 2);
+            if (ptr_scroll->start_line == line)
+            {
+                ptr_scroll->start_line = ptr_scroll->start_line->next_line;
+                ptr_scroll->start_line_pos = 0;
+                gui_buffer_ask_chat_refresh (buffer, 2);
+            }
         }
     }
     
@@ -718,6 +738,8 @@ gui_line_free_all (struct t_gui_buffer *buffer)
 
 /*
  * gui_line_get_notify_level: get notify level for a line
+ *                            return -1 if "notify_none" is found (no notify
+ *                            for line)
  */
 
 int
@@ -727,6 +749,8 @@ gui_line_get_notify_level (struct t_gui_line *line)
     
     for (i = 0; i < line->data->tags_count; i++)
     {
+        if (string_strcasecmp (line->data->tags_array[i], "notify_none") == 0)
+            return -1;
         if (string_strcasecmp (line->data->tags_array[i], "notify_highlight") == 0)
             return GUI_HOTLIST_HIGHLIGHT;
         if (string_strcasecmp (line->data->tags_array[i], "notify_private") == 0)
@@ -749,7 +773,7 @@ gui_line_add (struct t_gui_buffer *buffer, time_t date,
     struct t_gui_line *new_line;
     struct t_gui_line_data *new_line_data;
     struct t_gui_window *ptr_win;
-    char *message_for_signal;
+    char *message_for_signal, buffer_full_name[512];
     const char *nick;
     int notify_level, *max_notify_level;
     
@@ -814,12 +838,17 @@ gui_line_add (struct t_gui_buffer *buffer, time_t date,
     gui_line_add_to_list (buffer->own_lines, new_line);
     
     /* check if line is filtered or not */
-    new_line->data->displayed = gui_filter_check_line (buffer, new_line);
+    snprintf (buffer_full_name, sizeof (buffer_full_name), "%s.%s",
+              (!buffer->plugin && buffer->plugin_name_for_upgrade) ?
+              buffer->plugin_name_for_upgrade : plugin_get_name (buffer->plugin),
+              buffer->name);
+    new_line->data->displayed = gui_filter_check_line (new_line,
+                                                       buffer_full_name);
     if (new_line->data->displayed)
     {
         if (new_line->data->highlight)
         {
-            gui_hotlist_add (buffer, GUI_HOTLIST_HIGHLIGHT, NULL, 1);
+            (void) gui_hotlist_add (buffer, GUI_HOTLIST_HIGHLIGHT, NULL);
             if (!weechat_upgrading)
             {
                 message_for_signal = gui_chat_build_string_prefix_message (new_line);
@@ -846,7 +875,7 @@ gui_line_add (struct t_gui_buffer *buffer, time_t date,
                 }
             }
             if (notify_level >= GUI_HOTLIST_MIN)
-                gui_hotlist_add (buffer, notify_level, NULL, 1);
+                (void) gui_hotlist_add (buffer, notify_level, NULL);
         }
     }
     else
@@ -895,6 +924,7 @@ gui_line_add_y (struct t_gui_buffer *buffer, int y, const char *message)
 {
     struct t_gui_line *ptr_line, *new_line;
     struct t_gui_line_data *new_line_data;
+    char buffer_full_name[512];
     
     /* search if line exists for "y" */
     for (ptr_line = buffer->own_lines->first_line; ptr_line;
@@ -970,7 +1000,12 @@ gui_line_add_y (struct t_gui_buffer *buffer, int y, const char *message)
     ptr_line->data->message = (message) ? strdup (message) : strdup ("");
     
     /* check if line is filtered or not */
-    ptr_line->data->displayed = gui_filter_check_line (buffer, ptr_line);
+    snprintf (buffer_full_name, sizeof (buffer_full_name), "%s.%s",
+              (!buffer->plugin && buffer->plugin_name_for_upgrade) ?
+              buffer->plugin_name_for_upgrade : plugin_get_name (buffer->plugin),
+              buffer->name);
+    ptr_line->data->displayed = gui_filter_check_line (ptr_line,
+                                                       buffer_full_name);
     if (!ptr_line->data->displayed)
     {
         if (!buffer->own_lines->lines_hidden)
