@@ -1,23 +1,25 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* gui-nicklist.c: nicklist functions, used by all GUI */
-
+/*
+ * gui-nicklist.c: nicklist functions (used by all GUI)
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -45,13 +47,36 @@
 
 
 /*
- * gui_nicklist_changed_signal: send "nicklist_changed" signal
+ * gui_nicklist_send_signal: send a signal when something has changed in
+ *                           nicklist
  */
 
 void
-gui_nicklist_changed_signal ()
+gui_nicklist_send_signal (const char *signal, struct t_gui_buffer *buffer,
+                          const char *arguments)
 {
-    hook_signal_send ("nicklist_changed", WEECHAT_HOOK_SIGNAL_STRING, NULL);
+    char *str_args;
+    int length;
+    
+    if (buffer)
+    {
+        length = 128 + ((arguments) ? strlen (arguments) : 0) + 1 + 1;
+        str_args = malloc (length);
+        if (str_args)
+        {
+            snprintf (str_args, length,
+                      "0x%lx,%s",
+                      (long unsigned int)(buffer),
+                      (arguments) ? arguments : "");
+            hook_signal_send (signal, WEECHAT_HOOK_SIGNAL_STRING, str_args);
+            free (str_args);
+        }
+    }
+    else
+    {
+        hook_signal_send (signal, WEECHAT_HOOK_SIGNAL_STRING,
+                          (char *)arguments);
+    }
 }
 
 /*
@@ -119,15 +144,17 @@ gui_nicklist_insert_group_sorted (struct t_gui_nick_group **groups,
 }
 
 /*
- * gui_nicklist_search_group: search a group in buffer nicklist
+ * gui_nicklist_search_group_internal: search a group in buffer nicklist
  */
 
 struct t_gui_nick_group *
-gui_nicklist_search_group (struct t_gui_buffer *buffer,
-                           struct t_gui_nick_group *from_group,
-                           const char *name)
+gui_nicklist_search_group_internal (struct t_gui_buffer *buffer,
+                                    struct t_gui_nick_group *from_group,
+                                    const char *name,
+                                    int skip_digits)
 {
     struct t_gui_nick_group *ptr_group;
+    const char *ptr_name;
     
     if (!buffer)
         return NULL;
@@ -140,7 +167,10 @@ gui_nicklist_search_group (struct t_gui_buffer *buffer,
     
     if (from_group->childs)
     {
-        ptr_group = gui_nicklist_search_group (buffer, from_group->childs, name);
+        ptr_group = gui_nicklist_search_group_internal (buffer,
+                                                        from_group->childs,
+                                                        name,
+                                                        skip_digits);
         if (ptr_group)
             return ptr_group;
     }
@@ -148,7 +178,9 @@ gui_nicklist_search_group (struct t_gui_buffer *buffer,
     ptr_group = from_group;
     while (ptr_group)
     {
-        if (strcmp (ptr_group->name, name) == 0)
+        ptr_name = (skip_digits) ?
+            gui_nicklist_get_group_start(ptr_group->name) : ptr_group->name;
+        if (strcmp (ptr_name, name) == 0)
             return ptr_group;
         ptr_group = ptr_group->next_group;
     }
@@ -157,6 +189,23 @@ gui_nicklist_search_group (struct t_gui_buffer *buffer,
     return NULL;
 }
 
+/*
+ * gui_nicklist_search_group: search a group in buffer nicklist
+ */
+
+struct t_gui_nick_group *
+gui_nicklist_search_group (struct t_gui_buffer *buffer,
+                           struct t_gui_nick_group *from_group,
+                           const char *name)
+{
+    const char *ptr_name;
+    
+    ptr_name = gui_nicklist_get_group_start (name);
+    
+    return gui_nicklist_search_group_internal (buffer, from_group, name,
+                                               (ptr_name == name) ? 1 : 0);
+}
+
 /*
  * gui_nicklist_add_group: add a group to nicklist for a buffer
  */
@@ -201,7 +250,7 @@ gui_nicklist_add_group (struct t_gui_buffer *buffer,
     if (buffer->nicklist_display_groups && visible)
         buffer->nicklist_visible_count++;
     
-    gui_nicklist_changed_signal ();
+    gui_nicklist_send_signal ("nicklist_group_added", buffer, name);
     
     return new_group;
 }
@@ -342,7 +391,7 @@ gui_nicklist_add_nick (struct t_gui_buffer *buffer,
     if (visible)
         buffer->nicklist_visible_count++;
     
-    gui_nicklist_changed_signal ();
+    gui_nicklist_send_signal ("nicklist_nick_added", buffer, name);
     
     return new_nick;
 }
@@ -355,9 +404,13 @@ void
 gui_nicklist_remove_nick (struct t_gui_buffer *buffer,
                           struct t_gui_nick *nick)
 {
+    char *nick_removed;
+    
     if (!buffer || !nick)
         return;
     
+    nick_removed = (nick->name) ? strdup (nick->name) : NULL;
+    
     /* remove nick from list */
     if (nick->prev_nick)
         (nick->prev_nick)->next_nick = nick->next_nick;
@@ -386,7 +439,10 @@ gui_nicklist_remove_nick (struct t_gui_buffer *buffer,
     
     free (nick);
     
-    gui_nicklist_changed_signal ();
+    gui_nicklist_send_signal ("nicklist_nick_removed", buffer, nick_removed);
+    
+    if (nick_removed)
+        free (nick_removed);
 }
 
 /*
@@ -397,9 +453,13 @@ void
 gui_nicklist_remove_group (struct t_gui_buffer *buffer,
                            struct t_gui_nick_group *group)
 {
+    char *group_removed;
+    
     if (!buffer || !group)
         return;
     
+    group_removed = (group->name) ? strdup (group->name) : NULL;
+    
     /* remove childs first */
     while (group->childs)
     {
@@ -444,7 +504,10 @@ gui_nicklist_remove_group (struct t_gui_buffer *buffer,
     
     free (group);
     
-    gui_nicklist_changed_signal ();
+    gui_nicklist_send_signal ("nicklist_group_removed", buffer, group_removed);
+    
+    if (group_removed)
+        free (group_removed);
 }
 
 /*
@@ -544,7 +607,7 @@ gui_nicklist_get_next_item (struct t_gui_buffer *buffer,
  *                               after '|', otherwise it's beginning of name
  */
 
-char *
+const char *
 gui_nicklist_get_group_start (const char *name)
 {
     const char *ptr_name;
@@ -557,9 +620,9 @@ gui_nicklist_get_group_start (const char *name)
         ptr_name++;
     }
     if ((ptr_name[0] == '|') && (ptr_name != name))
-        return (char *)ptr_name + 1;
+        return ptr_name + 1;
     else
-        return (char *)name;
+        return name;
 }
 
 /*
@@ -612,23 +675,344 @@ gui_nicklist_get_max_length (struct t_gui_buffer *buffer,
 }
 
 /*
- * gui_nicklist_compute_visible_count: compute visible_count variable for a buffer
+ * gui_nicklist_compute_visible_count: compute visible_count variable for a
+ *                                     buffer
  */
 
 void
 gui_nicklist_compute_visible_count (struct t_gui_buffer *buffer,
                                     struct t_gui_nick_group *group)
 {
+    struct t_gui_nick_group *ptr_group;
+    struct t_gui_nick *ptr_nick;
+    
     if (!buffer || !group)
         return;
     
     /* count for childs */
-    if (group->childs)
-        gui_nicklist_compute_visible_count (buffer, group->childs);
+    for (ptr_group = group->childs; ptr_group;
+         ptr_group = ptr_group->next_group)
+    {
+        gui_nicklist_compute_visible_count (buffer, ptr_group);
+    }
     
     /* count current group */
     if (buffer->nicklist_display_groups && group->visible)
         buffer->nicklist_visible_count++;
+    
+    /* count nicks in group */
+    for (ptr_nick = group->nicks; ptr_nick; ptr_nick = ptr_nick->next_nick)
+    {
+        buffer->nicklist_visible_count++;
+    }
+}
+
+/*
+ * gui_nicklist_group_get_integer: get a group property as integer
+ */
+
+int
+gui_nicklist_group_get_integer (struct t_gui_buffer *buffer,
+                                struct t_gui_nick_group *group,
+                                const char *property)
+{
+    /* make C compiler happy */
+    (void) buffer;
+    
+    if (group && property)
+    {
+        if (string_strcasecmp (property, "visible") == 0)
+            return group->visible;
+        else if (string_strcasecmp (property, "level") == 0)
+            return group->level;
+    }
+    
+    return 0;
+}
+
+/*
+ * gui_nicklist_group_get_string: get a group property as string
+ */
+
+const char *
+gui_nicklist_group_get_string (struct t_gui_buffer *buffer,
+                               struct t_gui_nick_group *group,
+                               const char *property)
+{
+    /* make C compiler happy */
+    (void) buffer;
+    
+    if (group && property)
+    {
+        if (string_strcasecmp (property, "name") == 0)
+            return group->name;
+        else if (string_strcasecmp (property, "color") == 0)
+            return group->color;
+    }
+    
+    return NULL;
+}
+
+/*
+ * gui_nicklist_group_get_pointer: get a group property as pointer
+ */
+
+void *
+gui_nicklist_group_get_pointer (struct t_gui_buffer *buffer,
+                                struct t_gui_nick_group *group,
+                                const char *property)
+{
+    /* make C compiler happy */
+    (void) buffer;
+    
+    if (group && property)
+    {
+        if (string_strcasecmp (property, "parent") == 0)
+            return group->parent;
+    }
+    
+    return NULL;
+}
+
+/*
+ * gui_nicklist_group_set: set a group property (string)
+ */
+
+void
+gui_nicklist_group_set (struct t_gui_buffer *buffer,
+                        struct t_gui_nick_group *group,
+                        const char *property, const char *value)
+{
+    long number;
+    char *error;
+    int group_changed;
+    
+    if (!buffer || !group || !property || !value)
+        return;
+    
+    group_changed = 0;
+    
+    if (string_strcasecmp (property, "color") == 0)
+    {
+        if (group->color)
+            free (group->color);
+        group->color = (value[0]) ? strdup (value) : NULL;
+        group_changed = 1;
+    }
+    else if (string_strcasecmp (property, "visible") == 0)
+    {
+        error = NULL;
+        number = strtol (value, &error, 10);
+        if (error && !error[0])
+            group->visible = (number) ? 1 : 0;
+        group_changed = 1;
+    }
+    
+    if (group_changed)
+    {
+        gui_nicklist_send_signal ("nicklist_group_changed", buffer,
+                                  group->name);
+    }
+}
+
+/*
+ * gui_nicklist_nick_get_integer: get a nick property as integer
+ */
+
+int
+gui_nicklist_nick_get_integer (struct t_gui_buffer *buffer,
+                               struct t_gui_nick *nick,
+                               const char *property)
+{
+    /* make C compiler happy */
+    (void) buffer;
+    
+    if (nick && property)
+    {
+        if (string_strcasecmp (property, "visible") == 0)
+            return nick->visible;
+    }
+    
+    return 0;
+}
+
+/*
+ * gui_nicklist_nick_get_string: get a nick property as string
+ */
+
+const char *
+gui_nicklist_nick_get_string (struct t_gui_buffer *buffer,
+                              struct t_gui_nick *nick,
+                              const char *property)
+{
+    /* make C compiler happy */
+    (void) buffer;
+    
+    if (nick && property)
+    {
+        if (string_strcasecmp (property, "name") == 0)
+            return nick->name;
+        else if (string_strcasecmp (property, "color") == 0)
+            return nick->color;
+        else if (string_strcasecmp (property, "prefix") == 0)
+            return nick->prefix;
+        else if (string_strcasecmp (property, "prefix_color") == 0)
+            return nick->prefix_color;
+    }
+    
+    return NULL;
+}
+
+/*
+ * gui_nicklist_nick_get_pointer: get a nick property as pointer
+ */
+
+void *
+gui_nicklist_nick_get_pointer (struct t_gui_buffer *buffer,
+                               struct t_gui_nick *nick,
+                               const char *property)
+{
+    /* make C compiler happy */
+    (void) buffer;
+    
+    if (nick && property)
+    {
+        if (string_strcasecmp (property, "group") == 0)
+            return nick->group;
+    }
+    
+    return NULL;
+}
+
+/*
+ * gui_nicklist_nick_set: set a nick property (string)
+ */
+
+void
+gui_nicklist_nick_set (struct t_gui_buffer *buffer,
+                       struct t_gui_nick *nick,
+                       const char *property, const char *value)
+{
+    long number;
+    char *error;
+    int nick_changed;
+    
+    if (!buffer || !nick || !property || !value)
+        return;
+    
+    nick_changed = 0;
+    
+    if (string_strcasecmp (property, "color") == 0)
+    {
+        if (nick->color)
+            free (nick->color);
+        nick->color = (value[0]) ? strdup (value) : NULL;
+        nick_changed = 1;
+    }
+    else if (string_strcasecmp (property, "prefix") == 0)
+    {
+        if (nick->prefix)
+            free (nick->prefix);
+        nick->prefix = (value[0]) ? strdup (value) : NULL;
+        nick_changed = 1;
+    }
+    else if (string_strcasecmp (property, "prefix_color") == 0)
+    {
+        if (nick->prefix_color)
+            free (nick->prefix_color);
+        nick->prefix_color = (value[0]) ? strdup (value) : NULL;
+        nick_changed = 1;
+    }
+    else if (string_strcasecmp (property, "visible") == 0)
+    {
+        error = NULL;
+        number = strtol (value, &error, 10);
+        if (error && !error[0])
+            nick->visible = (number) ? 1 : 0;
+        nick_changed = 1;
+    }
+    
+    if (nick_changed)
+    {
+        gui_nicklist_send_signal ("nicklist_nick_changed", buffer,
+                                  nick->name);
+    }
+}
+
+/*
+ * gui_nicklist_add_group_to_infolist: add a group in an infolist
+ *                                     return 1 if ok, 0 if error
+ */
+
+int
+gui_nicklist_add_group_to_infolist (struct t_infolist *infolist,
+                                    struct t_gui_nick_group *group)
+{
+    struct t_infolist_item *ptr_item;
+    
+    if (!infolist || !group)
+        return 0;
+    
+    ptr_item = infolist_new_item (infolist);
+    if (!ptr_item)
+        return 0;
+    
+    if (!infolist_new_var_string (ptr_item, "type", "group"))
+        return 0;
+    if (group->parent)
+    {
+        if (!infolist_new_var_string (ptr_item, "parent_name", group->parent->name))
+            return 0;
+    }
+    if (!infolist_new_var_string (ptr_item, "name", group->name))
+        return 0;
+    if (!infolist_new_var_string (ptr_item, "color", group->color))
+        return 0;
+    if (!infolist_new_var_integer (ptr_item, "visible", group->visible))
+        return 0;
+    if (!infolist_new_var_integer (ptr_item, "level", group->level))
+        return 0;
+    
+    return 1;
+}
+
+/*
+ * gui_nicklist_add_nick_to_infolist: add a nick in an infolist
+ *                                    return 1 if ok, 0 if error
+ */
+
+int
+gui_nicklist_add_nick_to_infolist (struct t_infolist *infolist,
+                                   struct t_gui_nick *nick)
+{
+    struct t_infolist_item *ptr_item;
+    
+    if (!infolist || !nick)
+        return 0;
+    
+    ptr_item = infolist_new_item (infolist);
+    if (!ptr_item)
+        return 0;
+    
+    if (!infolist_new_var_string (ptr_item, "type", "nick"))
+        return 0;
+    if (nick->group)
+    {
+        if (!infolist_new_var_string (ptr_item, "group_name", nick->group->name))
+            return 0;
+    }
+    if (!infolist_new_var_string (ptr_item, "name", nick->name))
+        return 0;
+    if (!infolist_new_var_string (ptr_item, "color", nick->color))
+        return 0;
+    if (!infolist_new_var_string (ptr_item, "prefix", nick->prefix))
+        return 0;
+    if (!infolist_new_var_string (ptr_item, "prefix_color", nick->prefix_color))
+        return 0;
+    if (!infolist_new_var_integer (ptr_item, "visible", nick->visible))
+        return 0;
+    
+    return 1;
 }
 
 /*
@@ -638,62 +1022,43 @@ gui_nicklist_compute_visible_count (struct t_gui_buffer *buffer,
 
 int
 gui_nicklist_add_to_infolist (struct t_infolist *infolist,
-                              struct t_gui_buffer *buffer)
+                              struct t_gui_buffer *buffer,
+                              const char *name)
 {
-    struct t_infolist_item *ptr_item;
     struct t_gui_nick_group *ptr_group;
     struct t_gui_nick *ptr_nick;
     
     if (!infolist || !buffer)
         return 0;
     
+    /* add only one nick if asked */
+    if (name && (strncmp (name, "nick_", 5) == 0))
+    {
+        ptr_nick = gui_nicklist_search_nick (buffer, NULL, name + 5);
+        if (!ptr_nick)
+            return 0;
+        return gui_nicklist_add_nick_to_infolist (infolist, ptr_nick);
+    }
+    
+    /* add only one group if asked */
+    if (name && (strncmp (name, "group_", 6) == 0))
+    {
+        ptr_group = gui_nicklist_search_group (buffer, NULL, name + 6);
+        if (!ptr_group)
+            return 0;
+        return gui_nicklist_add_group_to_infolist (infolist, ptr_group);
+    }
+    
     ptr_group = NULL;
     ptr_nick = NULL;
     gui_nicklist_get_next_item (buffer, &ptr_group, &ptr_nick);
     while (ptr_group || ptr_nick)
     {
-        ptr_item = infolist_new_item (infolist);
-        if (!ptr_item)
-            return 0;
-        
         if (ptr_nick)
-        {
-            if (!infolist_new_var_string (ptr_item, "type", "nick"))
-                return 0;
-            if (ptr_nick->group)
-            {
-                if (!infolist_new_var_string (ptr_item, "group_name", ptr_nick->group->name))
-                    return 0;
-            }
-            if (!infolist_new_var_string (ptr_item, "name", ptr_nick->name))
-                return 0;
-            if (!infolist_new_var_string (ptr_item, "color", ptr_nick->color))
-                return 0;
-            if (!infolist_new_var_string (ptr_item, "prefix", ptr_nick->prefix))
-                return 0;
-            if (!infolist_new_var_string (ptr_item, "prefix_color", ptr_nick->prefix_color))
-                return 0;
-            if (!infolist_new_var_integer (ptr_item, "visible", ptr_nick->visible))
-                return 0;
-        }
+            gui_nicklist_add_nick_to_infolist (infolist, ptr_nick);
         else
-        {
-            if (!infolist_new_var_string (ptr_item, "type", "group"))
-                return 0;
-            if (ptr_group->parent)
-            {
-                if (!infolist_new_var_string (ptr_item, "parent_name", ptr_group->parent->name))
-                    return 0;
-            }
-            if (!infolist_new_var_string (ptr_item, "name", ptr_group->name))
-                return 0;
-            if (!infolist_new_var_string (ptr_item, "color", ptr_group->color))
-                return 0;
-            if (!infolist_new_var_integer (ptr_item, "visible", ptr_group->visible))
-                return 0;
-            if (!infolist_new_var_integer (ptr_item, "level", ptr_group->level))
-                return 0;
-        }
+            gui_nicklist_add_group_to_infolist (infolist, ptr_group);
+        
         gui_nicklist_get_next_item (buffer, &ptr_group, &ptr_nick);
     }
     
@@ -787,7 +1152,7 @@ gui_nicklist_print_log (struct t_gui_nick_group *group, int indent)
                   (indent * 2) + 6);
         log_printf (format, " ", ptr_nick->color);
         snprintf (format, sizeof (format),
-                  "%%-%dsprefix. . . . . : '%%c'",
+                  "%%-%dsprefix. . . . . : '%%s'",
                   (indent * 2) + 6);
         log_printf (format, " ", ptr_nick->prefix);
         snprintf (format, sizeof (format),
