@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 #ifndef __WEECHAT_GUI_NICKLIST_H
 #define __WEECHAT_GUI_NICKLIST_H 1
 
@@ -78,11 +78,39 @@ extern void gui_nicklist_remove_all (struct t_gui_buffer *buffer);
 extern void gui_nicklist_get_next_item (struct t_gui_buffer *buffer,
                                         struct t_gui_nick_group **group,
                                         struct t_gui_nick **nick);
-extern char *gui_nicklist_get_group_start (const char *name);
+extern const char *gui_nicklist_get_group_start (const char *name);
 extern void gui_nicklist_compute_visible_count (struct t_gui_buffer *buffer,
                                                 struct t_gui_nick_group *group);
+
+
+
+extern int gui_nicklist_group_get_integer (struct t_gui_buffer *buffer,
+                                           struct t_gui_nick_group *group,
+                                           const char *property);
+extern const char *gui_nicklist_group_get_string (struct t_gui_buffer *buffer,
+                                                  struct t_gui_nick_group *group,
+                                                  const char *property);
+extern void *gui_nicklist_group_get_pointer (struct t_gui_buffer *buffer,
+                                             struct t_gui_nick_group *group,
+                                             const char *property);
+extern void gui_nicklist_group_set (struct t_gui_buffer *buffer,
+                                    struct t_gui_nick_group *group,
+                                    const char *property, const char *value);
+extern int gui_nicklist_nick_get_integer (struct t_gui_buffer *buffer,
+                                          struct t_gui_nick *nick,
+                                          const char *property);
+extern const char *gui_nicklist_nick_get_string (struct t_gui_buffer *buffer,
+                                                 struct t_gui_nick *nick,
+                                                 const char *property);
+extern void *gui_nicklist_nick_get_pointer (struct t_gui_buffer *buffer,
+                                            struct t_gui_nick *nick,
+                                            const char *property);
+extern void gui_nicklist_nick_set (struct t_gui_buffer *buffer,
+                                   struct t_gui_nick *nick,
+                                   const char *property, const char *value);
 extern int gui_nicklist_add_to_infolist (struct t_infolist *infolist,
-                                         struct t_gui_buffer *buffer);
+                                         struct t_gui_buffer *buffer,
+                                         const char *name);
 extern void gui_nicklist_print_log (struct t_gui_nick_group *group, int indent);
 
-#endif /* gui-nicklist.h */
+#endif /* __WEECHAT_GUI_NICKLIST_H */
