@@ -67,20 +67,31 @@ struct t_gui_window
                                        /* be assigned later)                */
     
     /* scroll */
-    int first_line_displayed;          /* = 1 if first line is displayed    */
-    struct t_gui_line *start_line;     /* pointer to line if scrolling      */
-    int start_line_pos;                /* position in first line displayed  */
-    int scroll;                        /* = 1 if "MORE" should be displayed */
-    int scroll_lines_after;            /* number of lines after last line   */
-                                       /* displayed (with scrolling)        */
-    int scroll_reset_allowed;          /* reset scroll allowed (when using  */
-                                       /* keys like page_up/down, end, ..)  */
+    struct t_gui_window_scroll *scroll; /* scroll infos for each buffer     */
+                                        /* scrolled in this window          */
+
+    /* tree */
     struct t_gui_window_tree *ptr_tree;/* pointer to leaf in windows tree   */
     
     struct t_gui_window *prev_window;  /* link to previous window           */
     struct t_gui_window *next_window;  /* link to next window               */
 };
 
+struct t_gui_window_scroll
+{
+    struct t_gui_buffer *buffer;       /* buffer scrolled in window         */
+    int first_line_displayed;          /* = 1 if first line is displayed    */
+    struct t_gui_line *start_line;     /* pointer to line if scrolling      */
+    int start_line_pos;                /* position in first line displayed  */
+    int scrolling;                     /* = 1 if "MORE" should be displayed */
+    int lines_after;                   /* number of lines after last line   */
+                                       /* displayed (with scrolling)        */
+    int reset_allowed;                 /* reset scroll allowed (when using  */
+                                       /* keys like page_up/down, end, ..)  */
+    struct t_gui_window_scroll *prev_scroll; /* link to prev. buf. scrolled */
+    struct t_gui_window_scroll *next_scroll; /* link to next buf. scrolled  */
+};
+
 struct t_gui_window_tree
 {
     struct t_gui_window_tree *parent_node; /* pointer to parent node        */
@@ -108,11 +119,16 @@ extern int gui_window_tree_init (struct t_gui_window *window);
 extern void gui_window_tree_node_to_leaf (struct t_gui_window_tree *node,
                                           struct t_gui_window *window);
 extern void gui_window_tree_free (struct t_gui_window_tree **tree);
+extern void gui_window_scroll_switch (struct t_gui_window *window,
+                                      struct t_gui_buffer *buffer);
+extern void gui_window_scroll_remove_buffer (struct t_gui_window *window,
+                                             struct t_gui_buffer *buffer);
 extern struct t_gui_window *gui_window_new (struct t_gui_window *parent_window,
                                             struct t_gui_buffer *buffer,
                                             int x, int y, int width, int height,
                                             int width_pct, int height_pct);
 extern int gui_window_valid (struct t_gui_window *window);
+extern struct t_gui_window *gui_window_search_with_buffer (struct t_gui_buffer *buffer);
 extern int gui_window_get_integer (struct t_gui_window *window,
                                    const char *property);
 extern const char *gui_window_get_string (struct t_gui_window *window,
@@ -158,18 +174,21 @@ extern void gui_window_scroll_up (struct t_gui_window *window);
 extern void gui_window_scroll_down (struct t_gui_window *window);
 extern void gui_window_scroll_top (struct t_gui_window *window);
 extern void gui_window_scroll_bottom (struct t_gui_window *window);
-extern void gui_window_refresh_windows ();
 extern struct t_gui_window *gui_window_split_horizontal (struct t_gui_window *window,
                                                          int percentage);
 extern struct t_gui_window *gui_window_split_vertical (struct t_gui_window *window,
                                                        int percentage);
 extern void gui_window_resize (struct t_gui_window *window, int percentage);
+extern void gui_window_resize_delta (struct t_gui_window *window,
+                                     int delta_percentage);
 extern int gui_window_merge (struct t_gui_window *window);
 extern void gui_window_merge_all (struct t_gui_window *window);
 extern void gui_window_switch_up (struct t_gui_window *window);
 extern void gui_window_switch_down (struct t_gui_window *window);
 extern void gui_window_switch_left (struct t_gui_window *window);
 extern void gui_window_switch_right (struct t_gui_window *window);
+extern int gui_window_balance (struct t_gui_window_tree *tree);
+extern void gui_window_swap (struct t_gui_window *window, int direction);
 extern void gui_window_refresh_screen (int full_refresh);
 extern void gui_window_set_title (const char *title);
 extern void gui_window_term_display_infos ();
