@@ -150,8 +150,8 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
 {
     struct t_irc_channel *new_channel;
     struct t_gui_buffer *new_buffer;
-    int i, buffer_created, current_buffer_number, buffer_position;
-    char *buffer_name, str_number[32], str_group[32];
+    int i, buffer_created, current_buffer_number, buffer_position, manual_join;
+    char *buffer_name, str_number[32], str_group[32], *channel_name_lower;
     const char *prefix_modes;
     
     /* alloc memory for new channel */
@@ -260,6 +260,7 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
     new_channel->away_message = NULL;
     new_channel->has_quit_server = 0;
     new_channel->cycle = 0;
+    new_channel->part = 0;
     new_channel->display_creation_date = 0;
     new_channel->nick_completion_reset = 0;
     new_channel->pv_remote_nick_color = NULL;
@@ -283,12 +284,40 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
         server->channels = new_channel;
     server->last_channel = new_channel;
     
+    manual_join = 0;
+    channel_name_lower = NULL;
+    if (channel_type == IRC_CHANNEL_TYPE_CHANNEL)
+    {
+        channel_name_lower = strdup (channel_name);
+        if (channel_name_lower)
+        {
+            weechat_string_tolower (channel_name_lower);
+            manual_join = weechat_hashtable_has_key (server->manual_joins,
+                                                     channel_name_lower);
+        }
+    }
+    
     if (switch_to_channel)
     {
-        weechat_buffer_set (new_buffer, "display",
-                            (auto_switch) ? "auto" : "1");
+        if (channel_type == IRC_CHANNEL_TYPE_CHANNEL)
+        {
+            if ((manual_join && !weechat_config_boolean (irc_config_look_buffer_switch_join))
+                || (!manual_join && !weechat_config_boolean (irc_config_look_buffer_switch_autojoin)))
+                switch_to_channel = 0;
+        }
+        
+        if (switch_to_channel)
+        {
+            weechat_buffer_set (new_buffer, "display",
+                                (auto_switch) ? "auto" : "1");
+        }
     }
     
+    if (manual_join)
+        weechat_hashtable_remove (server->manual_joins, channel_name_lower);
+    if (channel_name_lower)
+        free (channel_name_lower);
+    
     weechat_hook_signal_send ((channel_type == IRC_CHANNEL_TYPE_CHANNEL) ?
                               "irc_channel_opened" : "irc_pv_opened",
                               WEECHAT_HOOK_SIGNAL_POINTER, new_buffer);
@@ -652,7 +681,7 @@ irc_channel_rejoin (struct t_irc_server *server, struct t_irc_channel *channel)
               (channel->key) ? " " : "",
               (channel->key) ? channel->key : "");
     
-    irc_command_join_server (server, join_args);
+    irc_command_join_server (server, join_args, 0);
 }
 
 /*
@@ -859,6 +888,8 @@ irc_channel_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "cycle", channel->cycle))
         return 0;
+    if (!weechat_infolist_new_var_integer (ptr_item, "part", channel->part))
+        return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "display_creation_date", channel->display_creation_date))
         return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "nick_completion_reset", channel->nick_completion_reset))
@@ -927,6 +958,7 @@ irc_channel_print_log (struct t_irc_channel *channel)
     weechat_log_printf ("       away_message . . . . . . : '%s'",  channel->away_message);
     weechat_log_printf ("       has_quit_server. . . . . : %d",    channel->has_quit_server);
     weechat_log_printf ("       cycle. . . . . . . . . . : %d",    channel->cycle);
+    weechat_log_printf ("       part . . . . . . . . . . : %d",    channel->part);
     weechat_log_printf ("       display_creation_date. . : %d",    channel->display_creation_date);
     weechat_log_printf ("       nick_completion_reset. . : %d",    channel->nick_completion_reset);
     weechat_log_printf ("       pv_remote_nick_color . . : '%s'",  channel->pv_remote_nick_color);
