@@ -57,6 +57,9 @@
 
 /* macros for WeeChat core and IRC colors */
 
+#define IRC_COLOR_BAR_FG weechat_color("bar_fg")
+#define IRC_COLOR_BAR_BG weechat_color("bar_bg")
+#define IRC_COLOR_BAR_DELIM weechat_color("bar_delim")
 #define IRC_COLOR_CHAT weechat_color("chat")
 #define IRC_COLOR_CHAT_CHANNEL weechat_color("chat_channel")
 #define IRC_COLOR_CHAT_DELIMITERS weechat_color("chat_delimiters")
@@ -72,16 +75,15 @@
 #define IRC_COLOR_NICK_PREFIX_USER weechat_color(weechat_config_string(irc_config_color_nick_prefix_user))
 #define IRC_COLOR_NICK_PREFIX weechat_color(weechat_config_string(irc_config_color_nick_prefix))
 #define IRC_COLOR_NICK_SUFFIX weechat_color(weechat_config_string(irc_config_color_nick_suffix))
-#define IRC_COLOR_BAR_FG weechat_color("bar_fg")
-#define IRC_COLOR_BAR_BG weechat_color("bar_bg")
-#define IRC_COLOR_BAR_DELIM weechat_color("bar_delim")
+#define IRC_COLOR_NOTICE weechat_color(weechat_config_string(irc_config_color_notice))
 #define IRC_COLOR_STATUS_NUMBER weechat_color("status_number")
 #define IRC_COLOR_STATUS_NAME weechat_color("status_name")
 #define IRC_COLOR_STATUS_NAME_SSL weechat_color("status_name_ssl")
 #define IRC_COLOR_MESSAGE_JOIN weechat_color(weechat_config_string(irc_config_color_message_join))
 #define IRC_COLOR_MESSAGE_QUIT weechat_color(weechat_config_string(irc_config_color_message_quit))
 #define IRC_COLOR_REASON_QUIT weechat_color(weechat_config_string(irc_config_color_reason_quit))
-#define IRC_COLOR_NOTICE weechat_color(weechat_config_string(irc_config_color_notice))
+#define IRC_COLOR_TOPIC_OLD weechat_color(weechat_config_string(irc_config_color_topic_old))
+#define IRC_COLOR_TOPIC_NEW weechat_color(weechat_config_string(irc_config_color_topic_new))
 #define IRC_COLOR_INPUT_NICK weechat_color(weechat_config_string(irc_config_color_input_nick))
 #define IRC_COLOR_ITEM_AWAY weechat_color(weechat_config_string(irc_config_color_item_away))
 #define IRC_COLOR_ITEM_CHANNEL_MODES weechat_color(weechat_config_string(irc_config_color_item_channel_modes))
