@@ -50,6 +50,10 @@ struct t_config_section *irc_config_section_server = NULL;
 
 /* IRC config, look section */
 
+struct t_config_option *irc_config_look_buffer_switch_autojoin;
+struct t_config_option *irc_config_look_buffer_switch_join;
+struct t_config_option *irc_config_look_color_nicks_in_names;
+struct t_config_option *irc_config_look_color_nicks_in_nicklist;
 struct t_config_option *irc_config_look_color_nicks_in_server_messages;
 struct t_config_option *irc_config_look_color_pv_nick_like_channel;
 struct t_config_option *irc_config_look_server_buffer;
@@ -88,16 +92,14 @@ struct t_config_option *irc_config_look_smart_filter;
 struct t_config_option *irc_config_look_smart_filter_delay;
 struct t_config_option *irc_config_look_smart_filter_join;
 struct t_config_option *irc_config_look_smart_filter_quit;
+struct t_config_option *irc_config_look_smart_filter_nick;
 struct t_config_option *irc_config_look_topic_strip_colors;
 
 /* IRC config, color section */
 
 struct t_config_option *irc_config_color_message_join;
 struct t_config_option *irc_config_color_message_quit;
-struct t_config_option *irc_config_color_nick_prefix_op;
-struct t_config_option *irc_config_color_nick_prefix_halfop;
-struct t_config_option *irc_config_color_nick_prefix_voice;
-struct t_config_option *irc_config_color_nick_prefix_user;
+struct t_config_option *irc_config_color_nick_prefixes;
 struct t_config_option *irc_config_color_nick_prefix;
 struct t_config_option *irc_config_color_nick_suffix;
 struct t_config_option *irc_config_color_notice;
@@ -107,6 +109,8 @@ struct t_config_option *irc_config_color_item_channel_modes;
 struct t_config_option *irc_config_color_item_lag_counting;
 struct t_config_option *irc_config_color_item_lag_finished;
 struct t_config_option *irc_config_color_reason_quit;
+struct t_config_option *irc_config_color_topic_old;
+struct t_config_option *irc_config_color_topic_new;
 
 /* IRC config, network section */
 
@@ -130,6 +134,7 @@ struct t_hook *irc_config_hook_config_nick_colors = NULL;
 char **irc_config_nick_colors = NULL;
 int irc_config_num_nick_colors = 0;
 struct t_hashtable *irc_config_hashtable_nick_color_force = NULL;
+struct t_hashtable *irc_config_hashtable_nick_prefixes = NULL;
 
 int irc_config_write_temp_servers = 0;
 
@@ -180,12 +185,24 @@ irc_config_compute_nick_colors ()
             for (ptr_nick = ptr_channel->nicks; ptr_nick;
                  ptr_nick = ptr_nick->next_nick)
             {
-                if (ptr_nick->color)
-                    free (ptr_nick->color);
-                ptr_nick->color = strdup (irc_nick_find_color (ptr_nick->name));
+                if (weechat_strcasecmp (ptr_nick->name, ptr_server->nick) != 0)
+                {
+                    if (ptr_nick->color)
+                        free (ptr_nick->color);
+                    ptr_nick->color = strdup (irc_nick_find_color (ptr_nick->name));
+                }
+            }
+            if (ptr_channel->pv_remote_nick_color)
+            {
+                free (ptr_channel->pv_remote_nick_color);
+                ptr_channel->pv_remote_nick_color = NULL;
             }
         }
     }
+    
+    /* if colors are displayed for nicks in nicklist, refresh them */
+    if (weechat_config_boolean(irc_config_look_color_nicks_in_nicklist))
+        irc_nick_nicklist_set_color_all ();
 }
 
 /*
@@ -229,6 +246,23 @@ irc_config_change_nick_colors_cb (void *data, const char *option,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * irc_config_change_look_color_nicks_in_nicklist: called when the
+ *                                                 "color nicks in nicklist"
+ *                                                 option is changed
+ */
+
+void
+irc_config_change_look_color_nicks_in_nicklist (void *data,
+                                                struct t_config_option *option)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) option;
+    
+    irc_nick_nicklist_set_color_all ();
+}
+
 /*
  * irc_config_change_look_server_buffer: called when the "one server buffer"
  *                                       option is changed
@@ -557,18 +591,52 @@ irc_config_change_color_item_lag (void *data,
 }
 
 /*
- * irc_config_change_color_nick_prefix: called when the color of a nick prefix
- *                                      is changed
+ * irc_config_change_color_nick_prefixes: called when the string with color of
+ *                                        nick prefixes is changed
  */
 
 void
-irc_config_change_color_nick_prefix (void *data,
-                                     struct t_config_option *option)
+irc_config_change_color_nick_prefixes (void *data,
+                                       struct t_config_option *option)
 {
+    char **items, *pos;
+    int num_items, i;
+    
     /* make C compiler happy */
     (void) data;
     (void) option;
     
+    if (!irc_config_hashtable_nick_prefixes)
+    {
+        irc_config_hashtable_nick_prefixes = weechat_hashtable_new (8,
+                                                                    WEECHAT_HASHTABLE_STRING,
+                                                                    WEECHAT_HASHTABLE_STRING,
+                                                                    NULL,
+                                                                    NULL);
+    }
+    else
+        weechat_hashtable_remove_all (irc_config_hashtable_nick_prefixes);
+    
+    items = weechat_string_split (weechat_config_string (irc_config_color_nick_prefixes),
+                                  ";", 0, 0, &num_items);
+    if (items)
+    {
+        for (i = 0; i < num_items; i++)
+        {
+            pos = strchr (items[i], ':');
+            if (pos)
+            {
+                pos[0] = '\0';
+                weechat_hashtable_set (irc_config_hashtable_nick_prefixes,
+                                       items[i],
+                                       pos + 1);
+            }
+        }
+        weechat_string_free_split (items);
+    }
+    
+    irc_nick_nicklist_set_prefix_color_all ();
+    
     weechat_bar_item_update ("input_prompt");
     weechat_bar_item_update ("nicklist");
 }
@@ -739,6 +807,76 @@ irc_config_server_default_change_cb (void *data, struct t_config_option *option)
     }
 }
 
+/*
+ * irc_config_check_gnutls_priorities: check string with GnuTLS priorities
+ *                                     return NULL if ok, or pointer to char
+ *                                     with error in string
+ */
+
+const char *
+irc_config_check_gnutls_priorities (const char *priorities)
+{
+#ifdef HAVE_GNUTLS
+    gnutls_priority_t priority_cache;
+    const char *pos_error;
+    int rc;
+    
+    if (!priorities || !priorities[0])
+        return NULL;
+
+    rc = gnutls_priority_init (&priority_cache, priorities, &pos_error);
+    if (rc == GNUTLS_E_SUCCESS)
+        return NULL;
+    if (pos_error)
+        return pos_error;
+    return priorities;
+#else
+    /* make C compiler happy */
+    (void) priorities;
+
+    return NULL;
+#endif
+}
+
+/*
+ * irc_config_server_check_value_cb: callback called to check a server option
+ *                                   when it is modified
+ */
+
+int
+irc_config_server_check_value_cb (void *data,
+                                  struct t_config_option *option,
+                                  const char *value)
+{
+    int index_option;
+    const char *pos_error;
+    
+    /* make C compiler happy */
+    (void) option;
+    
+    index_option = irc_server_search_option (data);
+    if (index_option >= 0)
+    {
+        switch (index_option)
+        {
+            case IRC_SERVER_OPTION_SSL_PRIORITIES:
+                pos_error = irc_config_check_gnutls_priorities (value);
+                if (pos_error)
+                {
+                    weechat_printf (NULL,
+                                    _("%s%s: invalid priorities string, error "
+                                      "at this position in string: \"%s\""),
+                                    weechat_prefix ("error"), IRC_PLUGIN_NAME,
+                                    pos_error);
+                    return 0;
+                }
+                break;
+        }
+    }
+    
+    return 1;
+}
+
 /*
  * irc_config_server_change_cb: callback called when a server option is modified
  */
@@ -898,7 +1036,7 @@ irc_config_msgbuffer_create_option (void *data,
                     option_name, "integer",
                     _("buffer used to display message received from IRC "
                       "server"),
-                    "weechat|current|private", 0, 0, value, value, 0,
+                    "weechat|server|current|private", 0, 0, value, value, 0,
                     NULL, NULL, NULL, NULL, NULL, NULL);
                 rc = (ptr_option) ?
                     WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE : WEECHAT_CONFIG_OPTION_SET_ERROR;
@@ -1124,6 +1262,8 @@ irc_config_server_new_option (struct t_config_file *config_file,
                               const char *default_value,
                               const char *value,
                               int null_value_allowed,
+                              void *callback_check_value,
+                              void *callback_check_value_data,
                               void *callback_change,
                               void *callback_change_data)
 {
@@ -1141,7 +1281,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1153,7 +1293,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1165,7 +1305,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1177,7 +1317,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1191,7 +1331,22 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
+                callback_change, callback_change_data,
+                NULL, NULL);
+            break;
+        case IRC_SERVER_OPTION_SSL_PRIORITIES:
+            new_option = weechat_config_new_option (
+                config_file, section,
+                option_name, "string",
+                N_("string with priorities for gnutls (for syntax, see "
+                   "documentation of function gnutls_priority_init in gnutls "
+                   "manual, common strings are: \"PERFORMANCE\", \"NORMAL\", "
+                    "\"SECURE128\", \"SECURE256\", \"EXPORT\", \"NONE\")"),
+                NULL, 0, 0,
+                default_value, value,
+                null_value_allowed,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1203,7 +1358,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, INT_MAX,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1215,7 +1370,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1227,7 +1382,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1239,7 +1394,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 "plain|dh-blowfish", 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1251,7 +1406,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1263,7 +1418,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1276,7 +1431,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 1, 3600,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1288,7 +1443,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1300,7 +1455,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1312,7 +1467,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 1, 65535,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1324,7 +1479,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1336,7 +1491,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1348,7 +1503,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1361,7 +1516,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1376,7 +1531,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1389,7 +1544,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 3600,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1402,7 +1557,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1414,7 +1569,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1426,7 +1581,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 3600*24,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1441,7 +1596,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 1, 3600,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1455,7 +1610,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 60,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1469,7 +1624,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 60,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1482,7 +1637,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 60 * 24 * 7,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1495,7 +1650,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 1000000,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1508,7 +1663,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1521,7 +1676,7 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 NULL, 0, 0,
                 default_value, value,
                 null_value_allowed,
-                NULL, NULL,
+                callback_check_value, callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1535,8 +1690,8 @@ irc_config_server_new_option (struct t_config_file *config_file,
                 default_value, value,
                 null_value_allowed,
                 (section == irc_config_section_server_default) ?
-                &irc_config_server_default_check_notify : NULL,
-                NULL,
+                &irc_config_server_default_check_notify : callback_check_value,
+                callback_check_value_data,
                 callback_change, callback_change_data,
                 NULL, NULL);
             break;
@@ -1721,6 +1876,8 @@ irc_config_server_create_default_options (struct t_config_section *section)
             irc_server_option_default[i],
             default_value,
             0,
+            &irc_config_server_check_value_cb,
+            irc_server_option_string[i],
             &irc_config_server_default_change_cb,
             irc_server_option_string[i]);
     }
@@ -1748,6 +1905,11 @@ irc_config_init ()
                                                                    WEECHAT_HASHTABLE_STRING,
                                                                    NULL,
                                                                    NULL);
+    irc_config_hashtable_nick_prefixes = weechat_hashtable_new (8,
+                                                                WEECHAT_HASHTABLE_STRING,
+                                                                WEECHAT_HASHTABLE_STRING,
+                                                                NULL,
+                                                                NULL);
     
     irc_config_file = weechat_config_new (IRC_CONFIG_NAME,
                                           &irc_config_reload, NULL);
@@ -1765,7 +1927,34 @@ irc_config_init ()
         weechat_config_free (irc_config_file);
         return 0;
     }
-    
+
+    irc_config_look_buffer_switch_autojoin = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "buffer_switch_autojoin", "boolean",
+        N_("auto switch to channel buffer when it is auto joined (with "
+           "server option \"autojoin\")"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL,
+        NULL, NULL, NULL, NULL);
+    irc_config_look_buffer_switch_join = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "buffer_switch_join", "boolean",
+        N_("auto switch to channel buffer when it is manually joined "
+           "(with /join command)"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL,
+        NULL, NULL, NULL, NULL);
+    irc_config_look_color_nicks_in_names = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "color_nicks_in_names", "boolean",
+        N_("use nick color in output of /names (or list of nicks displayed "
+           "when joining a channel)"),
+        NULL, 0, 0, "off", NULL, 0, NULL, NULL,
+        NULL, NULL, NULL, NULL);
+    irc_config_look_color_nicks_in_nicklist = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "color_nicks_in_nicklist", "boolean",
+        N_("use nick color in nicklist"),
+        NULL, 0, 0, "off", NULL, 0, NULL, NULL,
+        &irc_config_change_look_color_nicks_in_nicklist, NULL, NULL, NULL);
     irc_config_look_color_nicks_in_server_messages = weechat_config_new_option (
         irc_config_file, ptr_section,
         "color_nicks_in_server_messages", "boolean",
@@ -1991,8 +2180,8 @@ irc_config_init ()
     irc_config_look_smart_filter = weechat_config_new_option (
         irc_config_file, ptr_section,
         "smart_filter", "boolean",
-        N_("filter join/part/quit messages for a nick if not speaking for "
-           "some minutes on channel (you must create a filter on tag "
+        N_("filter join/part/quit/nick messages for a nick if not speaking "
+           "for some minutes on channel (you must create a filter on tag "
            "\"irc_smart_filter\")"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     irc_config_look_smart_filter_delay = weechat_config_new_option (
@@ -2003,13 +2192,21 @@ irc_config_init ()
     irc_config_look_smart_filter_join = weechat_config_new_option (
         irc_config_file, ptr_section,
         "smart_filter_join", "boolean",
+        /* TRANSLATORS: please do not translate "join" */
         N_("enable smart filter for \"join\" messages"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     irc_config_look_smart_filter_quit = weechat_config_new_option (
         irc_config_file, ptr_section,
         "smart_filter_quit", "boolean",
+        /* TRANSLATORS: please do not translate "part" and "quit" */
         N_("enable smart filter for \"part\" and \"quit\" messages"),
         NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+    irc_config_look_smart_filter_nick = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "smart_filter_nick", "boolean",
+        /* TRANSLATORS: please do not translate "nick" */
+        N_("enable smart filter for \"nick\" messages"),
+        NULL, 0, 0, "on", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     irc_config_look_topic_strip_colors = weechat_config_new_option (
         irc_config_file, ptr_section,
         "topic_strip_colors", "boolean",
@@ -2041,30 +2238,17 @@ irc_config_init ()
         N_("color for text in part/quit messages"),
         NULL, -1, 0, "red", NULL, 0, NULL, NULL,
         NULL, NULL, NULL, NULL);
-    irc_config_color_nick_prefix_op = weechat_config_new_option (
+    irc_config_color_nick_prefixes = weechat_config_new_option (
         irc_config_file, ptr_section,
-        "nick_prefix_op", "color",
-        N_("color for prefix of nick which is op/admin/owner on channel"),
-        NULL, -1, 0, "lightgreen", NULL, 0, NULL, NULL,
-        &irc_config_change_color_nick_prefix, NULL, NULL, NULL);
-    irc_config_color_nick_prefix_halfop = weechat_config_new_option (
-        irc_config_file, ptr_section,
-        "nick_prefix_halfop", "color",
-        N_("color for prefix of nick which is halfop on channel"),
-        NULL, -1, 0, "lightmagenta", NULL, 0, NULL, NULL,
-        &irc_config_change_color_nick_prefix, NULL, NULL, NULL);
-    irc_config_color_nick_prefix_voice = weechat_config_new_option (
-        irc_config_file, ptr_section,
-        "nick_prefix_voice", "color",
-        N_("color for prefix of nick which has voice on channel"),
-        NULL, -1, 0, "yellow", NULL, 0, NULL, NULL,
-        &irc_config_change_color_nick_prefix, NULL, NULL, NULL);
-    irc_config_color_nick_prefix_user = weechat_config_new_option (
-        irc_config_file, ptr_section,
-        "nick_prefix_user", "color",
-        N_("color for prefix of nick which is user on channel"),
-        NULL, -1, 0, "blue", NULL, 0, NULL, NULL,
-        &irc_config_change_color_nick_prefix, NULL, NULL, NULL);
+        "nick_prefixes", "string",
+        N_("color for nick prefixes using mode char (o=op, h=halfop, v=voice, "
+           "..), format is: \"o:color1;h:color2;v:color3\" (if a mode is not "
+           "found, WeeChat will try with next modes received from server "
+           "(\"PREFIX\"); a special mode \"*\" can be used as default color "
+           "if no mode has been found in list)"),
+        NULL, 0, 0, "q:lightred;a:lightcyan;o:lightgreen;h:lightmagenta;"
+        "v:yellow;*:lightblue", NULL, 0, NULL, NULL,
+        &irc_config_change_color_nick_prefixes, NULL, NULL, NULL);
     irc_config_color_nick_prefix = weechat_config_new_option (
         irc_config_file, ptr_section,
         "nick_prefix", "color",
@@ -2122,6 +2306,18 @@ irc_config_init ()
         N_("color for reason in part/quit messages"),
         NULL, -1, 0, "default", NULL, 0, NULL, NULL,
         NULL, NULL, NULL, NULL);
+    irc_config_color_topic_old = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "topic_old", "color",
+        N_("color for old channel topic (when topic is changed)"),
+        NULL, -1, 0, "darkgray", NULL, 0, NULL, NULL,
+        NULL, NULL, NULL, NULL);
+    irc_config_color_topic_new = weechat_config_new_option (
+        irc_config_file, ptr_section,
+        "topic_new", "color",
+        N_("color for new channel topic (when topic is changed)"),
+        NULL, -1, 0, "white", NULL, 0, NULL, NULL,
+        NULL, NULL, NULL, NULL);
     
     /* network */
     ptr_section = weechat_config_new_section (irc_config_file, "network",
@@ -2296,6 +2492,8 @@ irc_config_read ()
     if (rc == WEECHAT_CONFIG_READ_OK)
     {
         irc_notify_new_for_all_servers ();
+        irc_config_change_look_nick_color_force (NULL, NULL);
+        irc_config_change_color_nick_prefixes (NULL, NULL);
         irc_config_change_network_notify_check_ison (NULL, NULL);
         irc_config_change_network_notify_check_whois (NULL, NULL);
     }
@@ -2340,4 +2538,10 @@ irc_config_free ()
         weechat_hashtable_free (irc_config_hashtable_nick_color_force);
         irc_config_hashtable_nick_color_force = NULL;
     }
+    
+    if (irc_config_hashtable_nick_prefixes)
+    {
+        weechat_hashtable_free (irc_config_hashtable_nick_prefixes);
+        irc_config_hashtable_nick_prefixes = NULL;
+    }
 }
