@@ -1,23 +1,25 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* irc-mode.c: IRC channel/user modes management */
-
+/*
+ * irc-mode.c: IRC channel/user modes management
+ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -30,24 +32,6 @@
 #include "irc-nick.h"
 
 
-/*
- * irc_mode_channel_set_nick: set a mode for a nick on a channel
- */
-
-void
-irc_mode_channel_set_nick (struct t_irc_channel *channel, const char *nick,
-                           char set_flag, int flag)
-{
-    struct t_irc_nick *ptr_nick;
-    
-    if (nick)
-    {
-        ptr_nick = irc_nick_search (channel, nick);
-        if (ptr_nick)
-            irc_nick_set (channel, ptr_nick, (set_flag == '+'), flag);
-    }
-}
-
 /*
  * irc_mode_channel_set: set channel modes
  *                       return: 1 if channel modes are updated
@@ -61,6 +45,7 @@ irc_mode_channel_set (struct t_irc_server *server,
 {
     char *pos_args, *str_modes, set_flag, **argv, *pos, *ptr_arg;
     int modes_count, channel_modes_updated, argc, current_arg;
+    struct t_irc_nick *ptr_nick;
     
     if (!server || !channel || !modes)
         return 0;
@@ -118,25 +103,7 @@ irc_mode_channel_set (struct t_irc_server *server,
                 case '-':
                     set_flag = '-';
                     break;
-                case 'a': /* channel admin (unrealircd specific flag) */
-                    ptr_arg = ((current_arg >= 0) && (current_arg < argc)) ?
-                        argv[current_arg] : NULL;
-                    if (irc_mode_nick_prefix_allowed (server, '~'))
-                        irc_mode_channel_set_nick (channel, ptr_arg,
-                                                   set_flag, IRC_NICK_CHANADMIN);
-                    current_arg++;
-                    break;
                 case 'b': /* ban (ignored) */
-                    ptr_arg = ((current_arg >= 0) && (current_arg < argc)) ?
-                        argv[current_arg] : NULL;
-                    current_arg++;
-                    break;
-                case 'h': /* half-op */
-                    ptr_arg = ((current_arg >= 0) && (current_arg < argc)) ?
-                        argv[current_arg] : NULL;
-                    if (irc_mode_nick_prefix_allowed (server, '%'))
-                        irc_mode_channel_set_nick (channel, ptr_arg,
-                                                   set_flag, IRC_NICK_HALFOP);
                     current_arg++;
                     break;
                 case 'k': /* channel key */
@@ -168,41 +135,24 @@ irc_mode_channel_set (struct t_irc_server *server,
                     channel_modes_updated = 1;
                     current_arg++;
                     break;
-                case 'o': /* op */
-                    ptr_arg = ((current_arg >= 0) && (current_arg < argc)) ?
-                        argv[current_arg] : NULL;
-                    if (irc_mode_nick_prefix_allowed (server, '@'))
-                        irc_mode_channel_set_nick (channel, ptr_arg,
-                                                   set_flag, IRC_NICK_OP);
-                    current_arg++;
-                    break;
-                case 'q': /* channel owner (unrealircd specific flag) */
-                    ptr_arg = ((current_arg >= 0) && (current_arg < argc)) ?
-                        argv[current_arg] : NULL;
-                    if (irc_mode_nick_prefix_allowed (server, '~'))
-                        irc_mode_channel_set_nick (channel, ptr_arg,
-                                                   set_flag, IRC_NICK_CHANOWNER);
-                    current_arg++;
-                    break;
-                case 'u': /* channel user */
-                    ptr_arg = ((current_arg >= 0) && (current_arg < argc)) ?
-                        argv[current_arg] : NULL;
-                    if (irc_mode_nick_prefix_allowed (server, '-'))
-                        irc_mode_channel_set_nick (channel, ptr_arg,
-                                                   set_flag, IRC_NICK_CHANUSER);
-                    current_arg++;
-                    break;
-                case 'v': /* voice */
-                    ptr_arg = ((current_arg >= 0) && (current_arg < argc)) ?
-                        argv[current_arg] : NULL;
-                    if (irc_mode_nick_prefix_allowed (server, '+'))
-                        irc_mode_channel_set_nick (channel, ptr_arg,
-                                                   set_flag, IRC_NICK_VOICE);
-                    current_arg++;
-                    break;
                 default:
+                    if (irc_server_get_prefix_mode_index (server, pos[0]) >= 0)
+                    {
+                        ptr_arg = ((current_arg >= 0) && (current_arg < argc)) ?
+                            argv[current_arg] : NULL;
+                        if (ptr_arg)
+                        {
+                            ptr_nick = irc_nick_search (channel, ptr_arg);
+                            if (ptr_nick)
+                            {
+                                irc_nick_set_mode (server, channel, ptr_nick,
+                                                   (set_flag == '+'), pos[0]);
+                            }
+                        }
+                    }
+                    else
+                        channel_modes_updated = 1;
                     current_arg++;
-                    channel_modes_updated = 1;
                     break;
             }
             pos++;
@@ -277,10 +227,19 @@ irc_mode_user_remove (struct t_irc_server *server, char mode)
  */
 
 void
-irc_mode_user_set (struct t_irc_server *server, const char *modes)
+irc_mode_user_set (struct t_irc_server *server, const char *modes,
+                   int reset_modes)
 {
     char set_flag;
-    
+
+    if (reset_modes)
+    {
+        if (server->nick_modes)
+        {
+            free (server->nick_modes);
+            server->nick_modes = NULL;
+        }
+    }
     set_flag = '+';
     while (modes && modes[0])
     {
@@ -304,27 +263,5 @@ irc_mode_user_set (struct t_irc_server *server, const char *modes)
         }
         modes++;
     }
-}
-
-/*
- * irc_mode_nick_prefix_allowed: return <> 0 if nick prefix is allowed by server
- *                               for example :
- *                                 IRC:  005 (...) PREFIX=(ov)@+
- *                               => allowed prefixes: @+
- */
-
-int
-irc_mode_nick_prefix_allowed (struct t_irc_server *server, char prefix)
-{
-    char str[2];
-    
-    /* if server did not send any prefix info, then use default prefixes */
-    if (!server->prefix)
-    {
-        str[0] = prefix;
-        str[1] = '\0';
-        return (strpbrk (str, IRC_NICK_DEFAULT_PREFIXES_LIST)) ? 1 : 0;
-    }
-    
-    return (strchr (server->prefix, prefix) != NULL);
+    weechat_bar_item_update ("input_prompt");
 }
