@@ -1,40 +1,45 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 #ifndef __WEECHAT_IRC_PROTOCOL_H
 #define __WEECHAT_IRC_PROTOCOL_H 1
 
-#define IRC_PROTOCOL_GET_HOST                                           \
-    const char *nick, *address, *host;                                  \
-    if (argv[0][0] == ':')                                              \
-    {                                                                   \
-        nick = irc_protocol_get_nick_from_host (argv[0]);               \
-        address = irc_protocol_get_address_from_host (argv[0]);         \
-        host = argv[0] + 1;                                             \
-    }                                                                   \
-    else                                                                \
-    {                                                                   \
-        nick = NULL;                                                    \
-        address = NULL;                                                 \
-        host = NULL;                                                    \
-    }
+#define IRC_PROTOCOL_CALLBACK(__command)                                \
+    int                                                                 \
+    irc_protocol_cb_##__command (struct t_irc_server *server,           \
+                                 const char *nick,                      \
+                                 const char *address,                   \
+                                 const char *host,                      \
+                                 const char *command,                   \
+                                 int ignored,                           \
+                                 int argc,                              \
+                                 char **argv,                           \
+                                 char **argv_eol)
+
 #define IRC_PROTOCOL_MIN_ARGS(__min_args)                               \
+    (void) nick;                                                        \
+    (void) address;                                                     \
+    (void) host;                                                        \
+    (void) command;                                                     \
+    (void) ignored;                                                     \
+    (void) argv;                                                        \
+    (void) argv_eol;                                                    \
     if (argc < __min_args)                                              \
     {                                                                   \
         weechat_printf (server->buffer,                                 \
@@ -59,7 +64,10 @@
 
 struct t_irc_server;
 
-typedef int (t_irc_recv_func)(struct t_irc_server *server, const char *command,
+typedef int (t_irc_recv_func)(struct t_irc_server *server,
+                              const char *nick, const char *address,
+                              const char *host, const char *command,
+                              int ignored,
                               int argc, char **argv, char **argv_eol);
 
 struct t_irc_protocol_msg
@@ -69,10 +77,11 @@ struct t_irc_protocol_msg
     t_irc_recv_func *recv_function; /* function called when msg is received  */
 };
 
-extern const char *irc_protocol_get_nick_from_host (const char *host);
-extern const char *irc_protocol_tags (const char *command, const char *tags);
+extern const char *irc_protocol_tags (const char *command, const char *tags,
+                                      const char *nick);
 extern void irc_protocol_recv_command (struct t_irc_server *server,
-                                       const char *entire_line,
-                                       const char *command);
+                                       const char *irc_message,
+                                       const char *msg_command,
+                                       const char *msg_channel);
 
-#endif /* irc-protocol.h */
+#endif /* __WEECHAT_IRC_PROTOCOL_H */
