@@ -1,23 +1,25 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* logger-config.c: logger configuration options */
-
+/*
+ * logger-config.c: logger configuration options (file logger.conf)
+ */
 
 #include <stdlib.h>
 #include <limits.h>
@@ -63,10 +65,7 @@ logger_config_change_file_option_restart_log (void *data,
     (void) option;
     
     if (!logger_config_loading)
-    {
-        logger_stop_all ();
-        logger_start_buffer_all ();
-    }
+        logger_adjust_log_filenames ();
 }
 
 /*
@@ -80,9 +79,9 @@ logger_config_level_change (void *data,
     /* make C compiler happy */
     (void) data;
     (void) option;
-
+    
     if (!logger_config_loading)
-        logger_start_buffer_all ();
+        logger_start_buffer_all (1);
 }
 
 /*
@@ -102,7 +101,7 @@ logger_config_level_delete_option (void *data,
     
     weechat_config_option_free (option);
     
-    logger_start_buffer_all ();
+    logger_start_buffer_all (1);
     
     return WEECHAT_CONFIG_OPTION_UNSET_OK_REMOVED;
 }
@@ -161,7 +160,7 @@ logger_config_level_create_option (void *data,
     }
 
     if (!logger_config_loading)
-        logger_start_buffer_all ();
+        logger_start_buffer_all (1);
     
     return rc;
 }
@@ -205,10 +204,7 @@ logger_config_mask_change (void *data,
     (void) option;
 
     if (!logger_config_loading)
-    {
-        logger_stop_all ();
-        logger_start_buffer_all ();
-    }
+        logger_adjust_log_filenames ();
 }
 
 /*
@@ -228,8 +224,7 @@ logger_config_mask_delete_option (void *data,
     
     weechat_config_option_free (option);
     
-    logger_stop_all ();
-    logger_start_buffer_all ();
+    logger_adjust_log_filenames ();
     
     return WEECHAT_CONFIG_OPTION_UNSET_OK_REMOVED;
 }
@@ -286,12 +281,9 @@ logger_config_mask_create_option (void *data,
                 rc = WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE;
         }
     }
-
+    
     if (!logger_config_loading)
-    {
-        logger_stop_all ();
-        logger_start_buffer_all ();
-    }
+        logger_adjust_log_filenames ();
     
     return rc;
 }
@@ -369,8 +361,9 @@ logger_config_init ()
     logger_config_file_path = weechat_config_new_option (
         logger_config_file, ptr_section,
         "path", "string",
-        N_("path for WeeChat log files (\"%h\" will be replaced by WeeChat "
-           "home, \"~/.weechat\" by default)"),
+        N_("path for WeeChat log files; \"%h\" at beginning of string is "
+           "replaced by WeeChat home (\"~/.weechat\" by default); date "
+           "specifiers are permitted (see man strftime)"),
         NULL, 0, 0, "%h/logs/", NULL, 0, NULL, NULL,
         &logger_config_change_file_option_restart_log, NULL, NULL, NULL);
     logger_config_file_mask = weechat_config_new_option (
@@ -379,7 +372,8 @@ logger_config_init ()
         N_("default file name mask for log files (format is "
            "\"directory/to/file\" or \"file\", without first \"/\" because "
            "\"path\" option is used to build complete path to file); local "
-           "buffer variables are permitted"),
+           "buffer variables are permitted; date specifiers are permitted "
+           "(see man strftime)"),
         NULL, 0, 0, "$plugin.$name.weechatlog", NULL, 0, NULL, NULL,
         &logger_config_change_file_option_restart_log, NULL, NULL, NULL);
     logger_config_file_replacement_char = weechat_config_new_option (
