@@ -944,18 +944,19 @@ logger_backlog (struct t_gui_buffer *buffer, const char *filename, int lines)
         {
             if (datetime != 0)
             {
-                weechat_printf_date_tags (buffer, datetime, "no_highlight",
+                weechat_printf_date_tags (buffer, datetime,
+                                          "no_highlight,notify_none",
                                           "%s", pos_message + 1);
             }
             else
             {
-                weechat_printf_tags (buffer, "no_highlight",
+                weechat_printf_tags (buffer, "no_highlight,notify_none",
                                      "%s", ptr_lines->data);
             }
         }
         else
         {
-            weechat_printf_tags (buffer, "no_highlight",
+            weechat_printf_tags (buffer, "no_highlight,notify_none",
                                  "%s", ptr_lines->data);
         }
         num_lines++;
@@ -965,9 +966,9 @@ logger_backlog (struct t_gui_buffer *buffer, const char *filename, int lines)
         logger_tail_free (last_lines);
     if (num_lines > 0)
     {
-        weechat_printf (buffer,
-                        _("===\t========== End of backlog (%d lines) =========="),
-                        num_lines);
+        weechat_printf_tags (buffer, "no_highlight,notify_none",
+                             _("===\t========== End of backlog (%d lines) =========="),
+                             num_lines);
         weechat_buffer_set (buffer, "unread", "");
     }
     weechat_buffer_set (buffer, "print_hooks_enabled", "1");
@@ -1214,7 +1215,9 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     /* command /logger */
     weechat_hook_command ("logger",
                           N_("logger plugin configuration"),
-                          N_("[list | set level | disable]"),
+                          N_("list"
+                             " || set <level>"
+                             " || disable"),
                           N_("   list: show logging status for opened buffers\n"
                              "    set: set logging level on current buffer\n"
                              "  level: level for messages to be logged (0 = "
