@@ -168,6 +168,19 @@ plugin_api_config_set_plugin (struct t_weechat_plugin *plugin,
     return plugin_config_set (plugin->name, option_name, value);
 }
 
+/*
+ * plugin_api_config_set_desc_plugin: set description of a plugin config option
+ */
+
+void
+plugin_api_config_set_desc_plugin (struct t_weechat_plugin *plugin,
+                                   const char *option_name,
+                                   const char *description)
+{
+    if (plugin && option_name)
+        plugin_config_set_desc (plugin->name, option_name, description);
+}
+
 /*
  * plugin_api_config_unset_plugin: unset plugin config option
  */
@@ -1004,7 +1017,9 @@ plugin_api_init ()
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "hook", N_("list of hooks"),
                    NULL,
-                   N_("hook type: command, timer, .. (optional)"),
+                   N_("type,arguments (type is command/timer/.., arguments to "
+                      "get only some hooks (can start or end with \"*\" as "
+                      "wildcard), both are optional)"),
                    &plugin_api_infolist_get_internal, NULL);
     hook_infolist (NULL, "hotlist", N_("list of buffers in hotlist"),
                    NULL,
