@@ -575,6 +575,7 @@ plugin_load (const char *filename)
         new_plugin->config_get_plugin = &plugin_api_config_get_plugin;
         new_plugin->config_is_set_plugin = &plugin_api_config_is_set_plugin;
         new_plugin->config_set_plugin = &plugin_api_config_set_plugin;
+        new_plugin->config_set_desc_plugin = &plugin_api_config_set_desc_plugin;
         new_plugin->config_unset_plugin = &plugin_api_config_unset_plugin;
         
         new_plugin->prefix = &plugin_api_prefix;
@@ -619,7 +620,9 @@ plugin_load (const char *filename)
         new_plugin->buffer_set = &gui_buffer_set;
         new_plugin->buffer_set_pointer = &gui_buffer_set_pointer;
         new_plugin->buffer_string_replace_local_var = &gui_buffer_string_replace_local_var;
+        new_plugin->buffer_match_list = &gui_buffer_match_list;
         
+        new_plugin->window_search_with_buffer = &gui_window_search_with_buffer;
         new_plugin->window_get_integer = &gui_window_get_integer;
         new_plugin->window_get_string = &gui_window_get_string;
         new_plugin->window_get_pointer = &gui_window_get_pointer;
