@@ -1,23 +1,25 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* relay-buffer.c: display clients list on relay buffer */
-
+/*
+ * relay-buffer.c: display clients list on relay buffer
+ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -26,8 +28,10 @@
 
 #include "../weechat-plugin.h"
 #include "relay.h"
+#include "relay-buffer.h"
 #include "relay-client.h"
 #include "relay-config.h"
+#include "relay-raw.h"
 
 
 struct t_gui_buffer *relay_buffer = NULL;
@@ -123,9 +127,10 @@ relay_buffer_refresh (const char *hotlist)
             str_recv = weechat_string_format_size (ptr_client->bytes_recv);
             str_sent = weechat_string_format_size (ptr_client->bytes_sent);
             weechat_printf_y (relay_buffer, (line * 2) + 3,
-                              _("%s%-26s protocol: %s, received: %s, sent: %s"),
+                              _("%s%-26s id: %d, protocol: %s, received: %s, sent: %s"),
                               weechat_color(str_color),
                               " ",
+                              ptr_client->id,
                               relay_protocol_string[ptr_client->protocol],
                               (str_recv) ? str_recv : "?",
                               (str_sent) ? str_sent : "?");
@@ -155,42 +160,50 @@ relay_buffer_input_cb (void *data, struct t_gui_buffer *buffer,
     /* make C compiler happy */
     (void) data;
     
-    client = relay_client_search_by_number (relay_buffer_selected_line);
-    
-    /* disconnect client */
-    if (weechat_strcasecmp (input_data, "d") == 0)
+    if (buffer == relay_raw_buffer)
     {
-        if (client && !RELAY_CLIENT_HAS_ENDED(client->status))
+        if (weechat_strcasecmp (input_data, "q") == 0)
+            weechat_buffer_close (buffer);
+    }
+    else if (buffer == relay_buffer)
+    {
+        client = relay_client_search_by_number (relay_buffer_selected_line);
+        
+        /* disconnect client */
+        if (weechat_strcasecmp (input_data, "d") == 0)
         {
-            relay_client_disconnect (client);
+            if (client && !RELAY_CLIENT_HAS_ENDED(client->status))
+            {
+                relay_client_disconnect (client);
+                relay_buffer_refresh (WEECHAT_HOTLIST_MESSAGE);
+            }
+        }
+        /* purge old clients */
+        else if (weechat_strcasecmp (input_data, "p") == 0)
+        {
+            ptr_client = relay_clients;
+            while (ptr_client)
+            {
+                next_client = ptr_client->next_client;
+                if (RELAY_CLIENT_HAS_ENDED(ptr_client->status))
+                    relay_client_free (ptr_client);
+                ptr_client = next_client;
+            }
             relay_buffer_refresh (WEECHAT_HOTLIST_MESSAGE);
         }
-    }
-    /* purge old clients */
-    else if (weechat_strcasecmp (input_data, "p") == 0)
-    {
-        ptr_client = relay_clients;
-        while (ptr_client)
+        /* quit relay buffer (close it) */
+        else if (weechat_strcasecmp (input_data, "q") == 0)
         {
-            next_client = ptr_client->next_client;
-            if (RELAY_CLIENT_HAS_ENDED(ptr_client->status))
-                relay_client_free (ptr_client);
-            ptr_client = next_client;
+            weechat_buffer_close (buffer);
         }
-        relay_buffer_refresh (WEECHAT_HOTLIST_MESSAGE);
-    }
-    /* quit relay buffer (close it) */
-    else if (weechat_strcasecmp (input_data, "q") == 0)
-    {
-        weechat_buffer_close (buffer);
-    }
-    /* remove client */
-    else if (weechat_strcasecmp (input_data, "r") == 0)
-    {
-        if (client && RELAY_CLIENT_HAS_ENDED(client->status))
+        /* remove client */
+        else if (weechat_strcasecmp (input_data, "r") == 0)
         {
-            relay_client_free (client);
-            relay_buffer_refresh (WEECHAT_HOTLIST_MESSAGE);
+            if (client && RELAY_CLIENT_HAS_ENDED(client->status))
+            {
+                relay_client_free (client);
+                relay_buffer_refresh (WEECHAT_HOTLIST_MESSAGE);
+            }
         }
     }
     
@@ -206,9 +219,15 @@ relay_buffer_close_cb (void *data, struct t_gui_buffer *buffer)
 {
     /* make C compiler happy */
     (void) data;
-    (void) buffer;
     
-    relay_buffer = NULL;
+    if (buffer == relay_raw_buffer)
+    {
+        relay_raw_buffer = NULL;
+    }
+    else if (buffer == relay_buffer)
+    {
+        relay_buffer = NULL;
+    }
     
     return WEECHAT_RC_OK;
 }
@@ -222,7 +241,7 @@ relay_buffer_open ()
 {
     if (!relay_buffer)
     {
-        relay_buffer = weechat_buffer_new ("relay.list",
+        relay_buffer = weechat_buffer_new (RELAY_BUFFER_NAME,
                                            &relay_buffer_input_cb, NULL,
                                            &relay_buffer_close_cb, NULL);
         
