@@ -564,8 +564,11 @@ relay_client_irc_send_join_channels (struct t_relay_client *client)
     {
         while (weechat_infolist_next (infolist_channels))
         {
-            channel = weechat_infolist_string (infolist_channels, "name");
-            relay_client_irc_send_join (client, channel);
+            if (weechat_infolist_integer (infolist_channels, "nicks_count") > 0)
+            {
+                channel = weechat_infolist_string (infolist_channels, "name");
+                relay_client_irc_send_join (client, channel);
+            }
         }
         weechat_infolist_free (infolist_channels);
     }
@@ -708,7 +711,7 @@ relay_client_irc_recv_one_msg (struct t_relay_client *client, char *data)
             {
                 if (irc_args && irc_args[0]
                     && (strcmp (weechat_config_string (relay_config_network_password),
-                                irc_args) == 0))
+                                (irc_args[0] == ':') ? irc_args + 1 : irc_args) == 0))
                 {
                     RELAY_IRC_DATA(client, password_ok) = 1;
                 }
@@ -1014,7 +1017,10 @@ relay_client_irc_alloc_with_infolist (struct t_relay_client *client,
     {
         RELAY_IRC_DATA(client, address) = strdup (weechat_infolist_string (infolist, "address"));
         RELAY_IRC_DATA(client, password_ok) = weechat_infolist_integer (infolist, "password_ok");
-        RELAY_IRC_DATA(client, nick) = strdup (weechat_infolist_string (infolist, "nick"));
+        if (weechat_infolist_string (infolist, "nick"))
+            RELAY_IRC_DATA(client, nick) = strdup (weechat_infolist_string (infolist, "nick"));
+        else
+            RELAY_IRC_DATA(client, nick) = NULL;
         RELAY_IRC_DATA(client, user_received) = weechat_infolist_integer (infolist, "user_received");
         RELAY_IRC_DATA(client, connected) = weechat_infolist_integer (infolist, "connected");
         if (RELAY_IRC_DATA(client, connected))
