@@ -1,27 +1,33 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2005-2007 Emmanuel Bouthenot <kolter@openics.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* weechat-python.c: Python plugin for WeeChat */
-
+/*
+ * weechat-python.c: python plugin for WeeChat
+ */
 
 #undef _
 
 #include <Python.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #include "../../weechat-plugin.h"
 #include "../script.h"
@@ -31,9 +37,9 @@
 
 WEECHAT_PLUGIN_NAME(PYTHON_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION("Python plugin for WeeChat");
-WEECHAT_PLUGIN_AUTHOR("FlashCode <flashcode@flashtux.org>");
+WEECHAT_PLUGIN_AUTHOR("Sebastien Helleu <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
-WEECHAT_PLUGIN_LICENSE("GPL3");
+WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_python_plugin = NULL;
 
@@ -41,33 +47,173 @@ int python_quiet;
 struct t_plugin_script *python_scripts = NULL;
 struct t_plugin_script *last_python_script = NULL;
 struct t_plugin_script *python_current_script = NULL;
+struct t_plugin_script *python_registered_script = NULL;
 const char *python_current_script_filename = NULL;
 PyThreadState *python_mainThreadState = NULL;
+char *python2_bin = NULL;
 
-/* string used to execute action "install":
-   when signal "python_install_script" is received, name of string
-   is added to this string, to be installed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "install":
+ * when signal "python_install_script" is received, name of string
+ * is added to this string, to be installed later by a timer (when nothing is
+ * running in script)
+ */
 char *python_action_install_list = NULL;
 
-/* string used to execute action "remove":
-   when signal "python_remove_script" is received, name of string
-   is added to this string, to be removed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "remove":
+ * when signal "python_remove_script" is received, name of string
+ * is added to this string, to be removed later by a timer (when nothing is
+ * running in script)
+ */
 char *python_action_remove_list = NULL;
 
 char python_buffer_output[128];
 
 
 /*
- * weechat_python_exec: execute a Python script
+ * weechat_python_set_python2_bin: set path to python 2.x interpreter
+ */
+
+void
+weechat_python_set_python2_bin ()
+{
+    const char *dir_separator;
+    char *path, **paths, bin[4096];
+    char *versions[] = { "2", "2.7", "2.6", "2.5", "2.4", "2.3", "2.2", NULL };
+    int num_paths, i, j, rc;
+    struct stat stat_buf;
+    
+    python2_bin = NULL;
+    
+    dir_separator = weechat_info_get ("dir_separator", "");
+    path = getenv ("PATH");
+    
+    if (dir_separator && path)
+    {
+        paths = weechat_string_split (path, ":", 0, 0, &num_paths);
+        if (paths)
+        {
+            for (i = 0; i < num_paths; i++)
+            {
+                for (j = 0; versions[j]; j++)
+                {
+                    snprintf (bin, sizeof (bin), "%s%s%s%s",
+                              paths[i], dir_separator, "python",
+                              versions[j]);
+                    rc = stat (bin, &stat_buf);
+                    if ((rc == 0) && (S_ISREG(stat_buf.st_mode)))
+                    {
+                        python2_bin = strdup (bin);
+                        break;
+                    }
+                }
+                if (python2_bin)
+                    break;
+            }
+            weechat_string_free_split (paths);
+        }
+    }
+    
+    if (!python2_bin)
+        python2_bin = strdup ("python");
+}
+
+/*
+ * weechat_python_hashtable_map_cb: callback called for each key/value in a
+ *                                  hashtable
+ */
+
+void
+weechat_python_hashtable_map_cb (void *data,
+                                 struct t_hashtable *hashtable,
+                                 const void *key,
+                                 const void *value)
+{
+    PyObject *dict, *dict_key, *dict_value;
+
+    /* make C compiler happy */
+    (void) hashtable;
+    
+    dict = (PyObject *)data;
+    
+    dict_key = Py_BuildValue ("s", (const char *)key);
+    dict_value = Py_BuildValue ("s", (const char *)value);
+    
+    PyDict_SetItem (dict, dict_key, dict_value);
+    
+    Py_DECREF (dict_key);
+    Py_DECREF (dict_value);
+}
+
+/*
+ * weechat_python_hashtable_to_dict: get python dictionary with a WeeChat
+ *                                   hashtable
+ */
+
+PyObject *
+weechat_python_hashtable_to_dict (struct t_hashtable *hashtable)
+{
+    PyObject *dict;
+    
+    dict = PyDict_New ();
+    if (!dict)
+    {
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+    
+    weechat_hashtable_map (hashtable,
+                           &weechat_python_hashtable_map_cb,
+                           dict);
+    
+    return dict;
+}
+
+/*
+ * weechat_python_dict_to_hashtable: get WeeChat hashtable with python
+ *                                   dictionary
+ *                                   Hashtable returned has type string for
+ *                                   both keys and values
+ *                                   Note: hashtable has to be released after
+ *                                   use with call to weechat_hashtable_free()
+ */
+
+struct t_hashtable *
+weechat_python_dict_to_hashtable (PyObject *dict, int hashtable_size)
+{
+    struct t_hashtable *hashtable;
+    PyObject *key, *value;
+    Py_ssize_t pos;
+    char *str_key, *str_value;
+    
+    hashtable = weechat_hashtable_new (hashtable_size,
+                                       WEECHAT_HASHTABLE_STRING,
+                                       WEECHAT_HASHTABLE_STRING,
+                                       NULL,
+                                       NULL);
+    if (!hashtable)
+        return NULL;
+    
+    pos = 0;
+    while (PyDict_Next (dict, &pos, &key, &value))
+    {
+        str_key = PyString_AsString (key);
+        str_value = PyString_AsString (value);
+        weechat_hashtable_set (hashtable, str_key, str_value);
+    }
+    
+    return hashtable;
+}
+
+/*
+ * weechat_python_exec: execute a python function
  */
 
 void *
 weechat_python_exec (struct t_plugin_script *script,
-		     int ret_type, const char *function, char **argv)
+                     int ret_type, const char *function,
+                     char *format, void **argv)
 {
     struct t_plugin_script *old_python_current_script;
     PyThreadState *old_interpreter;
@@ -75,8 +221,8 @@ weechat_python_exec (struct t_plugin_script *script,
     PyObject *evDict;
     PyObject *evFunc;
     PyObject *rc;
-    void *ret_value;
-    int *ret_i;
+    void *argv2[16], *ret_value;
+    int i, argc, *ret_int;
     
     /* PyEval_AcquireLock (); */
     
@@ -97,117 +243,70 @@ weechat_python_exec (struct t_plugin_script *script,
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to run function \"%s\""),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME, function);
-	/* PyEval_ReleaseThread (python_current_script->interpreter); */
+        /* PyEval_ReleaseThread (python_current_script->interpreter); */
         if (old_interpreter)
             PyThreadState_Swap (old_interpreter);
-	return NULL;
+        return NULL;
     }
     
     python_current_script = script;
-    
+
     if (argv && argv[0])
     {
-        if (argv[1])
+        argc = strlen (format);
+        for (i = 0; i < 16; i++)
         {
-            if (argv[2])
-            {
-                if (argv[3])
-                {
-                    if (argv[4])
-                    {
-                        if (argv[5])
-                        {
-                            if (argv[6])
-                            {
-                                if (argv[7])
-                                {
-                                    rc = PyObject_CallFunction (evFunc, "ssssssss",
-                                                                argv[0], argv[1],
-                                                                argv[2], argv[3],
-                                                                argv[4], argv[5],
-                                                                argv[6], argv[7]);
-                                }
-                                else
-                                {
-                                    rc = PyObject_CallFunction (evFunc, "sssssss",
-                                                                argv[0], argv[1],
-                                                                argv[2], argv[3],
-                                                                argv[4], argv[5],
-                                                                argv[6]);
-                                }
-                            }
-                            else
-                            {
-                                rc = PyObject_CallFunction (evFunc, "ssssss",
-                                                            argv[0], argv[1],
-                                                            argv[2], argv[3],
-                                                            argv[4], argv[5]);
-                            }
-                        }
-                        else
-                        {
-                            rc = PyObject_CallFunction (evFunc, "sssss",
-                                                        argv[0], argv[1],
-                                                        argv[2], argv[3],
-                                                        argv[4]);
-                        }
-                    }
-                    else
-                    {
-                        rc = PyObject_CallFunction (evFunc, "ssss", argv[0],
-                                                    argv[1], argv[2], argv[3]);
-                    }
-                }
-                else
-                {
-                    rc = PyObject_CallFunction (evFunc, "sss", argv[0],
-                                                argv[1], argv[2]);
-                }
-            }
-            else
-            {
-                rc = PyObject_CallFunction (evFunc, "ss", argv[0], argv[1]);
-            }
-        }
-        else
-        {
-            rc = PyObject_CallFunction (evFunc, "s", argv[0]);
+            argv2[i] = (i < argc) ? argv[i] : NULL;
         }
+        rc = PyObject_CallFunction (evFunc, format,
+                                    argv2[0], argv2[1],
+                                    argv2[2], argv2[3],
+                                    argv2[4], argv2[5],
+                                    argv2[6], argv2[7],
+                                    argv2[8], argv2[9],
+                                    argv2[10], argv2[11],
+                                    argv2[12], argv2[13],
+                                    argv2[14], argv2[15]);
     }
     else
+    {
         rc = PyObject_CallFunction (evFunc, NULL);
+    }
     
     ret_value = NULL;
     
-    /* 
-       ugly hack : rc = NULL while 'return weechat.WEECHAT_RC_OK .... 
-       because of '#define WEECHAT_RC_OK 0'
-    */
+    /*
+     * ugly hack : rc = NULL while 'return weechat.WEECHAT_RC_OK ....
+     * because of '#define WEECHAT_RC_OK 0'
+     */
     if (rc ==  NULL)
-	rc = PyInt_FromLong (0);
+        rc = PyInt_FromLong (0);
     
     if (PyErr_Occurred())
     {
         PyErr_Print ();
         Py_XDECREF(rc);
     }
-    else if (PyString_Check (rc) && (ret_type == WEECHAT_SCRIPT_EXEC_STRING))
+    else if ((ret_type == WEECHAT_SCRIPT_EXEC_STRING) && (PyString_Check (rc)))
     {
-	if (PyString_AsString (rc))
-	    ret_value = strdup (PyString_AsString(rc));
-	else
-	    ret_value = NULL;
-	
-	Py_XDECREF(rc);
+        if (PyString_AsString (rc))
+            ret_value = strdup (PyString_AsString(rc));
+        else
+            ret_value = NULL;
+        Py_XDECREF(rc);
     }
-    else if (PyInt_Check (rc) && (ret_type == WEECHAT_SCRIPT_EXEC_INT))
+    else if ((ret_type == WEECHAT_SCRIPT_EXEC_INT) && (PyInt_Check (rc)))
     {
-	
-	ret_i = malloc (sizeof (*ret_i));
-	if (ret_i)
-	    *ret_i = (int) PyInt_AsLong(rc);
-        ret_value = ret_i;
-        
+        ret_int = malloc (sizeof (*ret_int));
+        if (ret_int)
+            *ret_int = (int) PyInt_AsLong(rc);
+        ret_value = ret_int;
+        Py_XDECREF(rc);
+    }
+    else if (ret_type == WEECHAT_SCRIPT_EXEC_HASHTABLE)
+    {
+        ret_value = weechat_python_dict_to_hashtable (rc,
+                                                      WEECHAT_SCRIPT_HASHTABLE_DEFAULT_SIZE);
         Py_XDECREF(rc);
     }
     else
@@ -227,8 +326,7 @@ weechat_python_exec (struct t_plugin_script *script,
     
     /* PyEval_ReleaseThread (python_current_script->interpreter); */
     
-    if (old_python_current_script)
-        python_current_script = old_python_current_script;
+    python_current_script = old_python_current_script;
     
     if (old_interpreter)
         PyThreadState_Swap (old_interpreter);
@@ -237,7 +335,7 @@ weechat_python_exec (struct t_plugin_script *script,
 }
 
 /*
- * weechat_python_output : redirection for stdout and stderr
+ * weechat_python_output: redirection for stdout and stderr
  */
 
 static PyObject *
@@ -253,39 +351,39 @@ weechat_python_output (PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple (args, "s", &msg))
     {
         if (strlen(python_buffer_output) > 0)
-	{
-	    weechat_printf (NULL,
+        {
+            weechat_printf (NULL,
                             weechat_gettext ("%s: stdout/stderr: %s%s"),
                             PYTHON_PLUGIN_NAME, python_buffer_output, "");
-	    python_buffer_output[0] = '\0';
-	}
+            python_buffer_output[0] = '\0';
+        }
     }
-    else 
+    else
     {
-	m = msg;
-	while ((p = strchr (m, '\n')) != NULL)
-	{
-	    *p = '\0';
-	    if (strlen (m) + strlen (python_buffer_output) > 0)
+        m = msg;
+        while ((p = strchr (m, '\n')) != NULL)
+        {
+            *p = '\0';
+            if (strlen (m) + strlen (python_buffer_output) > 0)
             {
-		weechat_printf (NULL,
+                weechat_printf (NULL,
                                 weechat_gettext ("%s: stdout/stderr: %s%s"),
                                 PYTHON_PLUGIN_NAME, python_buffer_output, m);
             }
-	    *p = '\n';
-	    python_buffer_output[0] = '\0';
-	    m = ++p;
-	}
-	
-	if (strlen(m) + strlen(python_buffer_output) > sizeof(python_buffer_output))
-	{
-	    weechat_printf (NULL,
+            *p = '\n';
+            python_buffer_output[0] = '\0';
+            m = ++p;
+        }
+
+        if (strlen(m) + strlen(python_buffer_output) > sizeof(python_buffer_output))
+        {
+            weechat_printf (NULL,
                             weechat_gettext ("%s: stdout/stderr: %s%s"),
                             PYTHON_PLUGIN_NAME, python_buffer_output, m);
-	    python_buffer_output[0] = '\0';
-	}
-	else
-	    strcat (python_buffer_output, m);
+            python_buffer_output[0] = '\0';
+        }
+        else
+            strcat (python_buffer_output, m);
     }
     
     Py_INCREF(Py_None);
@@ -334,6 +432,7 @@ weechat_python_load (const char *filename)
     }
     
     python_current_script = NULL;
+    python_registered_script = NULL;
     
     /* PyEval_AcquireLock (); */
     python_current_interpreter = Py_NewInterpreter ();
@@ -361,14 +460,14 @@ weechat_python_load (const char *filename)
                                          "module"),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME);
         fclose (fp);
-	
+        
         Py_EndInterpreter (python_current_interpreter);
         /* PyEval_ReleaseLock (); */
 
         return 0;
     }
 
-    /* adding $weechat_dir/python in $PYTHONPATH */    
+    /* adding $weechat_dir/python in $PYTHONPATH */
     python_path = PySys_GetObject ("path");
     weechat_home = weechat_info_get ("weechat_dir", "");
     if (weechat_home)
@@ -491,13 +590,13 @@ weechat_python_load (const char *filename)
 
     fclose (fp);
     
-    if (python_current_script == NULL)
+    if (!python_registered_script)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"register\" not "
                                          "found (or failed) in file \"%s\""),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME, filename);
-	
+        
         if (PyErr_Occurred ())
             PyErr_Print ();
         Py_EndInterpreter (python_current_interpreter);
@@ -505,6 +604,7 @@ weechat_python_load (const char *filename)
 
         return 0;
     }
+    python_current_script = python_registered_script;
     
     python_current_script->interpreter = (PyThreadState *) python_current_interpreter;
     /* PyEval_ReleaseThread (python_current_script->interpreter); */
@@ -536,16 +636,19 @@ weechat_python_unload (struct t_plugin_script *script)
     void *interpreter;
     PyThreadState *old_interpreter;
     
-    weechat_printf (NULL,
-                    weechat_gettext ("%s: unloading script \"%s\""),
-                    PYTHON_PLUGIN_NAME, script->name);
+    if ((weechat_python_plugin->debug >= 1) || !python_quiet)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s: unloading script \"%s\""),
+                        PYTHON_PLUGIN_NAME, script->name);
+    }
     
     if (script->shutdown_func && script->shutdown_func[0])
     {
         r = (int *) weechat_python_exec (script, WEECHAT_SCRIPT_EXEC_INT,
-					 script->shutdown_func, NULL);
-	if (r)
-	    free (r);
+                                         script->shutdown_func, NULL, NULL);
+        if (r)
+            free (r);
     }
 
     old_interpreter = PyThreadState_Swap (NULL);
@@ -603,6 +706,38 @@ weechat_python_unload_all ()
     }
 }
 
+/*
+ * weechat_python_reload_name: reload a Python script by name
+ */
+
+void
+weechat_python_reload_name (const char *name)
+{
+    struct t_plugin_script *ptr_script;
+    char *filename;
+    
+    ptr_script = script_search (weechat_python_plugin, python_scripts, name);
+    if (ptr_script)
+    {
+        filename = strdup (ptr_script->filename);
+        if (filename)
+        {
+            weechat_python_unload (ptr_script);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s: script \"%s\" unloaded"),
+                            PYTHON_PLUGIN_NAME, name);
+            weechat_python_load (filename);
+            free (filename);
+        }
+    }
+    else
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: script \"%s\" not loaded"),
+                        weechat_prefix ("error"), PYTHON_PLUGIN_NAME, name);
+    }
+}
+
 /*
  * weechat_python_cmd: callback for "/python" command
  */
@@ -669,6 +804,11 @@ weechat_python_command_cb (void *data, struct t_gui_buffer *buffer,
             if (path_script)
                 free (path_script);
         }
+        else if (weechat_strcasecmp (argv[1], "reload") == 0)
+        {
+            /* reload one Python script */
+            weechat_python_reload_name (argv_eol[2]);
+        }
         else if (weechat_strcasecmp (argv[1], "unload") == 0)
         {
             /* unload Python script */
@@ -706,6 +846,26 @@ weechat_python_completion_cb (void *data, const char *completion_item,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * weechat_python_info_cb: callback for info
+ */
+
+const char *
+weechat_python_info_cb (void *data, const char *info_name,
+                        const char *arguments)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) arguments;
+    
+    if (weechat_strcasecmp (info_name, "python2_bin") == 0)
+    {
+        return python2_bin;
+    }
+    
+    return NULL;
+}
+
 /*
  * weechat_python_infolist_cb: callback for infolist
  */
@@ -743,9 +903,12 @@ weechat_python_signal_debug_dump_cb (void *data, const char *signal,
     (void) data;
     (void) signal;
     (void) type_data;
-    (void) signal_data;
     
-    script_print_log (weechat_python_plugin, python_scripts);
+    if (!signal_data
+        || (weechat_strcasecmp ((char *)signal_data, PYTHON_PLUGIN_NAME) == 0))
+    {
+        script_print_log (weechat_python_plugin, python_scripts);
+    }
     
     return WEECHAT_RC_OK;
 }
@@ -848,6 +1011,16 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
     weechat_python_plugin = plugin;
     
+    /*
+     * hook info to get path to python 2.x interpreter
+     * (some scripts using hook_process need that)
+     */
+    weechat_python_set_python2_bin ();
+    weechat_hook_info ("python2_bin",
+                       N_("path to python 2.x interpreter"),
+                       NULL,
+                       &weechat_python_info_cb, NULL);
+    
     /* init stdout/stderr buffer */
     python_buffer_output[0] = '\0';
     
@@ -905,18 +1078,17 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
-    /* make C compiler happy */
-    (void) plugin;
-    
     /* unload all scripts */
-    weechat_python_unload_all ();
+    python_quiet = 1;
+    script_end (plugin, &python_scripts, &weechat_python_unload_all);
+    python_quiet = 0;
     
     /* free Python interpreter */
     if (python_mainThreadState != NULL)
     {
-	/* PyEval_AcquireLock (); */
+        /* PyEval_AcquireLock (); */
         PyThreadState_Swap (python_mainThreadState);
-	/* PyEval_ReleaseLock (); */
+        /* PyEval_ReleaseLock (); */
         python_mainThreadState = NULL;
     }
     
@@ -929,6 +1101,8 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
     }
     
     /* free some data */
+    if (python2_bin)
+        free (python2_bin);
     if (python_action_install_list)
         free (python_action_install_list);
     if (python_action_remove_list)
