@@ -1,23 +1,26 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2005-2007 Emmanuel Bouthenot <kolter@openics.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* weechat-ruby.c: Ruby plugin for WeeChat */
-
+/*
+ * weechat-ruby.c: ruby plugin for WeeChat
+ */
 
 #undef _
 
@@ -54,9 +57,9 @@
 
 WEECHAT_PLUGIN_NAME(RUBY_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION("Ruby plugin for WeeChat");
-WEECHAT_PLUGIN_AUTHOR("FlashCode <flashcode@flashtux.org>");
+WEECHAT_PLUGIN_AUTHOR("Sebastien Helleu <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
-WEECHAT_PLUGIN_LICENSE("GPL3");
+WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 
 struct t_weechat_plugin *weechat_ruby_plugin = NULL;
 
@@ -64,20 +67,23 @@ int ruby_quiet = 0;
 struct t_plugin_script *ruby_scripts = NULL;
 struct t_plugin_script *last_ruby_script = NULL;
 struct t_plugin_script *ruby_current_script = NULL;
+struct t_plugin_script *ruby_registered_script = NULL;
 const char *ruby_current_script_filename = NULL;
 
-/* string used to execute action "install":
-   when signal "ruby_install_script" is received, name of string
-   is added to this string, to be installed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "install":
+ * when signal "ruby_install_script" is received, name of string
+ * is added to this string, to be installed later by a timer (when nothing is
+ * running in script)
+ */
 char *ruby_action_install_list = NULL;
 
-/* string used to execute action "remove":
-   when signal "ruby_remove_script" is received, name of string
-   is added to this string, to be removed later by a timer (when nothing is
-   running in script)
-*/
+/*
+ * string used to execute action "remove":
+ * when signal "ruby_remove_script" is received, name of string
+ * is added to this string, to be removed later by a timer (when nothing is
+ * running in script)
+ */
 char *ruby_action_remove_list = NULL;
 
 VALUE ruby_mWeechat, ruby_mWeechatOutputs;
@@ -95,11 +101,94 @@ typedef struct protect_call_arg {
 } protect_call_arg_t;
 
 
-/* 
+/*
+ * weechat_ruby_hashtable_map_cb: callback called for each key/value in a
+ *                                hashtable
+ */
+
+void
+weechat_ruby_hashtable_map_cb (void *data,
+                               struct t_hashtable *hashtable,
+                               const void *key,
+                               const void *value)
+{
+    VALUE *hash;
+    
+    /* make C compiler happy */
+    (void) hashtable;
+    
+    hash = (VALUE *)data;
+    
+    rb_hash_aset (hash[0], rb_str_new2 ((char *)key),
+                  rb_str_new2 ((char *)value));
+}
+
+/*
+ * weechat_ruby_hashtable_to_hash: get ruby hash with a WeeChat hashtable
+ */
+
+VALUE
+weechat_ruby_hashtable_to_hash (struct t_hashtable *hashtable)
+{
+    VALUE hash;
+
+    hash = rb_hash_new ();
+    if (NIL_P (hash))
+        return Qnil;
+
+    weechat_hashtable_map (hashtable,
+                           &weechat_ruby_hashtable_map_cb,
+                           &hash);
+    
+    return hash;
+}
+
+int
+weechat_ruby_hash_foreach_cb (VALUE key, VALUE value, void *arg)
+{
+    struct t_hashtable *hashtable;
+    
+    hashtable = (struct t_hashtable *)arg;
+    if ((TYPE(key) == T_STRING) && (TYPE(value) == T_STRING))
+    {
+        weechat_hashtable_set (hashtable, StringValuePtr(key),
+                               StringValuePtr(value));
+    }
+    return 0;
+}
+
+/*
+ * weechat_ruby_hash_to_hashtable: get WeeChat hashtable with ruby hashtable
+ *                                 Hashtable returned has type string for
+ *                                 both keys and values
+ *                                 Note: hashtable has to be released after use
+ *                                 with call to weechat_hashtable_free()
+ */
+
+struct t_hashtable *
+weechat_ruby_hash_to_hashtable (VALUE hash, int hashtable_size)
+{
+    struct t_hashtable *hashtable;
+    
+    hashtable = weechat_hashtable_new (hashtable_size,
+                                       WEECHAT_HASHTABLE_STRING,
+                                       WEECHAT_HASHTABLE_STRING,
+                                       NULL,
+                                       NULL);
+    if (!hashtable)
+        return NULL;
+    
+    rb_hash_foreach (hash, &weechat_ruby_hash_foreach_cb,
+                     (unsigned long)hashtable);
+    
+    return hashtable;
+}
+
+/*
  * protect_funcall0 : used to protect a function call
  */
 
-static VALUE 
+static VALUE
 protect_funcall0 (VALUE arg)
 {
     return rb_funcall2 (((protect_call_arg_t *)arg)->recv,
@@ -108,33 +197,20 @@ protect_funcall0 (VALUE arg)
                         ((protect_call_arg_t *)arg)->argv);
 }
 
-/* 
- * rb_protect_funcall : function call in protect mode
+/*
+ * rb_protect_funcall: function call in protect mode
  */
 
 VALUE
-rb_protect_funcall (VALUE recv, ID mid, int *state, int argc, ...)
+rb_protect_funcall (VALUE recv, ID mid, int *state, int argc, VALUE *argv)
 {
-    va_list ap;
-    VALUE *argv;
     struct protect_call_arg arg;
-
-    if (argc > 0)
-    {
-        int i;
-        argv = ALLOCA_N(VALUE, argc);
-        va_start(ap, argc);
-        for (i = 0; i < argc; i++)
-            argv[i] = va_arg(ap, VALUE);
-        va_end(ap);
-    }
-    else
-        argv = 0;
+    
     arg.recv = recv;
     arg.mid = mid;
     arg.argc = argc;
     arg.argv = argv;
-    return rb_protect(protect_funcall0, (VALUE) &arg, state);
+    return rb_protect (protect_funcall0, (VALUE) &arg, state);
 }
 
 /*
@@ -144,7 +220,7 @@ rb_protect_funcall (VALUE recv, ID mid, int *state, int argc, ...)
 int
 weechat_ruby_print_exception (VALUE err)
 {
-    VALUE backtrace;
+    VALUE backtrace, tmp1, tmp2, tmp3;
     int i;
     int ruby_error;
     char* line;
@@ -153,26 +229,29 @@ weechat_ruby_print_exception (VALUE err)
     char* err_class;
     
     backtrace = rb_protect_funcall (err, rb_intern("backtrace"),
-                                    &ruby_error, 0);
-    err_msg = STR2CSTR(rb_protect_funcall(err, rb_intern("message"),
-                                          &ruby_error, 0));
-    err_class = STR2CSTR(rb_protect_funcall(rb_protect_funcall(err,
-                                                               rb_intern("class"),
-                                                               &ruby_error, 0),
-                                            rb_intern("name"), &ruby_error, 0));
+                                    &ruby_error, 0, NULL);
+
+    tmp1 = rb_protect_funcall(err, rb_intern("message"), &ruby_error, 0, NULL);
+    err_msg = StringValueCStr(tmp1);
+
+    tmp2 = rb_protect_funcall(rb_protect_funcall(err, rb_intern("class"),
+                                                 &ruby_error, 0, NULL),
+                              rb_intern("name"), &ruby_error, 0, NULL);
+    err_class = StringValuePtr(tmp2);
     
     if (strcmp (err_class, "SyntaxError") == 0)
     {
+	tmp3 = rb_inspect(err);
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: error: %s"),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME,
-                        STR2CSTR(rb_inspect(err)));
+                        StringValuePtr(tmp3));
     }
     else
     {
         for (i = 0; i < RARRAY_LEN(backtrace); i++)
         {
-            line = STR2CSTR(RARRAY_PTR(backtrace)[i]);
+            line = StringValuePtr(RARRAY_PTR(backtrace)[i]);
             cline = NULL;
             if (i == 0)
             {
@@ -216,168 +295,106 @@ weechat_ruby_print_exception (VALUE err)
 }
 
 /*
- * weechat_ruby_exec: call a ruby command
+ * weechat_ruby_exec: execute a ruby function
  */
 
 void *
 weechat_ruby_exec (struct t_plugin_script *script,
-		   int ret_type, const char *function, char **argv)
+                   int ret_type, const char *function,
+                   const char *format, void **argv)
 {
     VALUE rc, err;
-    int ruby_error, *ret_i;
+    int ruby_error, i, argc, *ret_i;
+    VALUE argv2[16];
     void *ret_value;
     struct t_plugin_script *old_ruby_current_script;
-
+    
     old_ruby_current_script = ruby_current_script;
     ruby_current_script = script;
     
-    if (argv && argv[0])
+    argc = 0;
+    if (format && format[0])
     {
-        if (argv[1])
+        argc = strlen (format);
+        for (i = 0; i < argc; i++)
         {
-            if (argv[2])
+            switch (format[i])
             {
-                if (argv[3])
-                {
-                    if (argv[4])
-                    {
-                        if (argv[5])
-                        {
-                            if (argv[6])
-                            {
-                                if (argv[7])
-                                {
-                                    rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
-                                                             &ruby_error, 8,
-                                                             rb_str_new2(argv[0]),
-                                                             rb_str_new2(argv[1]),
-                                                             rb_str_new2(argv[2]),
-                                                             rb_str_new2(argv[3]),
-                                                             rb_str_new2(argv[4]),
-                                                             rb_str_new2(argv[5]),
-                                                             rb_str_new2(argv[6]),
-                                                             rb_str_new2(argv[7]));
-                                }
-                                else
-                                {
-                                    rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
-                                                             &ruby_error, 7,
-                                                             rb_str_new2(argv[0]),
-                                                             rb_str_new2(argv[1]),
-                                                             rb_str_new2(argv[2]),
-                                                             rb_str_new2(argv[3]),
-                                                             rb_str_new2(argv[4]),
-                                                             rb_str_new2(argv[5]),
-                                                             rb_str_new2(argv[6]));
-                                }
-                            }
-                            else
-                            {
-                                rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
-                                                         &ruby_error, 6,
-                                                         rb_str_new2(argv[0]),
-                                                         rb_str_new2(argv[1]),
-                                                         rb_str_new2(argv[2]),
-                                                         rb_str_new2(argv[3]),
-                                                         rb_str_new2(argv[4]),
-                                                         rb_str_new2(argv[5]));
-                            }
-                        }
-                        else
-                        {
-                            rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
-                                                     &ruby_error, 5,
-                                                     rb_str_new2(argv[0]),
-                                                     rb_str_new2(argv[1]),
-                                                     rb_str_new2(argv[2]),
-                                                     rb_str_new2(argv[3]),
-                                                     rb_str_new2(argv[4]));
-                        }
-                    }
-                    else
-                    {
-                        rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
-                                                 &ruby_error, 4,
-                                                 rb_str_new2(argv[0]),
-                                                 rb_str_new2(argv[1]),
-                                                 rb_str_new2(argv[2]),
-                                                 rb_str_new2(argv[3]));
-                    }
-                }
-                else
-                {
-                    rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
-                                             &ruby_error, 3,
-                                             rb_str_new2(argv[0]),
-                                             rb_str_new2(argv[1]),
-                                             rb_str_new2(argv[2]));
-                }
-            }
-            else
-            {
-                rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
-                                         &ruby_error, 2,
-                                         rb_str_new2(argv[0]),
-                                         rb_str_new2(argv[1]));
+                case 's': /* string */
+                    argv2[i] = rb_str_new2 ((char *)argv[i]);
+                    break;
+                case 'i': /* integer */
+                    argv2[i] = INT2FIX (*((int *)argv[i]));
+                    break;
+                case 'h': /* hash */
+                    argv2[i] = (VALUE)argv[i];
+                    break;
             }
         }
-        else
-        {
-            rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
-                                     &ruby_error, 1,
-                                     rb_str_new2(argv[0]));
-        }
+    }
+    
+    if (argc > 0)
+    {
+        rc = rb_protect_funcall ((VALUE) script->interpreter,
+                                 rb_intern(function),
+                                 &ruby_error, argc, argv2);
     }
     else
     {
-        rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
-                                 &ruby_error, 0);
+        rc = rb_protect_funcall ((VALUE) script->interpreter,
+                                 rb_intern(function),
+                                 &ruby_error, 0, NULL);
     }
     
     if (ruby_error)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to run function \"%s\""),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, function);
-	
-      err = rb_gv_get("$!");
-      weechat_ruby_print_exception(err);
         
-	return NULL;
+        err = rb_gv_get("$!");
+        weechat_ruby_print_exception(err);
+        
+        return NULL;
     }
     
     if ((TYPE(rc) == T_STRING) && (ret_type == WEECHAT_SCRIPT_EXEC_STRING))
     {
-	if (STR2CSTR (rc))
-	    ret_value = strdup (STR2CSTR (rc));
-	else
-	    ret_value = NULL;
+        if (StringValuePtr (rc))
+            ret_value = strdup (StringValuePtr (rc));
+        else
+            ret_value = NULL;
     }
     else if ((TYPE(rc) == T_FIXNUM) && (ret_type == WEECHAT_SCRIPT_EXEC_INT))
     {
-	ret_i = malloc (sizeof (*ret_i));
-	if (ret_i)
-	    *ret_i = NUM2INT(rc);
-	ret_value = ret_i;
+        ret_i = malloc (sizeof (*ret_i));
+        if (ret_i)
+            *ret_i = NUM2INT(rc);
+        ret_value = ret_i;
+    }
+    else if (ret_type == WEECHAT_SCRIPT_EXEC_HASHTABLE)
+    {
+        ret_value = weechat_ruby_hash_to_hashtable (rc,
+                                                    WEECHAT_SCRIPT_HASHTABLE_DEFAULT_SIZE);
     }
     else
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"%s\" must return a "
                                          "valid value"),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, function);
         ruby_current_script = old_ruby_current_script;
-	return WEECHAT_RC_OK;
+        return WEECHAT_RC_OK;
     }
     
     if (ret_value == NULL)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: not enough memory in function "
                                          "\"%s\""),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, function);
         ruby_current_script = old_ruby_current_script;
-	return NULL;
+        return NULL;
     }
     
     ruby_current_script = old_ruby_current_script;
@@ -389,7 +406,7 @@ weechat_ruby_exec (struct t_plugin_script *script,
  * weechat_ruby_output: redirection for stdout and stderr
  */
 
-static VALUE 
+static VALUE
 weechat_ruby_output (VALUE self, VALUE str)
 {
     char *msg, *p, *m;
@@ -397,37 +414,37 @@ weechat_ruby_output (VALUE self, VALUE str)
     /* make C compiler happy */
     (void) self;
     
-    msg = strdup(STR2CSTR(str));
+    msg = strdup(StringValuePtr(str));
     
     m = msg;
     while ((p = strchr (m, '\n')) != NULL)
     {
-	*p = '\0';
-	if (strlen (m) + strlen (ruby_buffer_output) > 0)
+        *p = '\0';
+        if (strlen (m) + strlen (ruby_buffer_output) > 0)
         {
             weechat_printf (NULL,
                             weechat_gettext ("%s%s: stdout/stderr: %s%s"),
                             weechat_prefix ("error"), RUBY_PLUGIN_NAME,
                             ruby_buffer_output, m);
         }
-	*p = '\n';
-	ruby_buffer_output[0] = '\0';
-	m = ++p;
+        *p = '\n';
+        ruby_buffer_output[0] = '\0';
+        m = ++p;
     }
     
     if (strlen(m) + strlen(ruby_buffer_output) > sizeof(ruby_buffer_output))
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: stdout/stderr: %s%s"),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME,
                         ruby_buffer_output, m);
-	ruby_buffer_output[0] = '\0';
+        ruby_buffer_output[0] = '\0';
     }
     else
-	strcat (ruby_buffer_output, m);
+        strcat (ruby_buffer_output, m);
 
     if (msg)
-	free (msg);
+        free (msg);
     
     return Qnil;
 }
@@ -436,7 +453,7 @@ weechat_ruby_output (VALUE self, VALUE str)
  * weechat_ruby_output_flush: just for compatibility
  */
 
-static VALUE 
+static VALUE
 weechat_ruby_output_flush (VALUE self)
 {
     /* make C compiler happy */
@@ -445,7 +462,6 @@ weechat_ruby_output_flush (VALUE self)
     return Qnil;
 }
 
-
 /*
  * weechat_ruby_load: load a Ruby script
  */
@@ -454,7 +470,7 @@ int
 weechat_ruby_load (const char *filename)
 {
     char modname[64];
-    VALUE curModule, ruby_retcode, err;
+    VALUE curModule, ruby_retcode, err, argv[1];
     int ruby_error;
     struct stat buf;
     
@@ -474,6 +490,7 @@ weechat_ruby_load (const char *filename)
     }
     
     ruby_current_script = NULL;
+    ruby_registered_script = NULL;
     
     snprintf (modname, sizeof(modname), "%s%d", MOD_NAME_PREFIX, ruby_num);
     ruby_num++;
@@ -481,21 +498,22 @@ weechat_ruby_load (const char *filename)
     curModule = rb_define_module(modname);
 
     ruby_current_script_filename = filename;
-    
+
+    argv[0] = rb_str_new2 (filename);
     ruby_retcode = rb_protect_funcall (curModule, rb_intern("load_eval_file"),
-				       &ruby_error, 1, rb_str_new2(filename));
+                                       &ruby_error, 1, argv);
     
     if (ruby_retcode == Qnil)
     {
-      err = rb_gv_get("$!");
-      weechat_ruby_print_exception(err);
-	return 0;
+        err = rb_gv_get("$!");
+        weechat_ruby_print_exception(err);
+        return 0;
     }
     
     if (NUM2INT(ruby_retcode) != 0)
     {
-	switch (NUM2INT(ruby_retcode))
-	{
+        switch (NUM2INT(ruby_retcode))
+        {
             case 1:
                 weechat_printf (NULL,
                                 weechat_gettext ("%s%s: unable to read file "
@@ -518,47 +536,48 @@ weechat_ruby_load (const char *filename)
                                 weechat_prefix ("error"), RUBY_PLUGIN_NAME,
                                 filename);
                 break;
-	}
-	
-	if (NUM2INT(ruby_retcode) == 1 || NUM2INT(ruby_retcode) == 2)
-	{
-          weechat_ruby_print_exception(rb_iv_get(curModule, "@load_eval_file_error"));
-	}
-	
-	return 0;
+        }
+
+        if (NUM2INT(ruby_retcode) == 1 || NUM2INT(ruby_retcode) == 2)
+        {
+            weechat_ruby_print_exception(rb_iv_get(curModule, "@load_eval_file_error"));
+        }
+        
+        return 0;
     }
     
-    ruby_retcode = rb_protect_funcall (curModule, rb_intern("weechat_init"),
-                                       &ruby_error, 0);
+    (void) rb_protect_funcall (curModule, rb_intern("weechat_init"),
+                               &ruby_error, 0, NULL);
     
     if (ruby_error)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to eval function "
                                          "\"weechat_init\" in file \"%s\""),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, filename);
-	
-      err = rb_gv_get("$!");
-      weechat_ruby_print_exception(err);
-	
-	if (ruby_current_script != NULL)
+        
+        err = rb_gv_get("$!");
+        weechat_ruby_print_exception(err);
+
+        if (ruby_current_script != NULL)
         {
-	    script_remove (weechat_ruby_plugin,
+            script_remove (weechat_ruby_plugin,
                            &ruby_scripts, &last_ruby_script,
                            ruby_current_script);
         }
-        
-	return 0;
+
+        return 0;
     }
     
-    if (ruby_current_script == NULL)
+    if (!ruby_registered_script)
     {
-	weechat_printf (NULL,
+        weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"register\" not "
                                          "found (or failed) in file \"%s\""),
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME, filename);
         return 0;
     }
+    ruby_current_script = ruby_registered_script;
     
     ruby_current_script->interpreter = (VALUE *) curModule;
     rb_gc_register_address (ruby_current_script->interpreter);
@@ -587,21 +606,23 @@ void
 weechat_ruby_unload (struct t_plugin_script *script)
 {
     int *r;
-    char *ruby_argv[1] = { NULL };
     void *interpreter;
     
-    weechat_printf (NULL,
-                    weechat_gettext ("%s: unloading script \"%s\""),
-                    RUBY_PLUGIN_NAME, script->name);
+    if ((weechat_ruby_plugin->debug >= 1) || !ruby_quiet)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s: unloading script \"%s\""),
+                        RUBY_PLUGIN_NAME, script->name);
+    }
     
     if (script->shutdown_func && script->shutdown_func[0])
     {
         r = (int *) weechat_ruby_exec (script,
                                        WEECHAT_SCRIPT_EXEC_INT,
-				       script->shutdown_func,
-                                       ruby_argv);
-	if (r)
-	    free (r);
+                                       script->shutdown_func,
+                                       0, NULL);
+        if (r)
+            free (r);
     }
     
     interpreter = script->interpreter;
@@ -614,7 +635,7 @@ weechat_ruby_unload (struct t_plugin_script *script)
                    script);
     
     if (interpreter)
-	rb_gc_unregister_address (interpreter);
+        rb_gc_unregister_address (interpreter);
 }
 
 /*
@@ -642,6 +663,38 @@ weechat_ruby_unload_name (const char *name)
     }
 }
 
+/*
+ * weechat_ruby_reload_name: reload a Ruby script by name
+ */
+
+void
+weechat_ruby_reload_name (const char *name)
+{
+    struct t_plugin_script *ptr_script;
+    char *filename;
+    
+    ptr_script = script_search (weechat_ruby_plugin, ruby_scripts, name);
+    if (ptr_script)
+    {
+        filename = strdup (ptr_script->filename);
+        if (filename)
+        {
+            weechat_ruby_unload (ptr_script);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s: script \"%s\" unloaded"),
+                            RUBY_PLUGIN_NAME, name);
+            weechat_ruby_load (filename);
+            free (filename);
+        }
+    }
+    else
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: script \"%s\" not loaded"),
+                        weechat_prefix ("error"), RUBY_PLUGIN_NAME, name);
+    }
+}
+
 /*
  * weechat_ruby_unload_all: unload all Ruby scripts
  */
@@ -651,7 +704,7 @@ weechat_ruby_unload_all ()
 {
     while (ruby_scripts)
     {
-	weechat_ruby_unload (ruby_scripts);
+        weechat_ruby_unload (ruby_scripts);
     }
 }
 
@@ -721,6 +774,11 @@ weechat_ruby_command_cb (void *data, struct t_gui_buffer *buffer,
             if (path_script)
                 free (path_script);
         }
+        else if (weechat_strcasecmp (argv[1], "reload") == 0)
+        {
+            /* reload one Ruby script */
+            weechat_ruby_reload_name (argv_eol[2]);
+        }
         else if (weechat_strcasecmp (argv[1], "unload") == 0)
         {
             /* unload Ruby script */
@@ -793,9 +851,12 @@ weechat_ruby_signal_debug_dump_cb (void *data, const char *signal,
     (void) data;
     (void) signal;
     (void) type_data;
-    (void) signal_data;
     
-    script_print_log (weechat_ruby_plugin, ruby_scripts);
+    if (!signal_data
+        || (weechat_strcasecmp ((char *)signal_data, RUBY_PLUGIN_NAME) == 0))
+    {
+        script_print_log (weechat_ruby_plugin, ruby_scripts);
+    }
     
     return WEECHAT_RC_OK;
 }
@@ -927,7 +988,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
             "    rescue => e\n"
             "      return 1\n"
             "    end\n"
-            "\n"	   
+            "\n"
             "    begin\n"
             "      module_eval(lines)\n"
             "    rescue Exception => e\n"
@@ -941,7 +1002,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
             "      if meth.to_s == 'weechat_init'\n"
             "        has_init = true\n"
             "      end\n"
-            "      module_eval('module_function :' + meth)\n"
+            "      module_eval('module_function :' + meth.to_s)\n"
             "    end\n"
             "\n"
             "    unless has_init\n"
@@ -991,7 +1052,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
                         weechat_prefix ("error"), RUBY_PLUGIN_NAME);
         VALUE err = rb_gv_get("$!");
         weechat_ruby_print_exception(err);
-	return WEECHAT_RC_ERROR;
+        return WEECHAT_RC_ERROR;
     }
     
     ruby_quiet = 1;
@@ -1024,11 +1085,10 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
-    /* make C compiler happy */
-    (void) plugin;
-    
     /* unload all scripts */
-    weechat_ruby_unload_all ();
+    ruby_quiet = 1;
+    script_end (plugin, &ruby_scripts, &weechat_ruby_unload_all);
+    ruby_quiet = 0;
     
     ruby_cleanup (0);
     
