@@ -170,8 +170,10 @@ extern struct t_hook *script_api_hook_connect (struct t_weechat_plugin *weechat_
                                                int port,
                                                int sock,
                                                int ipv6,
-                                               void *gnutls_sess, void *gnutls_cb,
+                                               void *gnutls_sess,
+                                               void *gnutls_cb,
                                                int gnutls_dhkey_size,
+                                               const char *gnutls_priorities,
                                                const char *local_hostname,
                                                int (*callback)(void *data,
                                                                int status,
@@ -317,6 +319,10 @@ extern int script_api_config_is_set_plugin (struct t_weechat_plugin *weechat_plu
 extern int script_api_config_set_plugin (struct t_weechat_plugin *weechat_plugin,
                                          struct t_plugin_script *script,
                                          const char *option, const char *value);
+extern void script_api_config_set_desc_plugin (struct t_weechat_plugin *weechat_plugin,
+                                               struct t_plugin_script *script,
+                                               const char *option,
+                                               const char *description);
 extern int script_api_config_unset_plugin (struct t_weechat_plugin *weechat_plugin,
                                            struct t_plugin_script *script,
                                            const char *option);
