@@ -45,7 +45,7 @@ struct timeval;
  */
 
 /* API version (used to check that plugin has same API and can be loaded) */
-#define WEECHAT_PLUGIN_API_VERSION "20110102-01"
+#define WEECHAT_PLUGIN_API_VERSION "20110428-01"
 
 /* macros for defining plugin infos */
 #define WEECHAT_PLUGIN_NAME(__name)                                     \
@@ -127,6 +127,10 @@ struct timeval;
 #define WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR 7
 #define WEECHAT_HOOK_CONNECT_MEMORY_ERROR           8
 
+/* action for gnutls callback: verify or set certificate */
+#define WEECHAT_HOOK_CONNECT_GNUTLS_CB_VERIFY_CERT  0
+#define WEECHAT_HOOK_CONNECT_GNUTLS_CB_SET_CERT     1
+
 /* type of data for signal hooked */
 #define WEECHAT_HOOK_SIGNAL_STRING                  "string"
 #define WEECHAT_HOOK_SIGNAL_INT                     "int"
@@ -403,6 +407,9 @@ struct t_weechat_plugin
                                  const char *option_name);
     int (*config_set_plugin) (struct t_weechat_plugin *plugin,
                               const char *option_name, const char *value);
+    void (*config_set_desc_plugin) (struct t_weechat_plugin *plugin,
+                                    const char *option_name,
+                                    const char *description);
     int (*config_unset_plugin) (struct t_weechat_plugin *plugin,
                                 const char *option_name);
     
@@ -464,6 +471,7 @@ struct t_weechat_plugin
                                     int ipv6,
                                     void *gnutls_sess, void *gnutls_cb,
                                     int gnutls_dhkey_size,
+                                    const char *gnutls_priorities,
                                     const char *local_hostname,
                                     int (*callback)(void *data,
                                                     int status,
@@ -593,8 +601,10 @@ struct t_weechat_plugin
                                 const char *property, void *pointer);
     char *(*buffer_string_replace_local_var) (struct t_gui_buffer *buffer,
                                               const char *string);
+    int (*buffer_match_list) (struct t_gui_buffer *buffer, const char *string);
     
     /* windows */
+    struct t_gui_window *(*window_search_with_buffer) (struct t_gui_buffer *buffer);
     int (*window_get_integer) (struct t_gui_window *window,
                                const char *property);
     const char *(*window_get_string) (struct t_gui_window *window,
@@ -1095,6 +1105,9 @@ extern int weechat_plugin_end (struct t_weechat_plugin *plugin);
 #define weechat_config_set_plugin(__option, __value)                    \
     weechat_plugin->config_set_plugin(weechat_plugin, __option,         \
                                       __value)
+#define weechat_config_set_desc_plugin(__option, __description)         \
+    weechat_plugin->config_set_desc_plugin(weechat_plugin, __option,    \
+                                      __description)
 #define weechat_config_unset_plugin(__option)                           \
     weechat_plugin->config_unset_plugin(weechat_plugin, __option)
 
@@ -1147,12 +1160,13 @@ extern int weechat_plugin_end (struct t_weechat_plugin *plugin);
                                  __callback, __callback_data)
 #define weechat_hook_connect(__proxy, __address, __port, __sock,        \
                             __ipv6, __gnutls_sess, __gnutls_cb,         \
-                             __gnutls_dhkey_size, __local_hostname,     \
-                             __callback, __data)                        \
+                             __gnutls_dhkey_size, __gnutls_priorities,  \
+                             __local_hostname,  __callback, __data)     \
     weechat_plugin->hook_connect(weechat_plugin, __proxy, __address,    \
                                  __port, __sock, __ipv6, __gnutls_sess, \
                                  __gnutls_cb, __gnutls_dhkey_size,      \
-                                 __local_hostname, __callback, __data)
+                                 __gnutls_priorities, __local_hostname, \
+                                 __callback, __data)
 #define weechat_hook_print(__buffer, __tags, __msg, __strip__colors,    \
                            __callback, __data)                          \
     weechat_plugin->hook_print(weechat_plugin, __buffer, __tags,        \
@@ -1251,8 +1265,12 @@ extern int weechat_plugin_end (struct t_weechat_plugin *plugin);
     weechat_plugin->buffer_set_pointer(__buffer, __property, __pointer)
 #define weechat_buffer_string_replace_local_var(__buffer, __string)     \
     weechat_plugin->buffer_string_replace_local_var(__buffer, __string)
+#define weechat_buffer_match_list(__buffer, __string)                   \
+    weechat_plugin->buffer_match_list(__buffer, __string)
 
 /* windows */
+#define weechat_window_search_with_buffer(__buffer)                     \
+    weechat_plugin->window_search_with_buffer(__buffer)
 #define weechat_window_get_integer(__window, __property)                \
     weechat_plugin->window_get_integer(__window, __property)
 #define weechat_window_get_string(__window, __property)                 \
