@@ -1,23 +1,25 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* xfer-buffer.c: display xfer list on xfer buffer */
-
+/*
+ * xfer-buffer.c: display xfer list on xfer buffer
+ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -26,6 +28,7 @@
 
 #include "../weechat-plugin.h"
 #include "xfer.h"
+#include "xfer-buffer.h"
 #include "xfer-config.h"
 #include "xfer-network.h"
 
@@ -45,7 +48,7 @@ xfer_buffer_refresh (const char *hotlist)
     char str_color[256], suffix[32], status[64], date[128], eta[128];
     char *progress_bar, *str_pos, *str_total, *str_bytes_per_sec;
     int i, length, line, progress_bar_size, num_bars;
-    unsigned long pct_complete;
+    unsigned long long pct_complete;
     struct tm *date_tmp;
     
     if (xfer_buffer)
@@ -174,7 +177,7 @@ xfer_buffer_refresh (const char *hotlist)
                         pct_complete = 0;
                 }
                 else
-                    pct_complete = (unsigned long)(((float)(ptr_xfer->pos)/(float)(ptr_xfer->size)) * 100);
+                    pct_complete = (unsigned long long)(((float)(ptr_xfer->pos)/(float)(ptr_xfer->size)) * 100);
                 
                 /* position, total and bytes per second */
                 str_pos = weechat_string_format_size (ptr_xfer->pos);
@@ -186,7 +189,7 @@ xfer_buffer_refresh (const char *hotlist)
                 if (ptr_xfer->status == XFER_STATUS_ACTIVE)
                 {
                     snprintf (eta, sizeof (eta),
-                              "%s: %.2lu:%.2lu:%.2lu - ",
+                              "%s: %.2llu:%.2llu:%.2llu - ",
                               _("ETA"),
                               ptr_xfer->eta / 3600,
                               (ptr_xfer->eta / 60) % 60,
@@ -195,7 +198,7 @@ xfer_buffer_refresh (const char *hotlist)
                 
                 /* display second line for file with status, progress bar and estimated time */
                 weechat_printf_y (xfer_buffer, (line * 2) + 3,
-                                  "%s%s%s %s%s%s%s%3lu%%   %s / %s  (%s%s/s)",
+                                  "%s%s%s %s%s%s%s%3llu%%   %s / %s  (%s%s/s)",
                                   weechat_color(str_color),
                                   (line == xfer_buffer_selected_line) ? "*** " : "    ",
                                   (XFER_IS_SEND(ptr_xfer->type)) ? "<<--" : "-->>",
@@ -311,7 +314,7 @@ xfer_buffer_open ()
 {
     if (!xfer_buffer)
     {
-        xfer_buffer = weechat_buffer_new ("xfer.list",
+        xfer_buffer = weechat_buffer_new (XFER_BUFFER_NAME,
                                           &xfer_buffer_input_cb, NULL,
                                           &xfer_buffer_close_cb, NULL);
         
