@@ -1,23 +1,25 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* xfer-config.c: xfer configuration options */
-
+/*
+ * xfer-config.c: xfer configuration options (file xfer.conf)
+ */
 
 #include <stdlib.h>
 #include <limits.h>
@@ -261,13 +263,15 @@ xfer_config_init ()
     xfer_config_file_download_path = weechat_config_new_option (
         xfer_config_file, ptr_section,
         "download_path", "string",
-        N_("path for writing incoming files"),
+        N_("path for writing incoming files (\"%h\" will be replaced by "
+           "WeeChat home, \"~/.weechat\" by default)"),
         NULL, 0, 0, "%h/xfer", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     xfer_config_file_upload_path = weechat_config_new_option (
         xfer_config_file, ptr_section,
         "upload_path", "string",
         N_("path for reading files when sending (when no path is "
-           "specified by user)"),
+           "specified by user) (\"%h\" will be replaced by "
+           "WeeChat home, \"~/.weechat\" by default)"),
         NULL, 0, 0, "~", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     xfer_config_file_use_nick_in_filename = weechat_config_new_option (
         xfer_config_file, ptr_section,
