@@ -1,23 +1,25 @@
 /*
- * Copyright (c) 2003-2010 by FlashCode <flashcode@flashtux.org>
- * See README for License detail, AUTHORS for developers list.
+ * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of WeeChat, the extensible chat client.
+ *
+ * WeeChat is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * WeeChat is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* xfer-info.c: info and infolist hooks for xfer plugin */
-
+/*
+ * xfer-info.c: info and infolist hooks for xfer plugin
+ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -91,5 +93,7 @@ xfer_info_init ()
 {
     /* xfer infolist hooks */
     weechat_hook_infolist ("xfer", N_("list of xfer"),
+                           N_("xfer pointer (optional)"),
+                           NULL,
                            &xfer_info_get_infolist_cb, NULL);
 }
